@@ -56,18 +56,19 @@ static double last_search_enter_time = 0.0;
 
 static void OnChange_file_browser_sort_mode(cvar_t *var, char *string, qbool *cancel);
 
-cvar_t  file_browser_show_size       = {"file_browser_show_size",      "1"};
-cvar_t  file_browser_show_date       = {"file_browser_show_date",      "1"};
-cvar_t  file_browser_show_time       = {"file_browser_show_time",      "0"};
-cvar_t  file_browser_sort_mode       = {"file_browser_sort_mode",      "1",	CVAR_NONE, OnChange_file_browser_sort_mode};
-cvar_t  file_browser_show_status     = {"file_browser_show_status",    "1"};
-cvar_t  file_browser_strip_names     = {"file_browser_strip_names",    "1"};
-cvar_t  file_browser_interline       = {"file_browser_interline",      "0"};
-cvar_t  file_browser_scrollnames     = {"file_browser_scrollnames" ,   "1"};
-cvar_t	file_browser_selected_color  = {"file_browser_selected_color", "0 150 235 255", CVAR_COLOR};
-cvar_t  file_browser_file_color      = {"file_browser_file_color",     "255 255 255 255", CVAR_COLOR};
-cvar_t  file_browser_dir_color       = {"file_browser_dir_color",	   "170 80 0 255", CVAR_COLOR};
-cvar_t  file_browser_archive_color   = {"file_browser_archive_color",  "255 170 0 255", CVAR_COLOR};
+static cvar_t  file_browser_show_size       = {"file_browser_show_size",      "1"};
+static cvar_t  file_browser_show_date       = {"file_browser_show_date",      "1"};
+static cvar_t  file_browser_show_time       = {"file_browser_show_time",      "0"};
+static cvar_t  file_browser_sort_mode       = {"file_browser_sort_mode",      "1",	CVAR_NONE, OnChange_file_browser_sort_mode};
+static cvar_t  file_browser_show_status     = {"file_browser_show_status",    "1"};
+static cvar_t  file_browser_strip_names     = {"file_browser_strip_names",    "1"};
+static cvar_t  file_browser_interline       = {"file_browser_interline",      "0"};
+static cvar_t  file_browser_scrollnames     = {"file_browser_scrollnames" ,   "1"};
+static cvar_t  file_browser_selected_color  = {"file_browser_selected_color", "0 150 235 255", CVAR_COLOR};
+static cvar_t  file_browser_file_color      = {"file_browser_file_color",     "255 255 255 255", CVAR_COLOR};
+static cvar_t  file_browser_dir_color       = {"file_browser_dir_color",      "170 80 0 255", CVAR_COLOR};
+static cvar_t  file_browser_archive_color   = {"file_browser_archive_color",  "255 170 0 255", CVAR_COLOR};
+static cvar_t  file_browser_sort_archives   = {"file_browser_sort_archives",  "0",	CVAR_NONE, OnChange_file_browser_sort_mode };
 
 void EX_FileList_Init(void)
 {
@@ -84,6 +85,7 @@ void EX_FileList_Init(void)
 	Cvar_Register(&file_browser_selected_color);
 	Cvar_Register(&file_browser_dir_color);
 	Cvar_Register(&file_browser_archive_color);
+	Cvar_Register(&file_browser_sort_archives);
 	Cvar_ResetCurrentGroup();
 }
 
@@ -109,7 +111,7 @@ void FL_Init(filelist_t	*fl, char *initdir)
 	fl->show_dirup = true;
 	fl->show_dirs = true;
 
-	fl->scrollbar = ScrollBar_Create(NULL);
+	fl->scrollbar = ScrollBar_Create(NULL, fl->current_dir);
 
 #ifdef WITH_ZIP
 	fl->current_archive[0] = 0;
@@ -117,6 +119,11 @@ void FL_Init(filelist_t	*fl, char *initdir)
 #endif // WITH_ZIP
 }
 
+void FL_Shutdown(filelist_t* fl)
+{
+	Q_free(fl->scrollbar);
+}
+
 //
 // Set directory
 //
@@ -124,13 +131,34 @@ void FL_SetCurrentDir(filelist_t *fl, const char *dir)
 {
 	char buf[MAX_PATH+1];
 
-	if (Sys_fullpath(buf, dir, MAX_PATH+1) == NULL)
+	if (Sys_fullpath(buf, dir, MAX_PATH + 1) == NULL) {
 		return;
+	}
 
-	if (strlen(buf) > MAX_PATH)    // Should never fail in this
+	if (strlen(buf) > MAX_PATH) {
+		// Should never fail in this
 		return;
+	}
 
-	strlcpy (fl->current_dir, buf, sizeof(fl->current_dir));
+	// Try changing directory, block if this fails
+	{
+		char olddir[MAX_PATH + 1];
+
+		// Save the current directory. (we want to restore this later)
+		if (Sys_getcwd(olddir, MAX_PATH + 1) == NULL) {
+			return;
+		}
+
+		// Change to the new dir.
+		if (!Sys_chdir(buf)) {
+			return;
+		}
+
+		// restore
+		Sys_chdir(olddir);
+	}
+
+	strlcpy(fl->current_dir, buf, sizeof(fl->current_dir));
 	fl->need_refresh = true;
 }
 
@@ -396,7 +424,7 @@ int FL_CompareFunc(const void * p_d1, const void * p_d2)
 		return -1;
 	if (d2->is_archive  && !d1->is_archive)
 		return 1;
-	if (d1->is_archive && d2->is_archive)
+	if (!file_browser_sort_archives.integer && d1->is_archive && d2->is_archive)
 		return strcasecmp(d1->name, d2->name);
 #endif // WITH_ZIP
 
@@ -527,7 +555,6 @@ static void FL_FindHighlightEntry (filelist_t *fl)
 //
 // Read the ZIP file.
 //
-#ifndef WITH_VFS_ARCHIVE_LOADING
 void FL_ReadArchive (filelist_t *fl)
 {
 	int temp = 0;
@@ -661,157 +688,6 @@ finish:
 	// Find which item to highlight in the list.
 	FL_FindHighlightEntry(fl);
 }
-#else
-
-int FL_EnumerateArchive(char *desc, int size, void *param)
-{
-	// sys_dirent ent; // create an ent some how
-	filelist_t *fl = (filelist_t *) param;
-	filedesc_t *f;
-
-	if (fl->num_entries >= MAX_FILELIST_ENTRIES)
-	{
-		return 0;
-	}
-
-	// Pointer to the current file entry.
-	f = &fl->entries[fl->num_entries];
-	f->type_index = -1;
-
-	// Skip current/above dir and hidden files.
-	if (!strcmp(desc, ".") || !strcmp(desc, ".."))
-	{
-		return 1;
-	}
-
-	// Find registered type if it's not a directory.
-	// TODO: Do stuff with this....
-	//f->type_index = FL_FindRegisteredType (fl, &ent);
-
-#ifdef WITH_ZIP
-	if (FS_IsArchive (desc))
-	{
-		f->is_archive = true;
-	}
-#endif
-
-
-	// We're not interested in this file type since it wasn't registered.
-	// VFS-FIXME: We could do stuff with directories
-	//if (!ent.directory && f->type_index < 0)
-	//{
-	//	return 1;
-	//}
-	// Find registered type if it's not a directory.
-
-
-
-	// We found a file that we're interested in so save the info about it
-	// in the file description structure.
-	f->is_directory = 0; //ent.directory;
-
-	// Get the full path for the file.
-	snprintf (f->name, sizeof(f->name), "%s%s%s", fl->current_archive, PATH_SEPARATOR, desc);
-
-	f->size = size;
-	//memcpy(&f->time, &ent.time, sizeof(f->time));
-	memset(&f->time, 0, sizeof(f->time));
-
-	// Find friendly name.
-	FL_StripFileName(fl, f);
-
-	// Increase counter of how many files have been found.
-	fl->num_entries++;
-
-	return 1;
-}
-
-void FL_ReadArchive (filelist_t *fl) 
-{
-	searchpathfuncs_t *funcs;
-	vfsfile_t *vfs = NULL;
-	void  *archive_handle = NULL;
-	SYSTEMTIME archive_time;
-
-	// Save the file modification time of the current archive
-	archive_time = fl->entries[fl->current_entry].time;
-
-	/* Set up for error case */
-	fl->error         = true;
-	fl->need_refresh  = false;
-	fl->display_entry = 0;
-	fl->num_entries   = 0;
-	fl->current_entry = 0;
-
-	if (fl->current_archive == NULL)
-	{
-		return;
-	}
-
-	vfs = FS_OpenVFS(fl->current_archive, "rb", FS_NONE_OS);
-	if (!vfs) goto fail;
-	// Open the archive file.
-	funcs = FS_FileNameToSearchFunctions(fl->current_archive);
-	if (!funcs) goto fail;
-
-	archive_handle = funcs->OpenNew(vfs, fl->current_archive);
-	if (!archive_handle) goto fail;
-
-	fl->error = false;
-
-	// Make sure we don't have some garbage left from the previous dir.
-	// This caused a bug where some files inside of a zip would be
-	// regarded as zip files, since the files with the same index
-	// in the parent directory were zip files.
-	memset(fl->entries, 0, sizeof(fl->entries));
-
-	// Add ".." entry to allow navigating out of archive if setting says so.
-	if (fl->show_dirup)
-	{
-		// Pointer to the current file entry.
-		filedesc_t *f = &fl->entries[fl->num_entries];
-
-		// Populate the file descriptor
-		f->type_index = -1;
-		f->is_directory = true;
-		snprintf(f->name, sizeof(f->name), "..");
-		f->size = 0;
-		f->time = archive_time;
-
-		// Find friendly name.
-		FL_StripFileName(fl, f);
-
-		// Increase counter of how many files have been found.
-		fl->num_entries++;
-		if (fl->num_entries >= MAX_FILELIST_ENTRIES)
-		{
-			goto fail;
-		}
-	}
-
-
-	funcs->EnumerateFiles(archive_handle, "*", FL_EnumerateArchive, fl);
-	fl->need_resort = true;
-
-	// Re-sort the file list if needed.
-	if (fl->need_resort)
-	{
-		FL_SortDir (fl);
-	}
-
-	// Resort might have changed this.
-	fl->current_entry = 0;
-
-	// Find which item to highlight in the list.
-	FL_FindHighlightEntry(fl);
-
-fail:
-	if (archive_handle)
-		funcs->ClosePath(archive_handle); // Closes vfs as well
-	else if (vfs)
-		VFS_CLOSE(vfs);
-}
-#endif // WITH_VFS_ARCHIVE_LOADING
 #endif // WITH_ZIP
 
 //
@@ -1894,7 +1770,7 @@ void FL_Draw(filelist_t *fl, int x, int y, int w, int h)
 		// Draw a cursor character at the start of the line.
 		if (filenum == fl->current_entry)
 		{
-			line[0] = 141;
+			line[0] = (char)141;
 			UI_DrawGrayBox(x, y + rowh * (i + 2) + inter_dn, w, rowh);
 		}
 
@@ -1968,8 +1844,7 @@ static void OnChange_file_browser_sort_mode(cvar_t *var, char *string, qbool *ca
 	extern filelist_t configs_filelist;
 	extern filelist_t skins_filelist;
 
-	if(host_everything_loaded)
-	{
+	if (host_everything_loaded) {
 		demo_filelist.need_resort		= true;
 		help_index_fl.need_resort		= true;
 		help_tutorials_fl.need_resort	= true;
