@@ -22,12 +22,16 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include "quakedef.h"
 #include "gl_model.h"
-#include "gl_local.h"
 #include "vx_stuff.h"
 #include "rulesets.h"
 #include "pmove.h"
 #include "utils.h"
 #include "qsound.h"
+#include "qmb_particles.h"
+
+extern cvar_t gl_no24bit;
+
+temp_entity_list_t temp_entities;
 
 #define	MAX_BEAMS 32
 typedef struct 
@@ -52,7 +56,6 @@ typedef struct explosion_s
 	model_t *model;
 } explosion_t;
 
-temp_entity_list_t temp_entities;
 
 // cl_free_explosions = linear linked list of free explosions
 explosion_t	cl_explosions[MAX_EXPLOSIONS];
@@ -139,10 +142,10 @@ void CL_FreeExplosion(explosion_t *ex)
 	cl_free_explosions = ex;
 }
 
-static void CL_ParseBeam(int type) 
+static void CL_ParseBeam(int type, vec3_t end)
 {
 	int ent, i;
-	vec3_t start, end;
+	vec3_t start;
 	beam_t *b;
 	struct model_s *m;
 
@@ -159,6 +162,10 @@ static void CL_ParseBeam(int type)
 	if (CL_Demo_SkipMessage(true))
 		return;
 
+	if (cl.intermission && ent >= 1 && ent <= MAX_CLIENTS) {
+		return;
+	}
+
     // an experimental protocol extension:
     // TE_LIGHTNING1 with entity num in -512..-1 range is a rail trail
     if (type == 1 && (ent >= -512 && ent <= -1)) 
@@ -168,53 +175,49 @@ static void CL_ParseBeam(int type)
                 180 /* green */, 35 /* light blue */, 224 /* red */,
                 133 /* magenta... kinda */, 192 /* yellow */, 6 /* white */};
         int color;
-        int pnum, cnum;
+        int cnum;
 
         // -512..-257 are colored trails assigned to a specific
         // player, so overrides can be applied; encoded as follows:
         // 7654321076543210
         // 1111111nnnnnnccc  (n = player num, c = color code)
         cnum = ent & 7;
-        pnum = (ent >> 3) & 63;
-        if (pnum < MAX_CLIENTS)
-        {
-			// TODO: apply team/enemy overrides
-        }
+        //pnum = (ent >> 3) & 63;
         color = colors[cnum];
 
 		//color is ignored by most of trails
 		switch(r_instagibtrail.integer)
 		{
 			case 1:
-			R_ParticleTrail(start, end, &start, GRENADE_TRAIL);
+			R_ParticleTrail(start, end, GRENADE_TRAIL);
 			break;
 
 			case 2:
-			R_ParticleTrail(start, end, &start, ROCKET_TRAIL);
+			R_ParticleTrail(start, end, ROCKET_TRAIL);
 			break;
 
 			case 3:
-			R_ParticleTrail(start, end, &start, ALT_ROCKET_TRAIL);
+			R_ParticleTrail(start, end, ALT_ROCKET_TRAIL);
 			break;
 
 			case 4:
-			R_ParticleTrail(start, end, &start, BLOOD_TRAIL);
+			R_ParticleTrail(start, end, BLOOD_TRAIL);
 			break;
 
 			case 5:
-			R_ParticleTrail(start, end, &start, BIG_BLOOD_TRAIL);
+			R_ParticleTrail(start, end, BIG_BLOOD_TRAIL);
 			break;
 
 			case 6:
-			R_ParticleTrail(start, end, &start, TRACER1_TRAIL);
+			R_ParticleTrail(start, end, TRACER1_TRAIL);
 			break;
 
 			case 7:
-			R_ParticleTrail(start, end, &start, TRACER2_TRAIL);
+			R_ParticleTrail(start, end, TRACER2_TRAIL);
 			break;
 
 			case 8:
-			R_ParticleTrail(start, end, &start, VOOR_TRAIL);
+			R_ParticleTrail(start, end, VOOR_TRAIL);
 			break;
 
 			case 9:
@@ -222,7 +225,7 @@ static void CL_ParseBeam(int type)
 			break;
 
 			case 10:
-			R_ParticleTrail(start, end, &start, RAIL_TRAIL);
+			R_ParticleTrail(start, end, RAIL_TRAIL);
 			break;
 
 			case 11:
@@ -230,11 +233,11 @@ static void CL_ParseBeam(int type)
 			break;
 
 			case 12:
-			R_ParticleTrail(start, end, &start, LAVA_TRAIL);
+			R_ParticleTrail(start, end, LAVA_TRAIL);
 			break;
 
 			case 13:
-			R_ParticleTrail(start, end, &start, AMF_ROCKET_TRAIL);
+			R_ParticleTrail(start, end, AMF_ROCKET_TRAIL);
 			break;
 
 			default: break;
@@ -247,22 +250,22 @@ static void CL_ParseBeam(int type)
 	{
 		case 1:
 			if (!cl_bolt1_mod)
-				cl_bolt1_mod = Mod_ForName("progs/bolt.mdl", true);
+				cl_bolt1_mod = Mod_CustomModel(custom_model_bolt, true);
 			m = cl_bolt1_mod;
 			break;
 		case 2:
 			if (!cl_bolt2_mod)
-				cl_bolt2_mod = Mod_ForName("progs/bolt2.mdl", true);
+				cl_bolt2_mod = Mod_CustomModel(custom_model_bolt2, true);
 			m = cl_bolt2_mod;
 			break;
 		case 3:
 			if (!cl_bolt3_mod)
-				cl_bolt3_mod = Mod_ForName("progs/bolt3.mdl", true);
+				cl_bolt3_mod = Mod_CustomModel(custom_model_bolt3, true);
 			m = cl_bolt3_mod;
 			break;
 		case 4: default:
 			if (!cl_beam_mod)
-				cl_beam_mod = Mod_ForName("progs/beam.mdl", true);
+				cl_beam_mod = Mod_CustomModel(custom_model_beam, true);
 			m = cl_beam_mod;
 			break;
 	}
@@ -309,7 +312,7 @@ void CL_ExplosionSprite(vec3_t pos)
 	VectorCopy(pos, ex->origin);
 	ex->start = cl.time;
 	if (!cl_explo_mod)
-		cl_explo_mod = Mod_ForName("progs/s_explod.spr", true);
+		cl_explo_mod = Mod_CustomModel(custom_model_explosion, true);
 	ex->model = cl_explo_mod;
 }
 
@@ -402,90 +405,87 @@ static void CL_Parse_TE_SUPERSPIKE(vec3_t pos)
 static void CL_Parse_TE_EXPLOSION(vec3_t pos)
 {
 	dlight_t *dl;
+	extern cvar_t gl_part_explosions;
 
-	if (cls.state != ca_active)
+	if (cls.state != ca_active) {
 		return;
+	}
 
-	if (r_explosiontype.value == 2) 
-	{
-		if (amf_part_teleport.value)
+	if (r_explosiontype.value == 2) {
+		if (amf_part_teleport.value) {
 			VXTeleport(pos);
-		else
-		{
-			R_TeleportSplash(pos); // Teleport splash
 		}
-		
-		if (amf_coronas_tele.value)
-			NewCorona(C_BLUEFLASH, pos);
+		else {
+			// Teleport splash
+			R_TeleportSplash(pos);
+		}
+
+		if (amf_coronas_tele.integer) {
+			R_CoronasNew(C_BLUEFLASH, pos);
+		}
 	}
-	else if (r_explosiontype.value == 3) 
-	{	
-		R_RunParticleEffect(pos, vec3_origin, 225, 50);		//lightning blood
+	else if (r_explosiontype.value == 3) {
+		// lightning blood
+		R_RunParticleEffect(pos, vec3_origin, 225, 50);
 	}
-	else if (r_explosiontype.value == 4) 
-	{	
-		R_RunParticleEffect(pos, vec3_origin, 73, 20 * 32); // Big blood
+	else if (r_explosiontype.value == 4) {
+		// Big blood
+		R_RunParticleEffect(pos, vec3_origin, 73, 20 * 32);
 	}
-	else if (r_explosiontype.value == 5) 
-	{
-		R_RunParticleEffect(pos, vec3_origin, 0, 20 * 14);	// Double gunshot
+	else if (r_explosiontype.value == 5) {
+		// Double gunshot
+		R_RunParticleEffect(pos, vec3_origin, 0, 20 * 14);
 	}
-	else if (r_explosiontype.value == 6) 
-	{
-		if (amf_part_blobexplosion.value)
-		{
+	else if (r_explosiontype.value == 6) {
+		if (amf_part_blobexplosion.value) {
 			VXBlobExplosion(pos);
 		}
-		else
-		{
+		else {
 			R_BlobExplosion(pos); // Blob explosion
 		}
-		if (amf_coronas.value)
-			NewCorona(C_BLUEFLASH, pos);
+		if (amf_coronas.integer) {
+			R_CoronasNew(C_BLUEFLASH, pos);
+		}
 	}
-	else if (r_explosiontype.value == 7 && qmb_initialized && gl_part_explosions.value) 
-	{
+	else if (r_explosiontype.value == 7 && qmb_initialized && gl_part_explosions.value) {
 		QMB_DetpackExplosion(pos);	// Detpack explosion
 	}
-	else if (r_explosiontype.value == 8 && qmb_initialized)
-	{
+	else if (r_explosiontype.value == 8 && qmb_initialized) {
 		FuelRodExplosion(pos);
 	}
-	else if (r_explosiontype.value == 10)
-	{ /* Explosions turned off */ }
-	else
-	{	
+	else if (r_explosiontype.value == 10) {
+		/* Explosions turned off */
+	}
+	else {
 		// sprite and particles
 		if (amf_part_explosion.value)
 			VXExplosion(pos);
-		else
-		{
+		else {
 			R_ParticleExplosion(pos); // Normal explosion
 		}
 	}
 
-	if (r_explosionlight.value)	
-	{
+	if (r_explosionlight.value) {
 		dl = CL_AllocDlight(0);
 		VectorCopy(pos, dl->origin);
 		dl->radius = 150 + 200 * bound(0, r_explosionlight.value, 1);
 		dl->die = cl.time + 0.5;
 		dl->decay = 300;
-		if (r_explosiontype.value == 8)
-		{
+		if (r_explosiontype.value == 8) {
 			dl->type = lt_green;
 		}
-		else
-		{
-			customlight_t cst_lt = {0};
+		else {
+			customlight_t cst_lt = { 0 };
 			dlightColorEx(r_explosionlightcolor.value, r_explosionlightcolor.string, lt_explosion, true, &cst_lt);
 			dl->type = cst_lt.type;
-			
-			if (dl->type == lt_custom)
+
+			if (dl->type == lt_custom) {
 				VectorCopy(cst_lt.color, dl->color);
+			}
+		}
+		if (amf_coronas.integer && r_explosiontype.integer != 7 && r_explosiontype.integer != 2 && r_explosiontype.integer != 8) {
+			R_CoronasNew(C_FLASH, pos);
 		}
-		if (amf_coronas.value && r_explosiontype.value != 7 && r_explosiontype.value != 2 && r_explosiontype.value != 8)
-			NewCorona(C_FLASH, pos);
 	}
 
 	S_StartSound(-1, 0, cl_sfx_r_exp3, pos, 1, 1);
@@ -493,17 +493,16 @@ static void CL_Parse_TE_EXPLOSION(vec3_t pos)
 
 static void CL_Parse_TE_TAREXPLOSION(vec3_t pos)
 {
-	if (amf_part_blobexplosion.value)
-	{
+	if (amf_part_blobexplosion.value) {
 		VXBlobExplosion(pos);
 	}
-	else
-	{
+	else {
 		R_BlobExplosion(pos); // Blob explosion
 	}
 	
-	if (amf_coronas.value)
-		NewCorona (C_BLUEFLASH, pos);
+	if (amf_coronas.integer) {
+		R_CoronasNew(C_BLUEFLASH, pos);
+	}
 
 	S_StartSound(-1, 0, cl_sfx_r_exp3, pos, 1, 1);
 }
@@ -512,50 +511,51 @@ static void CL_Parse_TE_TELEPORT(vec3_t pos)
 {
 	if (r_telesplash.value)
 	{
-		if (amf_part_teleport.value)
+		if (amf_part_teleport.value) {
 			VXTeleport(pos);
-		else
-		{
+		}
+		else {
 			R_TeleportSplash(pos); // Teleport splash
 		}
-		if (amf_coronas_tele.value)
-			NewCorona(C_BLUEFLASH, pos);
+		if (amf_coronas_tele.integer) {
+			R_CoronasNew(C_BLUEFLASH, pos);
+		}
 	}
 }
 
-static void CL_Parse_TE_GUNSHOT(void)
+static void CL_Parse_TE_GUNSHOT(vec3_t pos)
 {
 	int count;
-	vec3_t pos;
 
-	if (cls.nqdemoplayback)
+	if (cls.nqdemoplayback) {
 		count = 1;
-	else
+	}
+	else {
 		count = MSG_ReadByte();
+	}
 
 	pos[0] = MSG_ReadCoord();
 	pos[1] = MSG_ReadCoord();
 	pos[2] = MSG_ReadCoord();
 
-	if (CL_Demo_SkipMessage(true))
+	if (CL_Demo_SkipMessage(true)) {
 		return;
+	}
 	
-	if (amf_part_gunshot.value && !Rulesets_RestrictParticles())
+	if (amf_part_gunshot.value && !Rulesets_RestrictParticles()) {
 		VXGunshot(pos, 5 * count * amf_part_gunshot.value);
-	else
-	{
+	}
+	else {
 		R_RunParticleEffect(pos, vec3_origin, 256 /* magic! */, 20 * count);
 	}
 }
 
-static void CL_Parse_TE_BLOOD(void)
+static void CL_Parse_TE_BLOOD(vec3_t pos)
 {
 	int count;
-	vec3_t pos;
 	dlight_t *dl;
 	
-	if (cls.nqdemoplayback)
-	{
+	if (cls.nqdemoplayback) {
 		// NQ_TE_EXPLOSION2
 		pos[0] = MSG_ReadCoord();
 		pos[1] = MSG_ReadCoord();
@@ -563,8 +563,9 @@ static void CL_Parse_TE_BLOOD(void)
 		MSG_ReadByte(); // colorStart
 		MSG_ReadByte(); // colorLength
 
-		if (CL_Demo_SkipMessage(true))
+		if (CL_Demo_SkipMessage(true)) {
 			return;
+		}
 
 		dl = CL_AllocDlight(0);
 		VectorCopy(pos, dl->origin);
@@ -580,34 +581,35 @@ static void CL_Parse_TE_BLOOD(void)
 	pos[1] = MSG_ReadCoord();
 	pos[2] = MSG_ReadCoord();
 
-	if (CL_Demo_SkipMessage(true))
+	if (CL_Demo_SkipMessage(true)) {
 		return;
+	}
 
-	if (amf_part_blood.value)
+	if (amf_part_blood.value) {
 		VXBlood(pos, 5 * count * amf_part_blood.value);
-	else
-	{
+	}
+	else {
 		R_RunParticleEffect(pos, vec3_origin, 73, 20 * count);
 	}
 }
 
-static void CL_Parse_TE_LIGHTNINGBLOOD(void)
+static void CL_Parse_TE_LIGHTNINGBLOOD(vec3_t pos)
 {
-	vec3_t pos;
-
-	if (cls.nqdemoplayback) 
-	{
+	if (cls.nqdemoplayback) {
 		// NQ_TE_BEAM - grappling hook beam
-		CL_ParseBeam(4);
+		CL_ParseBeam(4, pos);
 	}
-	else
-	{
+	else {
+		extern cvar_t gl_part_blood;
+
 		pos[0] = MSG_ReadCoord();
 		pos[1] = MSG_ReadCoord();
 		pos[2] = MSG_ReadCoord();
 
-		if (CL_Demo_SkipMessage(true))
+		if (CL_Demo_SkipMessage(true)) {
 			return;
+		}
+
 		R_RunParticleEffect(pos, vec3_origin, gl_part_blood.value?225:r_lgblood.value, 50); // 225 default
 	}
 }
@@ -627,35 +629,35 @@ void CL_ParseTEnt (void)
 	{
 		// Lightning bolts.
 		case TE_LIGHTNING1:			
-			CL_ParseBeam(1);
+			CL_ParseBeam(1, pos);
 			parsed = true;
 			break;
 
 		case TE_LIGHTNING2:
-			CL_ParseBeam(2);
+			CL_ParseBeam(2, pos);
 			parsed = true;
 			break;
 
 		case TE_LIGHTNING3:
-			CL_ParseBeam(3);
+			CL_ParseBeam(3, pos);
 			parsed = true;
 			break;
 
 		// Bullet hitting wall.
 		case TE_GUNSHOT:
-			CL_Parse_TE_GUNSHOT();
+			CL_Parse_TE_GUNSHOT(pos);
 			parsed = true;
 			break;
 
 		// Bullets hitting body.
 		case TE_BLOOD: 
-			CL_Parse_TE_BLOOD();
+			CL_Parse_TE_BLOOD(pos);
 			parsed = true;
 			break;
 
 		// Lightning hitting body
 		case TE_LIGHTNINGBLOOD: 
-			CL_Parse_TE_LIGHTNINGBLOOD();
+			CL_Parse_TE_LIGHTNINGBLOOD(pos);
 			parsed = true;
 			break;
 	}
@@ -667,8 +669,9 @@ void CL_ParseTEnt (void)
 		pos[1] = MSG_ReadCoord();
 		pos[2] = MSG_ReadCoord();
 
-		if (CL_Demo_SkipMessage(true))
+		if (CL_Demo_SkipMessage(true)) {
 			return;
+		}
 		
 		switch (type) 
 		{
@@ -714,6 +717,9 @@ void CL_ParseTEnt (void)
 				Host_Error("CL_ParseTEnt: unknown type %d", type);
 		}
 	}
+	else if (CL_Demo_SkipMessage(true)) {
+		return;
+	}
 
 	// Save the temp entities.
 	VectorCopy(pos, temp_entities.list[temp_entities.count].pos);
@@ -736,7 +742,7 @@ static float fakeshaft_policy (void)
 	}
 }
 
-void CL_UpdateBeams(void) 
+static void CL_UpdateBeams(void)
 {
 	int i;
 	beam_t *b;	
@@ -749,28 +755,32 @@ void CL_UpdateBeams(void)
 	int beamstodraw, j, k;
 	qbool sparks = false;
 	vec3_t beamstart[MAX_LIGHTNINGBEAMS], beamend[MAX_LIGHTNINGBEAMS];
+	vec3_t furthest_sparks;
 	beamstodraw = bound(1, amf_lightning.value, MAX_LIGHTNINGBEAMS);	
 
 	memset (&ent, 0, sizeof(entity_t));
 	ent.colormap = vid.colormap;
 
-	fakeshaft = bound(0, cl_fakeshaft.value, fakeshaft_policy());
+	fakeshaft = cl.intermission ? 0 : bound(0, cl_fakeshaft.value, fakeshaft_policy());
 
 	// Update lightning.
-	for (i = 0, b = cl_beams; i < MAX_BEAMS; i++, b++)	
-	{
-		if (!b->model || b->endtime < cl.time)
+	for (i = 0, b = cl_beams; i < MAX_BEAMS; i++, b++) {
+		sparks = false;
+
+		if (!b->model || b->endtime < cl.time) {
 			continue;
+		}
+		if (b->entity >= 1 && b->entity <= MAX_CLIENTS && cl.intermission) {
+			continue;
+		}
 
 		// if coming from the player, update the start position
-		if (b->entity == cl.viewplayernum + 1) 
-		{
+		if (b->entity == cl.viewplayernum + 1) {
 			VectorCopy (cl.simorg, b->start);
 			b->start[2] += cl.crouch + bound(-7, v_viewheight.value, 4);
 			VectorMA(b->start, r_shiftbeam.value, vright, b->start);
 
-			if (fakeshaft && (cl_fakeshaft_extra_updates.value || playerbeam_update))
-			{
+			if (fakeshaft && (cl_fakeshaft_extra_updates.value || playerbeam_update)) {
 				vec3_t	forward, v, org, ang, target_angles, target_origin;
 				float	delta;
 				trace_t	trace;
@@ -782,7 +792,8 @@ void CL_UpdateBeams(void)
 					frame_t *frame = &cl.frames[(cls.netchan.outgoing_sequence - 2) & UPDATE_MASK];
 					VectorCopy(frame->cmd.angles, target_angles);
 					VectorCopy(frame->playerstate[cl.viewplayernum].origin, target_origin);
-				} else {
+				}
+				else {
 					VectorCopy(cl.simangles, target_angles);
 					VectorCopy(cl.simorg, target_origin);
 				}
@@ -793,16 +804,19 @@ void CL_UpdateBeams(void)
 
 				// Lerp pitch.
 				ang[0] = -ang[0];
-				if (ang[0] < -180)
+				if (ang[0] < -180) {
 					ang[0] += 360;
+				}
 				ang[0] += (target_angles[0] - ang[0]) * fakeshaft;
 
 				// Lerp yaw.
 				delta = target_angles[1] - ang[1];
-				if (delta > 180)
+				if (delta > 180) {
 					delta -= 360;
-				if (delta < -180)
+				}
+				if (delta < -180) {
 					delta += 360;
+				}
 				ang[1] += delta * fakeshaft;
 				ang[2] = 0;
 
@@ -813,44 +827,44 @@ void CL_UpdateBeams(void)
 				VectorAdd (org, forward, b->end);
 
 				trace = PM_TraceLine (org, b->end);
-				if (trace.fraction < 1)
-					VectorCopy (trace.endpos, b->end);
+				if (trace.fraction < 1) {
+					VectorCopy(trace.endpos, b->end);
+				}
 			}
 		}
 
 		// Calculate pitch and yaw.
 		VectorSubtract (b->end, b->start, dist);
 
-		if (dist[1] == 0 && dist[0] == 0) 
-		{
+		if (dist[1] == 0 && dist[0] == 0) {
 			yaw = 0;
 			pitch = (dist[2] > 0) ? 90 : 270;
 		}
-		else 
-		{
+		else {
 			yaw = atan2(dist[1], dist[0]) * 180 / M_PI;
-			if (yaw < 0)
+			if (yaw < 0) {
 				yaw += 360;
+			}
 	
 			forward = sqrt (dist[0]*dist[0] + dist[1]*dist[1]);
 			pitch = atan2(dist[2], forward) * 180 / M_PI;
-			if (pitch < 0)
+			if (pitch < 0) {
 				pitch += 360;
+			}
 		}
 
 		// Add new entities for the lightning.
 		VectorCopy (b->start, org);
 		
-		for (k = 0; k < beamstodraw; k++)
-			VectorCopy (b->start, beamstart[k]);
+		for (k = 0; k < beamstodraw; k++) {
+			VectorCopy(b->start, beamstart[k]);
+		}
 		
 		d = VectorNormalize(dist);
 		VectorScale (dist, 30, dist);
 
-		for ( ; d > 0; d -= 30) 
-		{
-			if (!amf_lightning.value || Rulesets_RestrictParticles())
-			{
+		for ( ; d > 0; d -= 30) {
+			if (!amf_lightning.value || Rulesets_RestrictParticles()) {
 				VectorCopy (org, ent.origin);
 				ent.model = b->model;
 				ent.angles[0] = pitch;
@@ -862,52 +876,51 @@ void CL_UpdateBeams(void)
 
 				CL_AddEntity (&ent);
 			}
-			else
-			{
-				if (!ISPAUSED)
-				{
-					//VULT - Some people might like their lightning beams thicker
-					for (k=0;k<beamstodraw;k++)
-					{
-						VectorAdd (org, dist, beamend[k]);
-						if (!cl.racing) {
-							for (j = 0; j < 3; j++)
-								beamend[k][j] += f_rnd(-lg_size, lg_size);
+			else if (!ISPAUSED) {
+				//VULT - Some people might like their lightning beams thicker
+				for (k = 0; k < beamstodraw; k++) {
+					VectorAdd(org, dist, beamend[k]);
+					if (!cl.racing) {
+						for (j = 0; j < 3; j++) {
+							beamend[k][j] += f_rnd(-lg_size, lg_size);
 						}
-						VX_LightningBeam (beamstart[k], beamend[k]);
-						VectorCopy (beamend[k], beamstart[k]);
 					}
+					VX_LightningBeam(beamstart[k], beamend[k]);
+					VectorCopy(beamend[k], beamstart[k]);
 				}
 			}
 
 			// The infamous d-light glow has been replaced with a simple corona so it doesn't light up the room anymore.
-			if (amf_coronas.value && amf_lightning.value && !ISPAUSED)
-			{
-				if (b->entity == cl.viewplayernum + 1 && !((cls.demoplayback || cl.spectator) && amf_camera_chase.value))
-					NewCorona (C_SMALLLIGHTNING, org);
-				else
-					NewCorona (C_LIGHTNING, org);
+			if (amf_coronas.value && amf_lightning.value && !ISPAUSED) {
+				if (b->entity == cl.viewplayernum + 1 && !((cls.demoplayback || cl.spectator) && cl_camera_tpp.integer)) {
+					R_CoronasNew(C_SMALLLIGHTNING, org);
+				}
+				else {
+					R_CoronasNew(C_LIGHTNING, org);
+				}
 			}
 
 			VectorAdd (org, dist, org);
 
 			// LIGHTNING SPARKS
-			if (amf_lightning_sparks.value && !Rulesets_RestrictParticles() && !sparks && !ISPAUSED && !gl_no24bit.integer)
-			{
+			if (amf_lightning_sparks.value && !Rulesets_RestrictParticles() && !ISPAUSED && !gl_no24bit.integer) {
 				trace_t	trace;
 				trace = PM_TraceLine (org, b->end);
-				if (trace.fraction < 1)
-				{
-					//byte col[3] = {60,100,240};
-					SparkGen (trace.endpos, amf_lightning_color.color, (int)(3*amf_lightning_sparks.value), amf_lightning_sparks_size.value, 0.25);
+				if (trace.fraction < 1) {
+					VectorCopy(trace.endpos, furthest_sparks);
 					sparks = true;
 				}
 			}
 		}
-	}	
+
+		if (sparks) {
+			SparkGen(furthest_sparks, amf_lightning_color.color, (int)(3 * amf_lightning_sparks.value), amf_lightning_sparks_size.value, 0.25);
+		}
+	}
 }
 
-void CL_UpdateExplosions (void) {
+static void CL_UpdateExplosions(void)
+{
 	int f;
 	explosion_t	*ex, *next, *hnode;
 	entity_t ent;
@@ -933,12 +946,9 @@ void CL_UpdateExplosions (void) {
 	}
 }
 
-void CL_UpdateTEnts (void) {
-	CL_UpdateBeams ();
-	CL_UpdateExplosions ();
-
-	//VULT MOTION TRAILS
-	if (amf_motiontrails.value || MotionBlurCount) //dont stop removing trails if we have 'em
-		CL_UpdateBlurs();
+void CL_UpdateTEnts (void)
+{
+	CL_UpdateBeams();
+	CL_UpdateExplosions();
 }
 
