@@ -21,16 +21,20 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "quakedef.h"
 #include "vx_stuff.h"
 #include "gl_model.h"
-#include "gl_local.h"
 #include "teamplay.h"
 #include "rulesets.h"
 #include "utils.h"
 #include "hud.h"
 #include "hud_common.h"
+#include "mvd_utils.h"
+#include "r_matrix.h"
 
 #ifdef X11_GAMMA_WORKAROUND
 #include "tr_types.h"
 #endif
+#include "r_local.h"
+#include "r_renderer.h"
+#include "r_brushmodel.h"
 
 /*
 The view is allowed to move slightly from its true position for bobbing,
@@ -39,12 +43,14 @@ entities sent from the server may not include everything in the pvs, especially
 when crossing a water boudnary.
 */
 
+static cvar_t cl_bob      = { "cl_bob",      "0"   };
+static cvar_t cl_bobcycle = { "cl_bobcycle", "0.0" };
+static cvar_t cl_bobup    = { "cl_bobup",    "0.0" };
+static cvar_t cl_bobhead  = { "cl_bobhead",  "0"   };
+
 cvar_t	cl_rollspeed = {"cl_rollspeed", "200"};
 cvar_t	cl_rollangle = {"cl_rollangle", "0"};
 cvar_t	cl_rollalpha = {"cl_rollalpha", "20"};
-cvar_t	cl_bob = {"cl_bob", "0"};
-cvar_t	cl_bobcycle = {"cl_bobcycle", "0.0"};
-cvar_t	cl_bobup = {"cl_bobup", "0.0"};
 cvar_t	v_kicktime = {"v_kicktime", "0.0"};
 cvar_t	v_kickroll = {"v_kickroll", "0.0"};
 cvar_t	v_kickpitch = {"v_kickpitch", "0.0"};
@@ -52,7 +58,7 @@ cvar_t	v_gunkick = {"v_gunkick", "0"};
 cvar_t	v_viewheight = {"v_viewheight", "0"};
 
 cvar_t	cl_drawgun = {"r_drawviewmodel", "1"};
-cvar_t  r_nearclip = {"r_nearclip", "1", CVAR_RULESET_MAX | CVAR_RULESET_MIN, NULL, 1.f, 4.f, 0.1f};
+cvar_t  r_nearclip = {"r_nearclip", "2", CVAR_RULESET_MAX | CVAR_RULESET_MIN, NULL, R_MINIMUM_NEARCLIP, R_MAXIMUM_FARCLIP, R_MINIMUM_NEARCLIP };
 cvar_t	r_viewmodelsize = {"r_viewmodelSize", "1"};
 cvar_t	r_viewmodeloffset = {"r_viewmodeloffset", ""};
 cvar_t  r_viewpreselgun = {"r_viewpreselgun", "0"};
@@ -73,18 +79,23 @@ cvar_t	crosshairsize	= {"crosshairsize", "1"};
 cvar_t  cl_crossx = {"cl_crossx", "0"};
 cvar_t  cl_crossy = {"cl_crossy", "0"};
 
+// gamma updates are expensive in hw: update at lower fps, otherwise drops are severe
+static cvar_t vid_hwgamma_fps = { "vid_hwgamma_fps", "60" };
+
 // QW262: less flash grenade effect in demos
-cvar_t	cl_demoplay_flash = {"cl_demoplay_flash", ".33"};
+cvar_t cl_demoplay_flash       = { "cl_demoplay_flash",      "0.33" };
 
-cvar_t  v_contentblend = {"v_contentblend", "0.2"};
-cvar_t	v_damagecshift = {"v_damagecshift", "0.2"};
-cvar_t	v_quadcshift = {"v_quadcshift", "0.5"};
-cvar_t	v_suitcshift = {"v_suitcshift", "0.5"};
-cvar_t	v_ringcshift = {"v_ringcshift", "0.5"};
-cvar_t	v_pentcshift = {"v_pentcshift", "0.5"};
-cvar_t	v_dlightcshift = {"v_dlightcshift", "1"};
+cvar_t v_contentblend          = { "v_contentblend",         "0.2" };
+cvar_t v_damagecshift          = { "v_damagecshift",         "0.2" };
+cvar_t v_quadcshift            = { "v_quadcshift",           "0.5" };
+cvar_t v_suitcshift            = { "v_suitcshift",           "0.5" };
+cvar_t v_ringcshift            = { "v_ringcshift",           "0.5" };
+cvar_t v_pentcshift            = { "v_pentcshift",           "0.5" };
+cvar_t v_dlightcshift          = { "v_dlightcshift",         "1.0" };
+cvar_t v_dlightcolor           = { "v_dlightcolor",          "1.0" };
+cvar_t v_dlightcshiftpercent   = { "v_dlightcshiftpercent",  "0.5" };
 
-cvar_t	v_bonusflash = {"cl_bonusflash", "0"};
+cvar_t v_bonusflash            = { "cl_bonusflash",          "0.0" };
 
 float	v_dmg_time, v_dmg_roll, v_dmg_pitch;
 
@@ -193,7 +204,7 @@ void V_DriftPitch (void) {
 
 // don't count small mouse motion
 	if (cl.nodrift) {
-		if ( fabs(cl.frames[(cls.netchan.outgoing_sequence-1)&UPDATE_MASK].cmd.forwardmove) < 200)
+		if (abs(cl.frames[(cls.netchan.outgoing_sequence-1)&UPDATE_MASK].cmd.forwardmove) < 200)
 			cl.driftmove = 0;
 		else
 			cl.driftmove += cls.frametime;
@@ -243,13 +254,8 @@ cshift_t	cshift_lava = { {255,80,0}, 150 };
 cvar_t		gl_cshiftpercent = {"gl_cshiftpercent", "100"};
 cvar_t		gl_hwblend = {"gl_hwblend", "1"};
 float		v_blend[4];		// rgba 0.0 - 1.0
-#ifdef NDEBUG
-cvar_t		v_gamma = {"gl_gamma", "0.8"};
-cvar_t		v_contrast = {"gl_contrast", "1.3"};
-#else
 cvar_t		v_gamma = {"gl_gamma", "1.0"};
 cvar_t		v_contrast = {"gl_contrast", "1.0"};
-#endif
 
 #ifdef X11_GAMMA_WORKAROUND
 unsigned short ramps[3][4096];
@@ -289,9 +295,7 @@ void V_ParseDamage (void)
 	if (cl.cshifts[CSHIFT_DAMAGE].percent > 150)
 		cl.cshifts[CSHIFT_DAMAGE].percent = 150;
 
-	fraction = v_damagecshift.value;
-	if (fraction < 0) fraction = 0;
-	if (fraction > 1) fraction = 1;
+	fraction = bound(0, v_damagecshift.value, 1);
 	cl.cshifts[CSHIFT_DAMAGE].percent *= fraction;
 
 	if (armor > blood) {
@@ -323,8 +327,7 @@ void V_ParseDamage (void)
 // disconnect -->
 qbool flashed = false; // it should be used for f_flashout tirgger
 extern cvar_t v_gamma, v_contrast;
-#define flash_gamma 0.55
-#define flash_contrast 1.0
+
 void V_TF_FlashSettings(qbool flashed)
 {
 	static float old_gamma, old_contrast;
@@ -343,8 +346,8 @@ void V_TF_FlashSettings(qbool flashed)
 		old_contrast = v_contrast.value;
 
 		// set MTFL flash settings	
-		Cvar_SetValue(&v_gamma, flash_gamma);
-		Cvar_SetValue(&v_contrast, flash_contrast);
+		Cvar_SetValue(&v_gamma, MTFL_FLASH_GAMMA);
+		Cvar_SetValue(&v_contrast, MTFL_FLASH_CONTRAST);
 
 		// made gamma&contrast read only
 		Cvar_SetFlags(&v_gamma, Cvar_GetFlags(&v_gamma) | CVAR_ROM);
@@ -366,7 +369,7 @@ void V_TF_FlashStuff (void)
 	// 240 = Normal TF || 255 = Angel TF
 	if (cshift_empty.percent == 240 || cshift_empty.percent == 255 ) {
 		TP_ExecTrigger ("f_flash");
-		if (!flashed && (!strncasecmp(Rulesets_Ruleset(), "MTFL", 4))) {
+		if (!flashed && Rulesets_ToggleWhenFlashed()) {
 			V_TF_FlashSettings(true);
 		}
 
@@ -376,7 +379,7 @@ void V_TF_FlashStuff (void)
 
 	if (cshift_empty.percent == 160) {
 		// flashed by your own flash
-		if (!flashed && (!strncasecmp(Rulesets_Ruleset(), "MTFL", 4))) {
+		if (!flashed && Rulesets_ToggleWhenFlashed()) {
 			V_TF_FlashSettings(true);
 		}
 
@@ -392,7 +395,7 @@ void V_TF_FlashStuff (void)
 
 		if (flashed_for_10seconds || death_while_flashed) {
 			// turn gamma and contrast back
-			if (flashed && (!strncasecmp(Rulesets_Ruleset(), "MTFL", 4))) {
+			if (flashed && Rulesets_ToggleWhenFlashed()) {
 				V_TF_FlashSettings(false);
 			}
 			flashed = false;
@@ -400,7 +403,7 @@ void V_TF_FlashStuff (void)
 	}
 
 	if (cls.demoplayback && cshift_empty.destcolor[0] == cshift_empty.destcolor[1]) {
-		cshift_empty.percent *= cl_demoplay_flash.value / 1.0f;
+		cshift_empty.percent *= bound(0, cl_demoplay_flash.value, 1.0f);
 	}
 }
 // <-- disconnect
@@ -439,7 +442,9 @@ void V_BonusFlash_f (void) {
 }
 
 //Underwater, lava, etc each has a color shift
-void V_SetContentsColor (int contents) {
+void V_SetContentsColor (int contents)
+{
+	extern cvar_t gl_polyblend;
 
 	if (!v_contentblend.value) {
 		cl.cshifts[CSHIFT_CONTENTS] = cshift_empty;
@@ -463,19 +468,23 @@ void V_SetContentsColor (int contents) {
 			break;
 	}
 
-	if (v_contentblend.value > 0 && v_contentblend.value < 1 && contents != CONTENTS_EMPTY)
+	if (v_contentblend.value > 0 && v_contentblend.value < 1 && contents != CONTENTS_EMPTY) {
 		cl.cshifts[CSHIFT_CONTENTS].percent *= v_contentblend.value;
+	}
 
 	if (!gl_polyblend.value && !cl.teamfortress) {
 		cl.cshifts[CSHIFT_CONTENTS].percent = 0;
-	} else {
+	}
+	else {
 		// ignore gl_cshiftpercent on custom cshifts (set with v_cshift
 		// command) to avoid cheating in TF
 		if (contents != CONTENTS_EMPTY) {
-			if (!gl_polyblend.value)
+			if (!gl_polyblend.value) {
 				cl.cshifts[CSHIFT_CONTENTS].percent = 0;
-			else
+			}
+			else {
 				cl.cshifts[CSHIFT_CONTENTS].percent *= gl_cshiftpercent.value;
+			}
 		}
 		else {
 			cl.cshifts[CSHIFT_CONTENTS].percent *= 100;
@@ -483,52 +492,8 @@ void V_SetContentsColor (int contents) {
 	}
 }
 
-void V_AddWaterfog (int contents) {
-	extern cvar_t gl_waterfog_color_water;
-	extern cvar_t gl_waterfog_color_lava;
-	extern cvar_t gl_waterfog_color_slime;
-
-	float colors[4];
-
-	if (!gl_waterfog.value || COM_CheckParm ("-nomtex") || contents == CONTENTS_EMPTY || contents == CONTENTS_SOLID) {
-		glDisable(GL_FOG);
-		return;
-	}
-
-	switch (contents) {
-		case CONTENTS_LAVA:
-			colors[0] = (float) gl_waterfog_color_lava.color[0] / 255.0;
-			colors[1] = (float) gl_waterfog_color_lava.color[1] / 255.0;
-			colors[2] = (float) gl_waterfog_color_lava.color[2] / 255.0;
-			colors[3] = (float) gl_waterfog_color_lava.color[3] / 255.0;
-			break;
-		case CONTENTS_SLIME:
-			colors[0] = (float) gl_waterfog_color_slime.color[0] / 255.0;
-			colors[1] = (float) gl_waterfog_color_slime.color[1] / 255.0;
-			colors[2] = (float) gl_waterfog_color_slime.color[2] / 255.0;
-			colors[3] = (float) gl_waterfog_color_slime.color[3] / 255.0;
-			break;
-		default:
-			colors[0] = (float) gl_waterfog_color_water.color[0] / 255.0;
-			colors[1] = (float) gl_waterfog_color_water.color[1] / 255.0;
-			colors[2] = (float) gl_waterfog_color_water.color[2] / 255.0;
-			colors[3] = (float) gl_waterfog_color_water.color[3] / 255.0;
-			break;
-	}
-	
-	glFogfv(GL_FOG_COLOR, colors);
-	if (( (int) gl_waterfog.value ) == 2) {
-		glFogf(GL_FOG_DENSITY, 0.0002 + (0.0009 - 0.0002) * bound(0, gl_waterfog_density.value, 1));
-		glFogi(GL_FOG_MODE, GL_EXP);
-	} else {
-		glFogi(GL_FOG_MODE, GL_LINEAR);
-		glFogf(GL_FOG_START, 150.0f);	
-		glFogf(GL_FOG_END, 4250.0f - (4250.0f - 1536.0f) * bound (0, gl_waterfog_density.value, 1));	
-	}
-	glEnable(GL_FOG);
-}
-
-void V_CalcPowerupCshift (void) {
+void V_CalcPowerupCshift(void)
+{
 	float fraction;
 
 	if (cl.stats[STAT_ITEMS] & IT_QUAD)	{
@@ -560,40 +525,50 @@ void V_CalcPowerupCshift (void) {
 	}
 }
 
-void V_CalcBlend (void) {
-	float r, g, b, a, a2;
+void V_CalcBlend (void)
+{
+	float r, g, b, a, a2, t;
 	int j;
+	extern cvar_t gl_polyblend;
 
-	r = g = b = a= 0;
+	r = g = b = a = 0;
 
 	if (cls.state != ca_active) {
 		cl.cshifts[CSHIFT_CONTENTS] = cshift_empty;
 		cl.cshifts[CSHIFT_POWERUP].percent = 0;
-	} else {
+	}
+	else {
 		V_CalcPowerupCshift ();
 	}
 
 	// drop the damage value
-	cl.cshifts[CSHIFT_DAMAGE].percent -= cls.frametime * 150;
-	if (cl.cshifts[CSHIFT_DAMAGE].percent <= 0)
+	t = cls.frametime * 150;
+	cl.cshifts[CSHIFT_DAMAGE].percent -= t;
+	if (cl.cshifts[CSHIFT_DAMAGE].percent <= 0) {
 		cl.cshifts[CSHIFT_DAMAGE].percent = 0;
+	}
 
 	// drop the bonus value
 	cl.cshifts[CSHIFT_BONUS].percent -= cls.frametime * 100;
-	if (cl.cshifts[CSHIFT_BONUS].percent <= 0)
+	if (cl.cshifts[CSHIFT_BONUS].percent <= 0) {
 		cl.cshifts[CSHIFT_BONUS].percent = 0;
+	}
 
 	for (j = 0; j < NUM_CSHIFTS; j++)	{
-		if ((!gl_cshiftpercent.value || !gl_polyblend.value) && j != CSHIFT_CONTENTS)
+		if ((!gl_cshiftpercent.value || !gl_polyblend.value) && j != CSHIFT_CONTENTS) {
 			continue;
+		}
 
-		if (j == CSHIFT_CONTENTS)
+		if (j == CSHIFT_CONTENTS) {
 			a2 = cl.cshifts[j].percent / 100.0 / 255.0;
-		else
+		}
+		else {
 			a2 = ((cl.cshifts[j].percent * gl_cshiftpercent.value) / 100.0) / 255.0;
+		}
 
-		if (!a2)
+		if (!a2) {
 			continue;
+		}
 		a = a + a2 * (1-a);
 
 		a2 = a2 / a;
@@ -609,18 +584,41 @@ void V_CalcBlend (void) {
 	v_blend[3] = bound(0, v_blend[3], 1);
 }
 
-void V_AddLightBlend (float r, float g, float b, float a2) {
-	float	a;
+void V_AddLightBlend(float r, float g, float b, float a2, qbool suppress_polyblend)
+{
+	float a;
+	extern cvar_t gl_polyblend;
+	qbool shift_on_dlight = gl_polyblend.integer && (v_dlightcshift.integer == 1 || (v_dlightcshift.integer == 2 && !suppress_polyblend));
+	float percentage = bound(0, v_dlightcshiftpercent.value, 1);
 
-	if (!gl_polyblend.value || !gl_cshiftpercent.value || !v_dlightcshift.value)
+	if (percentage <= 0 || !shift_on_dlight) {
 		return;
+	}
 
-	a2 = a2 * bound(0, v_dlightcshift.value, 1) * gl_cshiftpercent.value / 100.0;
+	if (!v_dlightcolor.integer) {
+		r = 1.0f;
+		g = 0.5f;
+		b = 0.0f;
+	}
+	else {
+		// some kind of scaling, the normal colors aren't full red/blue etc
+		float max = max(r, max(g, b));
+
+		if (max > 0) {
+			r /= max;
+			g /= max;
+			b /= max;
+		}
+	}
+
+	a2 = bound(0, a2, 1);
+	a2 *= percentage;
 
 	v_blend[3] = a = v_blend[3] + a2 * (1 - v_blend[3]);
 
-	if (!a)
+	if (!a) {
 		return;
+	}
 
 	a2 = a2 / a;
 
@@ -629,77 +627,94 @@ void V_AddLightBlend (float r, float g, float b, float a2) {
 	v_blend[2] = v_blend[2] * (1 - a2) + b * a2;
 }
 
-void V_UpdatePalette (void) {
+void V_UpdatePalette (void)
+{
 	int i, j, c;
-	qbool new;
 	float current_gamma, current_contrast, a, rgb[3];
-	static float prev_blend[4];
-	static float old_gamma, old_contrast, old_hwblend;
+	static float prev_blend[4] = { 0, 0, 0, 0 };
+	static float old_gamma = 1, old_contrast = 1;
+	static qbool old_change_palette = false;
+	static double last_set = 0;
 	extern float vid_gamma;
+	float hw_vblend[4];
+	// whether or not to include content/damage palette shifts as part of the hw gamma
+	qbool change_palette = (vid_hwgamma_enabled && gl_hwblend.value && !cl.teamfortress);
+	int change_flags = 0;
 
-	new = false;
-
-	for (i = 0; i < 4; i++) {
-		if (v_blend[i] != prev_blend[i]) {
-			new = true;
-			prev_blend[i] = v_blend[i];
-		}
+	if (change_palette) {
+		memcpy(hw_vblend, v_blend, sizeof(hw_vblend));
 	}
-
-	current_gamma = bound (0.3, v_gamma.value, 3);
-	if (current_gamma != old_gamma || v_gamma.modified) {
-		v_gamma.modified = false;
-		old_gamma = current_gamma;
-		new = true;
+	else {
+		memset(hw_vblend, 0, sizeof(hw_vblend));
 	}
+	current_gamma = (vid_hwgamma_enabled ? bound(0.3, v_gamma.value, 3) : 1);
+	current_contrast = (vid_hwgamma_enabled ? bound(1, v_contrast.value, 3) : 1);
 
-	current_contrast = bound (1, v_contrast.value, 3);
-	if (current_contrast != old_contrast) {
-		old_contrast = current_contrast;
-		new = true;
-	}
+	change_flags |= (memcmp(hw_vblend, prev_blend, sizeof(hw_vblend)) ? 1 : 0);
+	change_flags |= (current_gamma != old_gamma || v_gamma.modified ? 2 : 0);
+	change_flags |= (current_contrast != old_contrast ? 4 : 0);
+	change_flags |= (change_palette != old_change_palette ? 8 : 0);
 
-	if (gl_hwblend.value != old_hwblend) {
-		new = true;
-		old_hwblend = gl_hwblend.value;
-	}
-
-	if (!new)
+	if (!change_flags) {
 		return;
-
-	a = v_blend[3];
-
-	if (!vid_hwgamma_enabled || !gl_hwblend.value || cl.teamfortress)
-		a = 0;
-
-	rgb[0] = 255 * v_blend[0] * a;
-	rgb[1] = 255 * v_blend[1] * a;
-	rgb[2] = 255 * v_blend[2] * a;
-
-	a = 1 - a;
-	if (vid_gamma != 1.0) {
-		current_contrast = pow (current_contrast, vid_gamma);
-		current_gamma = current_gamma/vid_gamma;
 	}
 
+	// don't update if not enough time has passed & only palette updating
+	if (change_flags == 1 && vid_hwgamma_fps.integer && curtime < last_set + (1.0f / max(10, vid_hwgamma_fps.integer))) {
+		return;
+	}
+
+	// store flags
+	old_change_palette = change_palette;
+	prev_blend[0] = hw_vblend[0];
+	prev_blend[1] = hw_vblend[1];
+	prev_blend[2] = hw_vblend[2];
+	prev_blend[3] = hw_vblend[3];
+	old_gamma = current_gamma;
+	old_contrast = current_contrast;
+	last_set = curtime;
+	v_gamma.modified = false;
+	if (developer.integer == 3) {
+		Con_DPrintf("palette: change_flags %d [%d %d %d %d] %.2f %.2f\n", change_flags, (int)(hw_vblend[0] * 255), (int)(hw_vblend[1] * 255), (int)(hw_vblend[2] * 255), (int)(hw_vblend[3] * 255), current_gamma, current_contrast);
+	}
+
+	a = hw_vblend[3];
+	if (R_OldGammaBehaviour() && vid_gamma != 1.0) {
+		current_contrast = pow(current_contrast, vid_gamma);
+		current_gamma = current_gamma / vid_gamma;
+	}
+
+	// Have to do this in a loop these days as certain color ranges will be blocked by OS
+	do {
+		float std_alpha;
+
+		rgb[0] = 255 * hw_vblend[0] * a;
+		rgb[1] = 255 * hw_vblend[1] * a;
+		rgb[2] = 255 * hw_vblend[2] * a;
+
+		std_alpha = 1 - a;
+
 #ifdef X11_GAMMA_WORKAROUND
-	a *= 256.0/glConfig.gammacrap.size;
-	for (i = 0; i < glConfig.gammacrap.size; i++) {
+		std_alpha *= 256.0 / glConfig.gammacrap.size;
+		for (i = 0; i < glConfig.gammacrap.size; i++) {
 #else
-	for (i = 0; i < 256; i++) {
+		for (i = 0; i < 256; i++) {
 #endif
-		for (j = 0; j < 3; j++) {
-			// apply blend and contrast
-			c = (i * a + rgb[j]) * current_contrast;
-			if (c > 255)
-				c = 255;
-			// apply gamma
-			c = 255 * pow(c / 255.5, current_gamma) + 0.5;
-			c = bound (0, c, 255);
-			ramps[j][i] = c << 8;
+			for (j = 0; j < 3; j++) {
+				// apply blend and contrast
+				c = (i * std_alpha + rgb[j]) * current_contrast;
+				if (c > 255) {
+					c = 255;
+				}
+				// apply gamma
+				c = 255 * pow(c / 255.5, current_gamma) + 0.5;
+				c = bound(0, c, 255);
+				ramps[j][i] = c << 8;
+			}
 		}
-	}
-	VID_SetDeviceGammaRamp ((unsigned short *) ramps);
+
+		a *= 0.8;
+	} while (VID_SetDeviceGammaRamp((unsigned short *)ramps) && a > 0.1);
 }
 
 // BorisU -->
@@ -716,7 +731,7 @@ void V_TF_ClearGrenadeEffects (void)
 	Cvar_SetValue (&v_idlescale, 0.0f);
 
 	// Flash effect off
-	if (flashed && (!strncasecmp(Rulesets_Ruleset(), "MTFL", 4))) {
+	if (flashed && Rulesets_ToggleWhenFlashed()) {
 		V_TF_FlashSettings (false);
 	}
 	flashed = false;
@@ -796,8 +811,8 @@ void V_CalcViewRoll (void) {
 // if user wish so, weapon pre-selection is also taken in account
 // todo: if user selects different weapon while the current one is still
 // firing, wait until the animation is finished
-static int V_CurrentWeaponModel(void) { 
-	extern int IN_BestWeaponReal(void);
+static int V_CurrentWeaponModel(void)
+{
 	extern cvar_t cl_weaponpreselect;
 	int bestgun;
 	int realw = cl.stats[STAT_WEAPON];
@@ -829,7 +844,7 @@ static int V_CurrentWeaponModel(void) {
 	}
 	else {
 		if (ShowPreselectedWeap() && r_viewpreselgun.integer && !view_message.weaponframe) {
-			bestgun = IN_BestWeaponReal();
+			bestgun = IN_BestWeaponReal(true);
 			if (bestgun == 1) {
 				return cl_modelindices[mi_vaxe];
 			}
@@ -842,7 +857,9 @@ static int V_CurrentWeaponModel(void) {
 }
 
 extern void TP_ParseWeaponModel(model_t *model);
-void V_AddViewWeapon (float bob) {
+
+static void V_AddViewWeapon(float bob)
+{
 	vec3_t forward, right, up;
 	centity_t *cent;
 	int gunmodel = V_CurrentWeaponModel();
@@ -852,9 +869,8 @@ void V_AddViewWeapon (float bob) {
 	TP_ParseWeaponModel(cl.model_precache[gunmodel]);
 
 	if (!cl_drawgun.value || (cl_drawgun.value == 2 && scr_fov.value > 90)
-		|| ((view_message.flags & (PF_GIB|PF_DEAD)))	
-		|| cl.stats[STAT_ITEMS] & IT_INVISIBILITY || cl.stats[STAT_HEALTH] <= 0 || !Cam_DrawViewModel())
-	{
+		|| ((view_message.flags & (PF_GIB | PF_DEAD)))
+		|| cl.stats[STAT_ITEMS] & IT_INVISIBILITY || cl.stats[STAT_HEALTH] <= 0 || !Cam_DrawViewModel()) {
 		cent->current.modelindex = 0;	//no model
 		return;
 	}
@@ -865,18 +881,19 @@ void V_AddViewWeapon (float bob) {
 	cent->current.angles[ROLL] = r_refdef.viewangles[ROLL];
 	//origin
 
-	AngleVectors (r_refdef.viewangles, forward, right, up);
-	VectorCopy (r_refdef.vieworg, cent->current.origin);
-	VectorMA (cent->current.origin, bob * 0.4, forward, cent->current.origin);
+	AngleVectors(r_refdef.viewangles, forward, right, up);
+	VectorCopy(r_refdef.vieworg, cent->current.origin);
+
+	VectorMA(cent->current.origin, bob * 0.4, forward, cent->current.origin);
 
 	if (r_viewmodeloffset.string[0]) {
 		float offset[3];
-		int size = sizeof(offset)/sizeof(offset[0]);
+		int size = sizeof(offset) / sizeof(offset[0]);
 
 		ParseFloats(r_viewmodeloffset.string, offset, &size);
-		VectorMA (cent->current.origin,  offset[0], right,   cent->current.origin);
-		VectorMA (cent->current.origin, -offset[1], up,      cent->current.origin);
-		VectorMA (cent->current.origin,  offset[2], forward, cent->current.origin);
+		VectorMA(cent->current.origin, offset[0], right, cent->current.origin);
+		VectorMA(cent->current.origin, -offset[1], up, cent->current.origin);
+		VectorMA(cent->current.origin, offset[2], forward, cent->current.origin);
 	}
 
 	// fudge position around to keep amount of weapon visible roughly equal with different FOV
@@ -893,7 +910,8 @@ void V_AddViewWeapon (float bob) {
 
 	if (cent->current.modelindex != gunmodel) {
 		cent->frametime = -1;
-	} else {
+	}
+	else {
 		if (cent->current.frame != view_message.weaponframe) {
 			cent->frametime = cl.time;
 			cent->oldframe = cent->current.frame;
@@ -904,7 +922,8 @@ void V_AddViewWeapon (float bob) {
 	cent->current.frame = view_message.weaponframe;
 }
 
-void V_CalcIntermissionRefdef (void) {
+static void V_CalcIntermissionRefdef(void)
+{
 	float old;
 
 	VectorCopy (cl.simorg, r_refdef.vieworg);
@@ -920,67 +939,78 @@ void V_CalcIntermissionRefdef (void) {
 	v_idlescale.value = old;
 }
 
-void V_CalcRefdef (void) {
+static void V_CalcRefdef(void)
+{
 	vec3_t forward;
 	float bob;
 	float height_adjustment;
 
-	V_DriftPitch ();
+	V_DriftPitch();
 
-	bob = V_CalcBob ();
-	
-	height_adjustment = v_viewheight.value ? bound (-7, v_viewheight.value, 4) : V_CalcBob ();
-	
+	bob = V_CalcBob();
+
+	height_adjustment = v_viewheight.value ? bound(-7, v_viewheight.value, 4) : V_CalcBob();
+	if (cl_bobhead.integer) {
+		height_adjustment += bob;
+		bob = 0;
+	}
 
 	// set up the refresh position
-	VectorCopy (cl.simorg, r_refdef.vieworg);
+	VectorCopy(cl.simorg, r_refdef.vieworg);
 
 	// never let it sit exactly on a node line, because a water plane can
 	// dissapear when viewed with the eye exactly on it.
 	// the server protocol only specifies to 1/8 pixel, so add 1/16 in each axis
-	r_refdef.vieworg[0] += 1.0/16;
-	r_refdef.vieworg[1] += 1.0/16;
-	r_refdef.vieworg[2] += 1.0/16;
+	r_refdef.vieworg[0] += 1.0 / 16;
+	r_refdef.vieworg[1] += 1.0 / 16;
+	r_refdef.vieworg[2] += 1.0 / 16;
 
 	// add view height
+	r_refdef.viewheight_test = 4;
 	if (view_message.flags & PF_GIB) {
 		r_refdef.vieworg[2] += 8;	// gib view height
-	} else if (view_message.flags & PF_DEAD && (cl.stats[STAT_HEALTH] <= 0)) {	
+	}
+	else if (view_message.flags & PF_DEAD && (cl.stats[STAT_HEALTH] <= 0)) {
 		r_refdef.vieworg[2] -= 16;	// corpse view height
-	} else {
+	}
+	else {
 		// normal view height
 		// Use STAT_VIEWHEIGHT in case of server support it or NQ demoplayback, if not then use default viewheight.
 		r_refdef.vieworg[2] += ((cl.z_ext & Z_EXT_VIEWHEIGHT) || cls.nqdemoplayback) ? cl.stats[STAT_VIEWHEIGHT] : DEFAULT_VIEWHEIGHT;
 
 		r_refdef.vieworg[2] += height_adjustment;
 
-		r_refdef.vieworg[2] += bob;
-		
 		// smooth out stair step ups
 		r_refdef.vieworg[2] += cl.crouch;
+
+		// standard offset
+		r_refdef.viewheight_test = 10;
 	}
 
 	// set up refresh view angles
-	VectorCopy (cl.simangles, r_refdef.viewangles);
-	V_CalcViewRoll ();
-	V_AddIdle ();
+	VectorCopy(cl.simangles, r_refdef.viewangles);
+	V_CalcViewRoll();
+	V_AddIdle();
 
 	if (v_gunkick.value) {
 		// add weapon kick offset
-		AngleVectors (r_refdef.viewangles, forward, NULL, NULL);
-		VectorMA (r_refdef.vieworg, cl.punchangle, forward, r_refdef.vieworg);
+		AngleVectors(r_refdef.viewangles, forward, NULL, NULL);
+		VectorMA(r_refdef.vieworg, cl.punchangle, forward, r_refdef.vieworg);
 
 		// add weapon kick angle
 		r_refdef.viewangles[PITCH] += cl.punchangle * 0.5;
 	}
-	
-	if (view_message.flags & PF_DEAD && (cl.stats[STAT_HEALTH] <= 0))
+
+	if (view_message.flags & PF_DEAD && (cl.stats[STAT_HEALTH] <= 0)) {
 		r_refdef.viewangles[ROLL] = 80;	// dead view angle
+	}
 
 	//VULT CAMERAS
 	CameraUpdate(view_message.flags & PF_DEAD);
-	V_AddViewWeapon (height_adjustment);
-	
+
+	// meag: really viewheight shouldn't be here, but it was incorrectly passed for years instead of bob,
+	//       and so without it the gun is rendered too far forward if e.g. viewheight -6
+	V_AddViewWeapon(bob + height_adjustment);
 }
 
 void DropPunchAngle (void) {
@@ -1005,51 +1035,87 @@ void DropPunchAngle (void) {
 //the entity origin, so any view position inside that will be valid
 extern vrect_t scr_vrect;
 
-void V_RenderView (void) {
+qbool V_PreRenderView(void)
+{
 	char *p;
 
 	cl.simangles[ROLL] = 0;	// FIXME @@@ 
 
 	if (cls.state != ca_active) {
-		V_CalcBlend ();
-		return;
+		V_CalcBlend();
+	}
+	else {
+		view_frame = &cl.frames[cl.validsequence & UPDATE_MASK];
+		if (!cls.nqdemoplayback) {
+			view_message = view_frame->playerstate[cl.viewplayernum];
+		}
+
+		DropPunchAngle();
+		if (cl.intermission) {
+			// intermission / finale rendering
+			V_CalcIntermissionRefdef();
+		}
+		else {
+			V_CalcRefdef();
+		}
+
+		MVD_PowerupCam_Frame();
+
+		R_PushDlights();
+
+		r_refdef2.time = cl.time;
+		r_refdef2.sin_time = sin(r_refdef2.time);
+		r_refdef2.cos_time = cos(r_refdef2.time);
+
+		// scroll parameters for powerup shells
+		r_refdef2.powerup_scroll_params[0] = cos(cl.time * 1.5);
+		r_refdef2.powerup_scroll_params[1] = sin(cl.time * 1.1);
+		r_refdef2.powerup_scroll_params[2] = cos(cl.time * -0.5);
+		r_refdef2.powerup_scroll_params[3] = sin(cl.time * -0.5);
+
+		// restrictions
+		r_refdef2.allow_cheats = cls.demoplayback || (Info_ValueForKey(cl.serverinfo, "*cheats")[0] && com_serveractive);
+		if (cls.demoplayback || cl.spectator) {
+			r_refdef2.allow_lumas = true;
+			r_refdef2.max_fbskins = 1;
+		}
+		else {
+			r_refdef2.allow_lumas = !strcmp(Info_ValueForKey(cl.serverinfo, "24bit_fbs"), "0") ? false : true;
+			r_refdef2.max_fbskins = *(p = Info_ValueForKey(cl.serverinfo, "fbskins")) ? bound(0, Q_atof(p), 1) : (cl.teamfortress ? 0 : 1);
+		}
+
+		// Only allow alpha water if the server allows it, or they are spectator and have novis enabled
+		{
+			extern cvar_t r_novis;
+
+			r_refdef2.max_watervis = *(p = Info_ValueForKey(cl.serverinfo, "watervis")) ? bound(0, Q_atof(p), 1) : 0;
+			if ((cls.demoplayback || cl.spectator) && (r_novis.integer || r_refdef2.max_watervis > 0)) {
+				// ignore server limit
+				r_refdef2.max_watervis = 1;
+			}
+			r_refdef2.wateralpha = R_WaterAlpha();  // relies on r_refdef2.max_watervis
+		}
+
+		// time-savers
+		{
+			extern cvar_t r_drawflat_mode, r_drawflat, r_fastturb, gl_caustics;
+			extern texture_ref underwatertexture;
+
+			r_refdef2.drawFlatFloors = r_drawflat_mode.integer == 0 && (r_drawflat.integer == 2 || r_drawflat.integer == 1);
+			r_refdef2.drawFlatWalls = r_drawflat_mode.integer == 0 && (r_drawflat.integer == 3 || r_drawflat.integer == 1);
+			r_refdef2.solidTexTurb = (!r_fastturb.integer && r_refdef2.wateralpha == 1);
+
+			r_refdef2.drawCaustics = (R_TextureReferenceIsValid(underwatertexture) && gl_caustics.integer);
+			r_refdef2.drawWorldOutlines = R_DrawWorldOutlines();
+			r_refdef2.distanceScale = tan(r_refdef.fov_x * (M_PI / 180) * 0.5f);
+			VectorScale(vpn, 0.002 * r_refdef2.distanceScale, r_refdef2.outline_vpn);
+			r_refdef2.outlineBase = 1 - DotProduct(r_origin, r_refdef2.outline_vpn);
+		}
 	}
 
-	view_frame = &cl.frames[cl.validsequence & UPDATE_MASK];
-	if (!cls.nqdemoplayback)
-		view_message = view_frame->playerstate[cl.viewplayernum];
+	renderer.PreRenderView();
 
-	DropPunchAngle ();
-	if (cl.intermission) // intermission / finale rendering		
-		V_CalcIntermissionRefdef ();	
-	else
-		V_CalcRefdef ();
-
-	R_PushDlights ();
-
-	r_refdef2.time = cl.time;
-
-	// restrictions
-	r_refdef2.allow_cheats = (Info_ValueForKey(cl.serverinfo, "*cheats")[0] && com_serveractive)
-		|| cls.demoplayback;
-	if (cls.demoplayback || cl.spectator)
-	{
-		r_refdef2.allow_lumas = true;
-		r_refdef2.max_fbskins = 1;
-		r_refdef2.max_watervis = 1;
-	}
-	else 
-	{
-		r_refdef2.allow_lumas = !strcmp(Info_ValueForKey(cl.serverinfo, "24bit_fbs"), "0") ? false : true;
-		r_refdef2.max_fbskins = *(p = Info_ValueForKey(cl.serverinfo, "fbskins")) ? bound(0, Q_atof(p), 1) :
-			cl.teamfortress ? 0 : 1;
-		r_refdef2.max_watervis = *(p = Info_ValueForKey(cl.serverinfo, "watervis")) ? bound(0, Q_atof(p), 1) : 0;
-	}
-
-//	r_refdef2.viewplayernum = Cam_PlayerNum();
-//	r_refdef2.lightstyles = cl_lightstyle;
-
-	R_RenderView ();
+	return cls.state == ca_active;
 }
 
 //============================================================================
@@ -1077,22 +1143,23 @@ void V_Init (void) {
 	Cvar_Register (&r_nearclip);
 
 	Cvar_SetCurrentGroup(CVAR_GROUP_CROSSHAIR);
-	Cvar_Register (&crosshaircolor);
-	Cvar_Register (&crosshair);
-	Cvar_Register (&crosshairsize);
-	Cvar_Register (&cl_crossx);
-	Cvar_Register (&cl_crossy);
+	Cvar_Register(&crosshaircolor);
+	Cvar_Register(&crosshair);
+	Cvar_Register(&crosshairsize);
+	Cvar_Register(&cl_crossx);
+	Cvar_Register(&cl_crossy);
 
 	Cvar_SetCurrentGroup(CVAR_GROUP_VIEWMODEL);
-	Cvar_Register (&cl_bob);
-	Cvar_Register (&cl_bobcycle);
-	Cvar_Register (&cl_bobup);
+	Cvar_Register(&cl_bob);
+	Cvar_Register(&cl_bobcycle);
+	Cvar_Register(&cl_bobup);
+	Cvar_Register(&cl_bobhead);
 
-	Cvar_Register (&cl_drawgun);
-	Cvar_Register (&r_viewmodelsize);
-	Cvar_Register (&r_viewmodeloffset);
-	Cvar_Register (&r_viewpreselgun);
-	Cvar_Register (&r_viewlastfired);
+	Cvar_Register(&cl_drawgun);
+	Cvar_Register(&r_viewmodelsize);
+	Cvar_Register(&r_viewmodeloffset);
+	Cvar_Register(&r_viewpreselgun);
+	Cvar_Register(&r_viewlastfired);
 
 	Cvar_SetCurrentGroup(CVAR_GROUP_VIEW);
 	Cvar_Register (&v_kicktime);
@@ -1115,16 +1182,34 @@ void V_Init (void) {
 	Cvar_Register (&v_pentcshift);
 	Cvar_Register (&cl_demoplay_flash); // from QW262
 
-	Cvar_Register (&v_dlightcshift);
-	Cvar_Register (&gl_cshiftpercent);
-	Cvar_Register (&gl_hwblend);
+	Cvar_Register(&v_dlightcshift);
+	Cvar_Register(&v_dlightcolor);
+	Cvar_Register(&v_dlightcshiftpercent);
+	Cvar_Register(&gl_cshiftpercent);
+	Cvar_Register(&gl_hwblend);
+	Cvar_Register(&vid_hwgamma_fps);
 
 	Cvar_SetCurrentGroup(CVAR_GROUP_SCREEN);
-	Cvar_Register (&v_gamma);
-	Cvar_Register (&v_contrast);
+	Cvar_Register(&v_gamma);
+	Cvar_Register(&v_contrast);
 
+	// we do not need this after host initialized
+	if (!host_initialized) {
+		int i;
+		float def_gamma = 1.0f;
+		extern float vid_gamma;
+
+		if ((i = COM_CheckParm(cmdline_param_client_gamma)) != 0 && i + 1 < COM_Argc()) {
+			def_gamma = Q_atof(COM_Argv(i + 1));
+			def_gamma = bound(0.3, def_gamma, 3);
+			Cvar_SetDefaultAndValue(&v_gamma, def_gamma, def_gamma);
+			vid_gamma = def_gamma;
+		}
+		else {
+			vid_gamma = 1.0;
+		}
+
+		v_gamma.modified = true;
+	}
 	Cvar_ResetCurrentGroup();
-
-	Cmd_AddLegacyCommand ("gamma", v_gamma.name);
-	Cmd_AddLegacyCommand ("contrast", v_contrast.name);
 }
