@@ -19,14 +19,12 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
 extern float v_blend[4];
-void V_AddLightBlend (float r, float g, float b, float a2);
-void V_AddWaterfog (int contents);		 
+void V_AddLightBlend (float r, float g, float b, float a2, qbool suppress_polyblend);
 
 extern cvar_t v_gamma;
 extern cvar_t v_contrast;
 
 void V_Init (void);
-void V_RenderView (void);
 void V_UpdatePalette (void);
 void V_ParseDamage (void);
 void V_SetContentsColor (int contents);
