@@ -28,10 +28,16 @@ $Id: client.h,v 1.74 2007-10-12 00:08:42 cokeman1982 Exp $
 */
 // client.h
 
+#ifndef EZQUAKE_CLIENT_HEADER
+#define EZQUAKE_CLIENT_HEADER
+
 #if defined(_MSC_VER) && !defined(__attribute__)
 #define __attribute__(A) /**/
 #endif
 
+#define MAXWEAPONS 10
+#define MAX_ANTIROLLOVER_LEVELS 32
+
 #define MAX_STATIC_SOUNDS 256
 typedef struct
 {
@@ -44,23 +50,39 @@ typedef struct
 extern cvar_t cl_demospeed;
 extern cvar_t cl_demoteamplay;
 
-#define QTV_PLAYBACK		2			// cls.mvdplayback == QTV_PLAYBACK if QTV playback
+#define MVD_FILE_PLAYBACK   1
+#define QTV_PLAYBACK        2           // cls.mvdplayback == QTV_PLAYBACK if QTV playback
 #define ISPAUSED (cl.paused || (!cl_demospeed.value && cls.demoplayback && cls.mvdplayback != QTV_PLAYBACK && !cls.timedemo))
 #define	MAX_PROJECTILES	32
 
+typedef enum {
+	skin_base,
+	skin_base_teammate,
+	skin_fb,
+	skin_dead,
+	skin_dead_teammate,
+	skin_textures
+} skin_texture_t;
+
 #define MV_VIEWS 4
 
 typedef struct 
 {
-	char			name[16];
-	qbool			failedload;		// the name isn't a valid skin
-	qbool			warned;			// warning about falied to load was alredy printed
-	int				width, height;	// this is valid for pcx too, but used for 32bit skins only
-	int				bpp;			// used in gl,  bpp = 1 for pcx and 4 for 32bit skins
-	int				texnum;			// texture num, used for 32bit skins, speed up
-	cache_user_t	cache;
+	char            name[16];
+	qbool           failedload;             // the name isn't a valid skin
+	qbool           warned;                 // warning about falied to load was alredy printed
+	int             width, height;          // this is valid for pcx too, but used for 32bit skins only
+	int             bpp;                    // used in gl, bpp = 1 for pcx and 4 for 32bit skins
+	texture_ref     texnum[skin_textures];  // texture num, used for 32bit skins, speed up
+	byte*           cached_data;
 } skin_t;
 
+enum {
+	dbg_antilag_rewind_present = 1,
+	dbg_antilag_client_present = 2,
+	dbg_antilag_position_set   = 4
+};
+
 // player_state_t is the information needed by a player entity
 // to do move prediction and to generate a drawable entity
 typedef struct 
@@ -91,15 +113,28 @@ typedef struct
 	qbool		onground;
 	qbool		jump_held;
 	int			jump_msec;		// Fix bunny-hop flickering.
+
+	vec3_t      current_origin; // current location (no antilag applied)
+	vec3_t      rewind_origin;  // location antilag server has rewound to
+	vec3_t      client_origin;  // location client rendered the player
+	int         antilag_flags;  // bitmask: dbg_antilag_rewind_present | dbg_antilag_client_present
 } player_state_t;
 
 #define	MAX_SCOREBOARDNAME	16
+typedef enum {
+	gender_unknown = 0,
+	gender_male,
+	gender_female,
+	gender_neutral
+} gender_id;
+
 typedef struct player_info_s 
 {
 	int		userid;
 
 	// Scoreboard information.
 	char	name[MAX_SCOREBOARDNAME];
+	char    shortname[MAX_SCOREBOARDNAME];           // used in tracker, when user wants to remove prefixes
 	float	entertime;
 	int		frags;
 	int		ping;
@@ -109,9 +144,11 @@ typedef struct player_info_s
 	// Skin information.
 	unsigned char   topcolor;
 	unsigned char   bottomcolor;
+	qbool           teammate;
 
 	unsigned char   _topcolor;
 	unsigned char   _bottomcolor;
+	qbool           _teammate;
 
 	unsigned char   real_topcolor;
 	unsigned char   real_bottomcolor;
@@ -129,8 +166,6 @@ typedef struct player_info_s
 	qbool	skin_refresh;
 	qbool	ignored;                // for ignore
 	qbool   vignored;               // for voip-ignore
-	qbool	validated;              // for authentication
-	char	f_server[16];           // for f_server responses
 
 	// VULT DEATH EFFECT
 	// Better putting the dead flag here instead of on the entity so whats dead stays dead
@@ -141,11 +176,20 @@ typedef struct player_info_s
 	char	_team[MAX_INFO_STRING];
 	int     known_team_color;
 
-	// 
+	// used for showing stack/health bar hud elements
 	double  max_health_last_set;
 	int     max_health;
 	double  prev_health_last_set;
 	int     prev_health;
+
+	// authentication
+	char	loginname[MAX_SCOREBOARDNAME];
+	char    loginflag[8];
+	int     loginflag_id;
+
+	// extracted from userinfo
+	int           chatflag;
+	gender_id     gender;
 } __attribute__((aligned(64))) player_info_t;
 
 
@@ -161,30 +205,26 @@ typedef struct
 typedef struct 
 {
 	// Generated on client side.
-	usercmd_t			cmd;				// Cmd that generated the frame.
-	double				senttime;			// Time cmd was sent off.
-	int					delta_sequence;		// Sequence number to delta from, -1 = full update.
-	int					sentsize;
+	usercmd_t           cmd;                        // Cmd that generated the frame.
+	double              senttime;                   // Time cmd was sent off.
+	int                 delta_sequence;             // Sequence number to delta from, -1 = full update.
+	int                 sentsize;
 
 	// Received from server.
-	double				receivedtime;		// Time message was received, or -1.
-	player_state_t		playerstate[MAX_CLIENTS];	// Message received that reflects performing the usercmd.
-	packet_entities_t	packet_entities;
-	qbool				invalid;			// True if the packet_entities delta was invalid
-	int					receivedsize;
-	int					seq_when_received;
+	double              receivedtime;               // Time message was received, or -1.
+	player_state_t      playerstate[MAX_CLIENTS];   // Message received that reflects performing the usercmd.
+	packet_entities_t   packet_entities;
+	qbool               invalid;                    // True if the packet_entities delta was invalid
+	int                 receivedsize;
+	int                 seq_when_received;
+
+	qbool               in_qwd;
 } frame_t;
 
-// Holds the fileposition for a given time in a demo
-// these are saved as a demo is being read, and can
-// then be used for rewinding.
-typedef struct demo_keyframe_s
-{
-	unsigned long			filepos;	// The position in the demo file where the keyframe can be found.
-	double					timestamp;	// The time stamp in question.
-	struct demo_keyframe_s	*prev;
-	// TODO : Add any state information needed to reproduce this frame.
-} demo_keyframe_t;
+typedef struct centity_trail_s {
+	vec3_t          stop;          // last position the trail was generated in
+	double          lasttime;      // last time this trail generated a particle, or 0 if new
+} centity_trail_t;
 
 typedef struct 
 {
@@ -192,12 +232,9 @@ typedef struct
 	entity_state_t	current;
 
 	vec3_t			lerp_origin;
-	vec3_t			trail_origin;
 
 	vec3_t			velocity; // hack
 
-	int				flags;
-
 	vec3_t			old_origin;
 	vec3_t			old_angles;
 	int				oldframe;
@@ -213,14 +250,17 @@ typedef struct
 	int				old_vw_frame;	// player entities only
 
 	int             contents;
-} centity_t;
 
-#define CENT_TRAILDRAWN		1
+	centity_trail_t trails[4];
+	float           particle_emittime;
+	int             trail_number;   // this changes as trails are killed and entity re-used (can be used to detect reset)
+	int             corona_id;
+} centity_t;
 
 typedef struct 
 {
 	int		destcolor[3];
-	int		percent;		// 0-256
+	float	percent;		// 0-256
 } cshift_t;
 
 #define	CSHIFT_CONTENTS	0
@@ -232,7 +272,7 @@ typedef struct
 
 // clientState_t should hold all pieces of the client state
 
-#define	MAX_DLIGHTS			64
+#define	MAX_DLIGHTS			32
 #define	MAX_STYLESTRING		64
 
 typedef enum {lt_default, lt_muzzleflash, lt_explosion, lt_rocket,
@@ -263,7 +303,6 @@ typedef struct {
 	char	map[MAX_STYLESTRING];
 } lightstyle_t;
 
-#define	MAX_EFRAGS			512
 #define	MAX_STATIC_ENTITIES	512		// torches, etc
 #define	MAX_DEMOS			8
 #define	MAX_DEMONAME		16
@@ -288,11 +327,13 @@ typedef enum
 
 typedef enum demoseekingtype_e
 {
-	DST_SEEKING_NONE = 0, ///< seeking nothing
-	DST_SEEKING_NORMAL = 1, ///< demo_jump seeking
-	DST_SEEKING_DEMOMARK, ///< demo_jump_mark seeking
-	DST_SEEKING_STATUS, ///< demo_jump_status seeking
-	DST_SEEKING_FOUND ///< mark/status seeking, hint that we are done and should stop seeking
+	DST_SEEKING_NONE = 0,         ///< seeking nothing
+	DST_SEEKING_NORMAL = 1,       ///< demo_jump seeking
+	DST_SEEKING_DEMOMARK,         ///< demo_jump_mark seeking
+	DST_SEEKING_STATUS,           ///< demo_jump_status seeking
+	DST_SEEKING_FOUND,            ///< mark/status seeking, hint that we are done and should stop seeking
+	DST_SEEKING_END,              ///< demo_jump_end seeking (intermission or 1 second before end)
+	DST_SEEKING_FOUND_NOREWIND,   ///< mark/status seeking, no automatic rewind
 } demoseekingtype_t;
 
 typedef enum
@@ -320,6 +361,24 @@ typedef struct {
 	demoseekingstatus_condition_t *conditions;
 } demoseekingstatus_t;
 
+#define TIMEDEMO_OFF      0
+#define TIMEDEMO_CLASSIC  1
+#define TIMEDEMO_FIXEDFPS 2
+
+#define TIMEDEMO_FIXEDFPS_DEFAULT (308)
+#define TIMEDEMO_FIXEDFPS_MINIMUM (20)
+#define TIMEDEMO_FIXEDFPS_MAXIMUM (10000)
+
+typedef struct {
+	double last_snapshot_time;
+	double last_run_time;
+	double lastfps_value;
+	double lastframetime_value;
+	double time_of_last_minfps_update;
+	double time_of_last_maxframetime_update;
+	int fps_count;
+} perfinfo_t;
+
 /// A structure that is persistent through an arbitrary number of server connections.
 typedef struct
 {
@@ -413,25 +472,34 @@ typedef struct
 	// Demo recording info must be here, because record
 	// is started before entering a map (and clearing clientState_t)
 	//
-	qbool		demorecording;
-	qbool		demoplayback;
+	qbool       demorecording;
+	qbool       demoplayback;
 	demoseekingtype_t demoseeking;
 	demoseekingstatus_t demoseekingstatus;
-	qbool		demorewinding;
-	char		demoname[MAX_PATH];
-	qbool		nqdemoplayback;
-	qbool		timedemo;
-	double		td_lastframe;       ///< To meter out one message a frame.
-	int			td_startframe;      ///< cls.framecount at start
-	double		td_starttime;       ///< Realtime at second frame of timedemo.
+	qbool       demorewinding;
+	char        demoname[MAX_PATH];
+	qbool       nqdemoplayback;
+	int         timedemo;
+	double      td_lastframe;              ///< To meter out one message a frame.
+	int         td_startframe;             ///< cls.framecount at start
+	double      td_starttime;              ///< Realtime at second frame of timedemo.
+	double      td_frametime;              ///< frametime for stop-motion timedemo (timedemo2)
+	int         td_frametime_stats[1000];  ///< keep track of performance (to 0.1ms level, if it's over 100ms we're in bad shape)
+	int         td_frametime_max;          ///< worse ms score so far
+	int         td_frametime_max_frame;    ///< which frame# did we get that on?
+	double      td_nonrendering;           ///< time not in the rendering hot loop
 
 	qbool		mvdrecording;		///< this is not real mvd recording, but just cut particular moment of mvd stream
 
-	byte		demomessage_data[MAX_MSGLEN * 2];
 	sizebuf_t	demomessage;
 
-	double		fps;
-	double		min_fps;
+	double      fps;
+	double      min_fps;
+	double      avg_frametime;
+	double      max_frametime;
+
+	// FPS stats for performance info
+	perfinfo_t  fps_stats;
 
 	int			challenge;
 
@@ -453,16 +521,43 @@ typedef struct
 	int			lasttype;		///< The type of the last demo message.
 	qbool		findtrack;
 
+	// authenticating via web server
+	char        auth_logintoken[128];
 } clientPersistent_t;
 
 extern clientPersistent_t	cls;
 
+typedef struct antilag_pos_s {
+	vec3_t      pos;
+	qbool       present;
+	vec3_t      clientpos;
+	qbool       clientpresent;
+} antilag_pos_t;
+
+typedef struct antilag_stats_s {
+	double      client_rewind_distance;
+	double      client_rewind_samples;
+} antilag_stats_t;
 
 // cl.paused flags
 
 #define PAUSED_SERVER		1
 #define PAUSED_DEMO			2
 
+#define MAX_DAMAGE_NOTIFICATION_TIME 1.5
+#define MAX_DAMAGE_NOTIFICATIONS 15 // ((int)(10 * MAX_DAMAGE_NOTIFICATION_TIME))
+
+typedef struct scr_damage_s {
+	char text[64];
+	double time;
+	vec3_t origin;
+	vec3_t offset;
+	float vel[2];
+
+	// position on screen
+	float x, y;
+	qbool visible;
+} scr_damage_t;
 
 /// a structure that is wiped completely at every server signon
 typedef struct {
@@ -489,6 +584,9 @@ typedef struct {
 
 	int			parsecount;			///< server message counter
 	int			oldparsecount;
+	int         parsecountmod;
+	double      parsecounttime;
+
 
 	int			validsequence;		///< this is the sequence number of the last good
 									///< packetentity_t we got.  If this is 0, we can't
@@ -624,13 +722,53 @@ typedef struct {
 	int         cif_flags;
 
 	int         scoring_system;
+
+	char        fixed_team_names[4][16];
 	qbool       mvd_ktx_markers;
 
 	// r_viewmodellastfired
 	int         lastfired;
 	int         lastviewplayernum;
 
-	char        fixed_team_names[4][16];
+	// Weapon preferences
+	int         weapon_order[MAXWEAPONS];
+	int         weapon_order_clientside[MAXWEAPONS];
+	int         weapon_order_sequence_set;
+	qbool       weapon_order_use_clientside;
+
+	// anti-rollover weapon firing
+	int         ar_weapon_orders[MAX_ANTIROLLOVER_LEVELS][MAXWEAPONS];
+	int         ar_keycodes[MAX_ANTIROLLOVER_LEVELS];
+	int         ar_count;
+
+	// When teamlock 1 is specified, lock in the selected team and don't change again
+	char        teamlock1_teamname[16];
+
+	// authenticating via web server
+	char        auth_challenge[128];
+
+	// camera tracking
+	int         autocam;              // CAM_NONE or CAM_TRACK
+	int         spec_track;           // player# of who we are tracking
+	int         ideal_track;          // The currently tracked player.
+	qbool       spec_locked;          // Is the spectator locked to a players view or free flying.
+
+	// antilag debug playback
+	antilag_pos_t antilag_positions[MAX_CLIENTS];
+	antilag_stats_t antilag_stats[MAX_CLIENTS];
+
+	// demoinfo (stats file embedded in demo)
+	int         demoinfo_blocknumber;
+	int         demoinfo_bytes;
+
+	// damage notifications
+	scr_damage_t damage_notifications[MAX_DAMAGE_NOTIFICATIONS];
+
+	// fog info read from worldspawn
+	vec3_t map_fog_color;
+	float map_fog_density;
+	qbool map_fog_enabled;
+	float map_fog_sky;
 } clientState_t;
 
 #define SCORING_SYSTEM_DEFAULT   0
@@ -639,14 +777,35 @@ typedef struct {
 
 extern	clientState_t	cl;
 
+typedef enum visentlist_entrytype_s {
+	visent_outlines,
+	visent_firstpass,
+	visent_normal,
+	visent_alpha,
+	visent_shells,
+	visent_additive,
+
+	visent_max
+} visentlist_entrytype_t;
+
+typedef struct visentity_s {
+	entity_t    ent;
+	modtype_t   type;
+	float       distance;
+
+	qbool       draw[visent_max];
+} visentity_t;
+
+#define MAX_STANDARD_ENTITIES 512
+
 typedef struct visentlist_s {
-	entity_t	*list;
-	int			count;
-	int			max;
-	int			alpha;
+	visentity_t list[MAX_STANDARD_ENTITIES];
+	int         count;
+
+	int         typecount[visent_max];
 } visentlist_t;
 
-extern visentlist_t cl_firstpassents, cl_visents, cl_alphaents;
+extern visentlist_t cl_visents;
 
 // ezQuake cvars
 extern cvar_t cl_floodprot;
@@ -676,6 +835,8 @@ extern cvar_t r_explosionlight;
 extern cvar_t r_explosiontype;
 extern cvar_t r_flagcolor;
 extern cvar_t r_lightflicker;
+extern cvar_t r_lightmap_lateupload;
+extern cvar_t r_lightmap_packbytexture;
 extern cvar_t r_telesplash;
 extern cvar_t r_shaftalpha;
 
@@ -702,7 +863,6 @@ extern cvar_t r_powerupglow;
 
 // FIXME, allocate dynamically
 extern	centity_t        cl_entities[CL_MAX_EDICTS];
-extern	efrag_t          cl_efrags[MAX_EFRAGS];
 extern	entity_t         cl_static_entities[MAX_STATIC_ENTITIES];
 extern	lightstyle_t     cl_lightstyle[MAX_LIGHTSTYLES];
 extern	dlight_t         cl_dlights[MAX_DLIGHTS];
@@ -759,6 +919,7 @@ void Demo_AdjustSpeed(void);
 qbool CL_IsDemoExtension(const char *filename);
 qbool CL_Demo_SkipMessage(qbool skip_if_seeking);
 qbool CL_Demo_NotForTrackedPlayer(void);
+qbool CL_DemoExtensionMatch(const char* path);
 
 void CL_AutoRecord_StopMatch(void);
 void CL_AutoRecord_CancelMatch(void);
@@ -766,6 +927,9 @@ void CL_AutoRecord_StartMatch(char *demoname);
 qbool CL_AutoRecord_Status(void);
 void CL_AutoRecord_SaveMatch(void);
 
+qbool SCR_QTVBufferToBeDrawn(int options);
+int Demo_BufferSize(int* ms);
+
 extern double demostarttime;
 extern double nextdemotime, olddemotime;
 
@@ -870,7 +1034,8 @@ void CL_InitTEntsCvar(void);
 void CL_ClearTEnts (void);
 void CL_ParseTEnt (void);
 void CL_ExplosionSprite (vec3_t);
-void CL_UpdateTEnts (void);
+void CL_ClearPlayerBeams(void);
+void CL_UpdateTEnts(void);
 
 // cl_ents.c
 typedef enum cl_modelindex_s {
@@ -886,12 +1051,11 @@ typedef enum cl_modelindex_s {
 
 extern int cl_modelindices[cl_num_modelindices];
 extern char *cl_modelnames[cl_num_modelindices];
-extern struct model_s *cl_flame0_model;
 
 void CL_InitEnts(void);
 void CL_AddEntity (entity_t *ent);
 void CL_ClearScene (void) ;
-void CL_AddParticleTrail(entity_t* ent, centity_t* cent, vec3_t* old_origin, customlight_t* cst_lt, entity_state_t *state);
+void CL_AddParticleTrail(entity_t* ent, centity_t* cent, customlight_t* cst_lt, entity_state_t *state);
 
 dlighttype_t dlightColor(float f, dlighttype_t def, qbool random);
 customlight_t *dlightColorEx(float f, char *str, dlighttype_t def, qbool random, customlight_t *l);
@@ -908,6 +1072,7 @@ void CL_ClearProjectiles (void);
 void CL_ParsePacketEntities (qbool delta);
 void CL_SetSolidEntities (void);
 void CL_ParsePlayerinfo (void);
+void CL_StorePausePredictionLocations(void);
 
 
 void MVD_Interpolate(void);
@@ -919,15 +1084,13 @@ void CL_ParseProjectiles(qbool indexed);
 void CL_InitPrediction(void);
 void CL_PredictMove(qbool physframe);
 void CL_PredictUsercmd(player_state_t *from, player_state_t *to, usercmd_t *u);
+void CL_DisableLerpMove(void);
 
 // cl_cam.c
 void vectoangles(vec3_t vec, vec3_t ang);
 #define CAM_NONE	0
 #define CAM_TRACK	1
 
-extern int	autocam;
-extern int	spec_track; // player# of who we are tracking
-
 int WhoIsSpectated (void);
 void CL_Cam_SetKiller(int killernum, int victimnum);
 void Cam_Angles_Set(float pitch, float yaw, float roll);
@@ -946,13 +1109,15 @@ int Cam_TrackNum(void);
 void Cam_Lock(int playernum);
 
 // skin.c
-void	Skin_Find (player_info_t *sc);
-char	*Skin_FindName (player_info_t *sc);
-byte	*Skin_Cache (skin_t *skin, qbool no_baseskin);
-void	Skin_Skins_f (void);
-void	Skin_AllSkins_f (void);
-void	Skin_NextDownload (void);
-void	Skin_ShowSkins_f (void);
+void Skin_Skins_f(void);
+void Skin_Clear(qbool download);
+void Skin_AllSkins_f(void);
+void Skin_NextDownload(void);
+void Skin_ShowSkins_f(void);
+void Skins_PreCache(void);
+void Skin_InvalidateTextures(void);
+void Skin_UserInfoChange(int slot, player_info_t* player, const char* key);
+void Skin_RegisterCvars(void);
 
 // match_tools.c
 void MT_Init(void);
@@ -961,6 +1126,7 @@ void MT_Disconnect(void);
 void MT_NewMap(void);
 char *MT_MatchName(void);
 char *MT_ShortStatus(void);
+void MT_Shutdown(void);
 
 // fragstats.c
 
@@ -969,6 +1135,7 @@ void Stats_Reset(void);
 void Stats_NewMap(void);
 void Stats_EnterSlot(int num);
 void Stats_ParsePrint(char *s, int level, cfrags_format *cff);
+void Stats_Shutdown(void);
 
 qbool Stats_IsActive(void);
 qbool Stats_IsFlagsParsed(void);
@@ -980,7 +1147,8 @@ void Stats_GetFlagStats(int num, int *stats);
 
 void CL_CalcPlayerFPS(player_info_t *info, int msec);
 
-// TODO : These should not be defined here, they should be extern!
+qbool CL_DrawnPlayerPosition(int player_num, float* pos, int* msec);
+
 //
 // Multiview vars
 // ===================================================================================
@@ -997,6 +1165,7 @@ int CL_MultiviewCurrentView (void);
 int CL_MultiviewNextPlayer (void);
 int CL_MultiviewAutotrackSlot (void);
 int CL_MultiviewMainView(void);
+void CL_MultiviewInsetSetScreenCoordinates(int x, int y, int width, int height);
 
 void CL_MultiviewSetTrackSlot (int trackNum, int player);
 void CL_MultiviewResetCvars (void);
@@ -1011,6 +1180,9 @@ void CL_MultiviewDemoFinish (void);
 void CL_MultiviewDemoStartRewind (void);
 void CL_MultiviewDemoStopRewind (void);
 
+// Restore stats for main view hud
+void CL_MultiviewInsetRestoreStats(void);
+
 // Weapons
 centity_t* CL_WeaponModelForView(void);
 
@@ -1018,11 +1190,13 @@ centity_t* CL_WeaponModelForView(void);
 // client side min_ping aka delay
 
 extern cvar_t cl_delay_packet;
+extern cvar_t cl_delay_packet_target;
 extern cvar_t cl_delay_packet_dev;
 
 #define CL_MAX_DELAYED_PACKETS 16 /* 13 * 16 = 208 ms, should be enough */
 #define CL_MAX_PACKET_DELAY 75 /* total delay two times more */
 #define CL_MAX_PACKET_DELAY_DEVIATION 5
+#define CL_MAX_PACKET_DELAY_TARGET 155
 
 typedef struct cl_delayed_packet_s
 {
@@ -1036,6 +1210,7 @@ typedef struct cl_delayed_packet_s
 
 qbool CL_QueInputPacket(void);
 qbool CL_UnqueOutputPacket(qbool sendall);
+void CL_ClearQueuedPackets(void);
 
 // ===================================================================================
 
@@ -1073,8 +1248,8 @@ typedef struct scr_sshot_target_s {
 	byte* buffer;
 	qbool freeMemory;
 	qbool movie_capture;
-	int width;
-	int height;
+	size_t width;
+	size_t height;
 	image_format_t format;
 } scr_sshot_target_t;
 
@@ -1083,8 +1258,26 @@ int SCR_ScreenshotWrite(scr_sshot_target_t* target_params);
 qbool Movie_AnimatedPNG(void);
 
 qbool Movie_BackgroundCapture(scr_sshot_target_t* params);
-byte* Movie_TempBuffer(int width, int height);
+byte* Movie_TempBuffer(size_t width, size_t height);
 qbool Movie_BackgroundInitialise(void);
 void Movie_BackgroundShutdown(void);
 
+void Cache_Flush(void);
+
 #define DEFAULT_CHAT_SOUND "misc/talk.wav"
+
+#ifdef WITH_RENDERING_TRACE
+void Dev_VidFrameStart(void);
+void Dev_VidFrameTrace(void);
+void Dev_VidTextureDump(void);
+void Dev_TextureList(void);
+#endif
+
+// weapons scripts
+int IN_BestWeapon(qbool rendering_only);
+int IN_BestWeaponReal(qbool rendering_only);
+
+// hud_common.c
+void CL_RemovePrefixFromName(int player);
+
+#endif // EZQUAKE_CLIENT_HEADER
