@@ -24,11 +24,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #endif
 
 #include "quakedef.h"
-#ifdef WITH_TCL
-#include "embed_tcl.h"
-#endif
 #include "gl_model.h"
-#include "gl_local.h"
 #include "teamplay.h"
 #include "rulesets.h"
 #include "tp_triggers.h"
@@ -36,6 +32,20 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "utils.h"
 #include "keys.h"
 
+typedef struct {
+	char name[MAX_MACRO_NAME];
+	char *(*func) (void);
+	int teamplay;
+} macro_command_t;
+
+#define MACRO_DEF(x) { #x, NULL, 0 }
+
+static macro_command_t macro_commands[num_macros] = {
+#include "macro_ids.h"
+};
+
+#undef MACRO_DEF
+
 qbool CL_CheckServerCommand (void);
 
 static void Cmd_ExecuteStringEx (cbuf_t *context, char *text);
@@ -51,7 +61,6 @@ cbuf_t cbuf_svc;
 cbuf_t cbuf_safe, cbuf_formatted_comms;
 cbuf_t cbuf_server;
 
-char *hud262_load_buff = NULL;
 cbuf_t *cbuf_current = NULL;
 
 //=============================================================================
@@ -60,40 +69,12 @@ cbuf_t *cbuf_current = NULL;
 //This allows commands like: bind g "impulse 5 ; +attack ; wait ; -attack ; impulse 2"
 void Cmd_Wait_f (void)
 {
-#ifdef WITH_TCL
-	if (in_tcl) {
-			Com_Printf ("command wait cant be used with TCL\n");
-			return;
-	}
-#endif
 	if (cbuf_current)
 		cbuf_current->wait = true;
 
 	return;
 }
 
-void Hud262_CatchStringsOnLoad(char *line)
-{
-	char *tmpbuff;
-
-	if (Utils_RegExpMatch("^((\\s+)?(?i)hud262_(add|alpha|bg|blink|disable|enable|position|width))", line))
-	{
-		if (hud262_load_buff == NULL)
-		{
-			hud262_load_buff = (char*) Q_malloc( (strlen(line) + 2) * sizeof(char));
-			snprintf(hud262_load_buff, strlen(line) + 2, "%s\n", line);
-		}
-		else
-		{
-			tmpbuff = (char *) Q_malloc(strlen(hud262_load_buff) + 1);
-			strcpy(tmpbuff, hud262_load_buff);
-			hud262_load_buff = (char *) Q_realloc(hud262_load_buff, (strlen(tmpbuff) + strlen(line) + 2) * sizeof(char));
-			snprintf(hud262_load_buff, strlen(tmpbuff) + strlen(line) + 2, "%s%s\n", tmpbuff, line);
-			Q_free(tmpbuff);
-		}
-	}
-}
-
 /*
 =============================================================================
 						COMMAND BUFFER
@@ -118,6 +99,12 @@ void Cbuf_Execute (void)
 	Cbuf_ExecuteEx (&cbuf_server);
 }
 
+void Cbuf_Flush(cbuf_t* cbuf)
+{
+	while (cbuf->text_end > cbuf->text_start)
+		Cbuf_ExecuteEx(cbuf);
+}
+
 //fuh : ideally we should have 'cbuf_t *Cbuf_Register(int maxsize, int flags, qbool (*blockcmd)(void))
 //fuh : so that cbuf_svc and cbuf_safe can be registered outside cmd.c in cl_* .c
 //fuh : flags can be used to deal with newline termination etc for cbuf_svc, and *blockcmd can be used for blocking cmd's for cbuf_svc
@@ -145,7 +132,7 @@ void Cbuf_Init (void)
 //Adds command text at the end of the buffer
 void Cbuf_AddTextEx (cbuf_t *cbuf, const char *text)
 {
-	int new_start, new_bufsize;
+	size_t new_start, new_bufsize;
 	size_t len;
 
 	len = strlen (text);
@@ -174,7 +161,7 @@ void Cbuf_AddTextEx (cbuf_t *cbuf, const char *text)
 //Adds command text at the beginning of the buffer
 void Cbuf_InsertTextEx (cbuf_t *cbuf, const char *text)
 {
-	int new_start, new_bufsize;
+	size_t new_start, new_bufsize;
 	size_t len;
 
 	len = strlen (text);
@@ -204,7 +191,8 @@ void Cbuf_InsertTextEx (cbuf_t *cbuf, const char *text)
 
 void Cbuf_ExecuteEx (cbuf_t *cbuf)
 {
-	int i, j, cursize, nextsize;
+	size_t i, j;
+	size_t cursize, nextsize;
 	char *text, line[1024], *src, *dest;
 	qbool comment;
 	int quotes;
@@ -313,8 +301,9 @@ void Cbuf_ExecuteEx (cbuf_t *cbuf)
 		// (some cvars are not created/initialized at the time when we want to use them in hud262)
 		// we should save these commands to buffer and execute it when all
 		// cvars will be created
-		if(!host_initialized)
-			Hud262_CatchStringsOnLoad(line);
+		if (!host_initialized) {
+			Hud_262CatchStringsOnLoad(line);
+		}
 
 		Cmd_ExecuteStringEx (cbuf, line);	// execute the command line
 
@@ -461,7 +450,6 @@ void Cmd_Exec_f (void)
 {
 	char *f, name[MAX_OSPATH];
 	char reset_bindphysical[128];
-	int mark;
 	qbool server_command = false;
 
 	if (Cmd_Argc () != 2) {
@@ -474,14 +462,13 @@ void Cmd_Exec_f (void)
 #endif
 
 	strlcpy (name, Cmd_Argv(1), sizeof(name) - 4);
-	mark = Hunk_LowMark();
-	if (!(f = (char *) FS_LoadHunkFile (name, NULL)))	{
+	if (!(f = (char *) FS_LoadHeapFile(name, NULL)))	{
 		const char *p;
 		p = COM_SkipPath (name);
 		if (!strchr (p, '.')) {
 			// no extension, so try the default (.cfg)
 			strlcat (name, ".cfg", sizeof (name));
-			f = (char *) FS_LoadHunkFile (name, NULL);
+			f = (char *)FS_LoadHeapFile(name, NULL);
 		}
 		if (!f) {
 			Com_Printf ("couldn't exec %s\n", Cmd_Argv(1));
@@ -509,7 +496,7 @@ void Cmd_Exec_f (void)
 		Cbuf_InsertTextEx(&cbuf_main, "con_bindphysical 1\n");
 	}
 	
-	Hunk_FreeToLowMark (mark);
+	Q_free(f);
 }
 
 //Just prints the rest of the line to the console
@@ -577,11 +564,9 @@ char *Cmd_AliasString (char *name)
 
 	key = Com_HashKey (name) % ALIAS_HASHPOOL_SIZE;
 	for (alias = cmd_alias_hash[key]; alias; alias = alias->hash_next) {
-		if (!strcasecmp(name, alias->name))
-#ifdef WITH_TCL
-			if (!(alias->flags & ALIAS_TCL))
-#endif
-				return alias->value;
+		if (!strcasecmp(name, alias->name)) {
+			return alias->value;
+		}
 	}
 	return NULL;
 }
@@ -600,37 +585,27 @@ void Cmd_Viewalias_f (void)
 	for (i=1; i<Cmd_Argc(); i++) {
 		name = Cmd_Argv(i);
 
-
-		if ( IsRegexp(name) ) {
-			if (!ReSearchInit(name))
+		if (IsRegexp(name)) {
+			if (!ReSearchInitEx(name, false)) {
 				return;
+			}
 			Com_Printf ("Current alias commands:\n");
 
-			for (alias = cmd_alias, i=m=0; alias ; alias=alias->next, i++)
+			for (alias = cmd_alias, i = m = 0; alias; alias = alias->next, i++) {
 				if (ReSearchMatch(alias->name)) {
-#ifdef WITH_TCL
-					if (alias->flags & ALIAS_TCL)
-						Com_Printf ("%s : Tcl procedure\n", alias->name);
-					else
-#endif
-						Com_Printf ("%s : %s\n", alias->name, alias->value);
+					Com_Printf("%s : %s\n", alias->name, alias->value);
 					m++;
 				}
+			}
 
 			Com_Printf ("------------\n%i/%i aliases\n", m, i);
 			ReSearchDone();
-
-
-		} else 	{
-			if ((alias = Cmd_FindAlias(name)))
-#ifdef WITH_TCL
-				if (alias->flags & ALIAS_TCL)
-					Com_Printf ("%s : Tcl procedure\n", name);
-				else
-#endif
-					Com_Printf ("%s : \"%s\"\n", Cmd_Argv(i), alias->value);
-			else
-				Com_Printf ("No such alias: %s\n", Cmd_Argv(i));
+		}
+		else if ((alias = Cmd_FindAlias(name))) {
+			Com_Printf("%s : \"%s\"\n", Cmd_Argv(i), alias->value);
+		}
+		else {
+			Com_Printf("No such alias: %s\n", Cmd_Argv(i));
 		}
 	}
 }
@@ -679,9 +654,9 @@ void Cmd_AliasList_f (void)
 		assert(!"count == MAX_SORTED_ALIASES");
 
 	c = Cmd_Argc();
-	if (c>1)
-		if (!ReSearchInit(Cmd_Argv(1)))
-			return;
+	if (c > 1 && !ReSearchInitEx(Cmd_Argv(1), false)) {
+		return;
+	}
 
 	Com_Printf ("List of aliases:\n");
 	for (i = 0; i < count; i++) {
@@ -870,9 +845,11 @@ void Cmd_UnAlias (qbool use_regex)
 	for (i=1; i<Cmd_Argc(); i++) {
 		name = Cmd_Argv(i);
 
-		if (use_regex && (re_search = IsRegexp(name)))
-			if(!ReSearchInit(name))
+		if (use_regex && (re_search = IsRegexp(name))) {
+			if (!ReSearchInitEx(name, false)) {
 				continue;
+			}
+		}
 
 		if (strlen(name) >= MAX_ALIAS_NAME) {
 			Com_Printf ("Alias name is too long: \"%s\"\n", Cmd_Argv(i));
@@ -957,15 +934,9 @@ void DeleteServerAliases(void)
 =============================================================================
 */
 
-typedef struct legacycmd_s
-{
-	char *oldname, *newname;
-	struct legacycmd_s *next;
-} legacycmd_t;
+legacycmd_t *legacycmds = NULL;
 
-static legacycmd_t *legacycmds = NULL;
-
-void Cmd_AddLegacyCommand (char *oldname, char *newname)
+void Cmd_AddLegacyCommand(char *oldname, char *newname)
 {
 	legacycmd_t *cmd;
 	cmd = (legacycmd_t *) Q_malloc(sizeof(legacycmd_t));
@@ -974,6 +945,7 @@ void Cmd_AddLegacyCommand (char *oldname, char *newname)
 
 	cmd->oldname = oldname;
 	cmd->newname = newname;
+	cmd->dummy_cmd.name = oldname;
 }
 
 qbool Cmd_IsLegacyCommand (char *oldname)
@@ -1075,7 +1047,7 @@ char *Cmd_MakeArgsEx (tokenizecontext_t *ctx, int start)
 }
 
 // Parses the given string into command line tokens.
-void Cmd_TokenizeStringEx (tokenizecontext_t *ctx, char *text)
+void Cmd_TokenizeStringEx2(tokenizecontext_t *ctx, const char *text, qbool curlybraces)
 {
 	int idx = 0, token_len;
 
@@ -1097,7 +1069,7 @@ void Cmd_TokenizeStringEx (tokenizecontext_t *ctx, char *text)
 		if (ctx->cmd_argc == 1)
 			strlcpy(ctx->cmd_args, text, sizeof(ctx->cmd_args));
 
-		text = COM_Parse (text);
+		text = COM_ParseEx(text, curlybraces);
 		if (!text)
 			return;
 
@@ -1118,6 +1090,11 @@ void Cmd_TokenizeStringEx (tokenizecontext_t *ctx, char *text)
 	}
 }
 
+void Cmd_TokenizeStringEx(tokenizecontext_t* ctx, const char* text)
+{
+	Cmd_TokenizeStringEx2(ctx, text, false);
+}
+
 // and wrappers for backward compatibility
 
 int Cmd_Argc (void)
@@ -1144,7 +1121,7 @@ char *Cmd_MakeArgs (int start)
 }
 
 //Parses the given string into command line tokens.
-void Cmd_TokenizeString (char *text)
+void Cmd_TokenizeString (const char *text)
 {
 	Cmd_TokenizeStringEx(&cmd_tokenizecontext, text);
 }
@@ -1216,8 +1193,8 @@ qbool Cmd_AddRemCommand (char *cmd_name, xcommand_t function)
 	cmd->name = (char*)(cmd+1); // points to extra space created after the structure
 	strcpy(cmd->name, cmd_name);
 	cmd->function = function;
-	cmd->next = cmd_functions;
 	cmd->zmalloced = true;
+	cmd->next = cmd_functions;
 	cmd_functions = cmd;
 	cmd->hash_next = cmd_hash_array[key];
 	cmd_hash_array[key] = cmd;
@@ -1324,6 +1301,7 @@ char *Cmd_CompleteCommand (char *partial)
 	cmd_function_t *cmd;
 	int len;
 	cmd_alias_t *alias;
+	legacycmd_t* legacy_cmd;
 
 	len = strlen(partial);
 
@@ -1337,6 +1315,11 @@ char *Cmd_CompleteCommand (char *partial)
 	for (alias = cmd_alias; alias; alias = alias->next)
 		if (!strcasecmp (partial, alias->name))
 			return alias->name;
+	for (legacy_cmd = legacycmds; legacy_cmd; legacy_cmd = legacy_cmd->next) {
+		if (!strcasecmp(partial, legacy_cmd->oldname)) {
+			return legacy_cmd->oldname;
+		}
+	}
 
 	// check for partial match
 	for (cmd = cmd_functions; cmd; cmd = cmd->next)
@@ -1345,6 +1328,11 @@ char *Cmd_CompleteCommand (char *partial)
 	for (alias = cmd_alias; alias; alias = alias->next)
 		if (!strncasecmp (partial, alias->name, len))
 			return alias->name;
+	for (legacy_cmd = legacycmds; legacy_cmd; legacy_cmd = legacy_cmd->next) {
+		if (!strncasecmp(partial, legacy_cmd->oldname, len)) {
+			return legacy_cmd->oldname;
+		}
+	}
 
 	return NULL;
 }
@@ -1352,15 +1340,25 @@ char *Cmd_CompleteCommand (char *partial)
 int Cmd_CompleteCountPossible (char *partial)
 {
 	cmd_function_t *cmd;
+	legacycmd_t* legacy_cmd;
 	int len, c = 0;
 
 	len = strlen(partial);
 	if (!len)
 		return 0;
 
-	for (cmd = cmd_functions; cmd; cmd = cmd->next)
-		if (!strncasecmp (partial, cmd->name, len))
+	for (cmd = cmd_functions; cmd; cmd = cmd->next) {
+		if (!strncasecmp(partial, cmd->name, len)) {
 			c++;
+		}
+	}
+
+	// Also check legacy commands
+	for (legacy_cmd = legacycmds; legacy_cmd; legacy_cmd = legacy_cmd->next) {
+		if (!strncasecmp(partial, legacy_cmd->oldname, len)) {
+			++c;
+		}
+	}
 
 	return c;
 }
@@ -1404,9 +1402,11 @@ void Cmd_CmdList (qbool use_regex)
 
 	pattern = (Cmd_Argc() > 1) ? Cmd_Argv(1) : NULL;
 
-	if (((c = Cmd_Argc()) > 1) && use_regex)
-		if (!ReSearchInit(Cmd_Argv(1)))
+	if (((c = Cmd_Argc()) > 1) && use_regex) {
+		if (!ReSearchInitEx(Cmd_Argv(1), false)) {
 			return;
+		}
+	}
 
 	Com_Printf ("List of commands:\n");
 	for (i = 0; i < count; i++) {
@@ -1439,63 +1439,61 @@ void Cmd_CmdList_re_f (void)
 	Cmd_CmdList (true);
 }
 
-#define MAX_MACROS 70
-
-static macro_command_t macro_commands[MAX_MACROS];
-static int macro_count = 0;
-
-void Cmd_ReInitAllMacro (void)
+void Cmd_ReInitAllMacro(void)
 {
 	int i;
-	int teamplay;
+	int teamplay = (int)Rulesets_RestrictTriggers();
 
-	teamplay = (int) Rulesets_RestrictTriggers ();
-
-	for (i = 0; i < macro_count; i++)
-		if (macro_commands[i].teamplay != MACRO_NORULES)
+	for (i = 0; i < num_macros; i++) {
+		if (macro_commands[i].teamplay != MACRO_NORULES) {
 			macro_commands[i].teamplay = teamplay;
+		}
+	}
 }
 
-void Cmd_AddMacroEx (const char *s, char *(*f) (void), int teamplay)
+void Cmd_AddMacroEx(macro_id id, char *(*f) (void), int teamplay)
 {
-	if (macro_count == MAX_MACROS)
-		Sys_Error ("Cmd_AddMacro: macro_count == MAX_MACROS");
+	if (id < 0 || id >= num_macros) {
+		return;
+	}
 
-	snprintf (macro_commands[macro_count].name, sizeof (macro_commands[macro_count].name), "%s", s);
-	macro_commands[macro_count].func = f;
-	macro_commands[macro_count].teamplay = teamplay;
-
-#ifdef WITH_TCL
-// disconnect: it seems macro are safe with TCL NOW
-//	if (!teamplay)	// don't allow teamplay protected macros since there's no protection for this in TCL yet
-		TCL_RegisterMacro (macro_commands + macro_count);
-#endif
-
-	macro_count++;
+	macro_commands[id].func = f;
+	macro_commands[id].teamplay = teamplay;
 }
 
-void Cmd_AddMacro (const char *s, char *(*f) (void))
+void Cmd_AddMacro (macro_id id, char *(*f) (void))
 {
-	Cmd_AddMacroEx (s, f, MACRO_NORULES);
+	Cmd_AddMacroEx(id, f, MACRO_NORULES);
 }
 
-char *Cmd_MacroString (const char *s, int *macro_length)
+char *Cmd_MacroString (const char* s, int *macro_length)
 {
 	int i;
 	macro_command_t	*macro;
-
-	for (i = 0; i < macro_count; i++) {
-		macro = &macro_commands[i];
-		if (!strncasecmp (s, macro->name, strlen (macro->name))) {
-			if (cbuf_current == &cbuf_main && (macro->teamplay == MACRO_DISALLOWED))
-				cbuf_current = &cbuf_formatted_comms;
-			*macro_length = strlen (macro->name);
-			return macro->func();
-		}
-		macro++;
-	}
+	int best = -1;
+	int best_length = -1;
 
 	*macro_length = 0;
+	for (i = 0; i < num_macros; i++) {
+		macro = &macro_commands[i];
+		if (macro->func) {
+			int name_length = strlen(macro->name);
+			if (!strncasecmp(s, macro->name, name_length)) {
+				if (best_length == -1 || best_length < name_length) {
+					best = i;
+					best_length = name_length;
+				}
+			}
+		}
+	}
+
+	if (best >= 0) {
+		if (cbuf_current == &cbuf_main && (macro_commands[best].teamplay == MACRO_DISALLOWED)) {
+			cbuf_current = &cbuf_formatted_comms;
+		}
+		*macro_length = best_length;
+		return macro_commands[best].func();
+	}
 
 	return NULL;
 }
@@ -1505,35 +1503,44 @@ static int Cmd_MacroCompare (const void *p1, const void *p2)
 	return strcmp ((*((macro_command_t **) p1))->name, (*((macro_command_t **) p2))->name);
 }
 
+const char* Cmd_MacroName(macro_id id)
+{
+	return macro_commands[id].name;
+}
+
+qbool Cmd_MacroTeamplayRestricted(macro_id id)
+{
+	return macro_commands[id].teamplay;
+}
+
 void Cmd_MacroList_f (void)
 {
 	int i, c, m = 0;
-	static macro_command_t *sorted_macros[MAX_MACROS];
+	static macro_command_t* sorted_macros[num_macros];
 
-	for (i = 0; i < macro_count; i++)
+	for (i = 0; i < num_macros; i++) {
 		sorted_macros[i] = &macro_commands[i];
-	qsort (sorted_macros, macro_count, sizeof (macro_command_t *), Cmd_MacroCompare);
-
-	if (macro_count == MAX_MACROS)
-		assert(!"count == MAX_MACROS");
+	}
+	qsort(sorted_macros, num_macros, sizeof (macro_command_t *), Cmd_MacroCompare);
 
 	c = Cmd_Argc();
-	if (c > 1)
-		if (!ReSearchInit (Cmd_Argv (1)))
-			return;
+	if (c > 1 && !ReSearchInitEx(Cmd_Argv(1), false)) {
+		return;
+	}
 
 	Com_Printf ("List of macros:\n");
-	for (i = 0; i < macro_count; i++) {
+	for (i = 0; i < num_macros; i++) {
 		if (c==1 || ReSearchMatch (sorted_macros[i]->name)) {
 			Com_Printf ("$%s\n", sorted_macros[i]->name);
 			m++;
 		}
 	}
 
-	if (c > 1)
+	if (c > 1) {
 		ReSearchDone();
+	}
 
-	Com_Printf ("------------\n%i/%i macros\n", m, macro_count);
+	Com_Printf ("------------\n%i/%i macros\n", m, num_macros);
 }
 
 void TP_SetDefaultMacroFormat(char* cvar_lookup, int* fixed_width, int* alignment);
@@ -1723,8 +1730,8 @@ static void Cmd_ExecuteStringEx (cbuf_t *context, char *text)
 	oldcontext = cbuf_current;
 	cbuf_current = context;
 
-	Cmd_ExpandString (text, text_exp);
-	Cmd_TokenizeString (text_exp);
+	Cmd_ExpandString(text, text_exp);
+	Cmd_TokenizeStringEx2(&cmd_tokenizecontext, text_exp, cl_curlybraces.integer);
 
 	if (!Cmd_Argc())
 		goto done; // no tokens
@@ -1734,6 +1741,13 @@ static void Cmd_ExecuteStringEx (cbuf_t *context, char *text)
 			goto done;
 	}
 
+#ifndef CLIENTONLY
+	// 'status' on remote ktx servers..
+	if (!strcmp(Cmd_Argv(0), "status") && Cmd_Argc() == 1 && Cmd_FindAlias("status")) {
+		goto checkaliases;
+	}
+#endif
+
 	// check functions
 	if ((cmd = Cmd_FindCommand(Cmd_Argv(0)))) {
 		if (gtf || cbuf_current == &cbuf_safe) {
@@ -1774,15 +1788,9 @@ checkaliases:
 	if ((a = Cmd_FindAlias(Cmd_Argv(0)))) {
 
 		// QW262 -->
-#ifdef WITH_TCL
-		if (a->flags & ALIAS_TCL)
-		{
-			TCL_ExecuteAlias (a);
-			goto done;
+		if (a->value[0] == '\0') {
+			goto done; // alias is empty.
 		}
-#endif
-
-		if (a->value[0]=='\0') goto done; // alias is empty.
 
 		if(a->flags & ALIAS_HAS_PARAMETERS) { // %parameters are given in alias definition
 			s=a->value;
@@ -2343,16 +2351,6 @@ void Cmd_Cvar_Out_f (void)
 // <-- QW262
 
 
-
-/*
-============
-Cmd_Shutdown
-============
-*/
-void Cmd_Shutdown(void)
-{
-}
-
 void Cmd_Init (void)
 {
 	// register our commands
@@ -2391,3 +2389,41 @@ void Cmd_Init (void)
 	      sizeof(msgtrigger_commands[0]),Commands_Compare_Func);
 }
 
+void Cmd_Shutdown(void)
+{
+	int i;
+	cmd_alias_t* alias;
+	cmd_alias_t* next_alias;
+	cmd_function_t* cmd;
+	cmd_function_t* next_cmd;
+	legacycmd_t* legacycmd;
+	legacycmd_t* next_legacycmd;
+
+	Sys_Printf("Cmd_Shutdown(aliases)\n");
+	for (i = 0; i < sizeof(cmd_alias_hash) / sizeof(cmd_alias_hash[0]); ++i) {
+		cmd_alias_hash[i] = NULL;
+	}
+
+	for (alias = cmd_alias; alias; alias = next_alias) {
+		next_alias = alias->next;
+		Q_free(alias->value);
+		Q_free(alias);
+	}
+	cmd_alias = NULL;
+
+	Sys_Printf("Cmd_Shutdown(functions)\n");
+	for (cmd = cmd_functions; cmd; cmd = next_cmd) {
+		next_cmd = cmd->next;
+
+		if (cmd->zmalloced) {
+			Q_free(cmd);
+		}
+	}
+	cmd_functions = NULL;
+
+	Sys_Printf("Cmd_Shutdown(legacy)\n");
+	for (legacycmd = legacycmds; legacycmd; legacycmd = next_legacycmd) {
+		next_legacycmd = legacycmd->next;
+		Q_free(legacycmd);
+	}
+}
