@@ -28,7 +28,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <stdio.h>
 #endif
 #include "gl_model.h"
-#include "gl_local.h"
 #if defined(_WIN32) || defined(__linux__) || defined(__FreeBSD__)
 #include "tr_types.h"
 #endif // _WIN32 || __linux__ || __FreeBSD__
@@ -38,7 +37,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "utils.h"
 #include "localtime.h"
 #include "irc.h"
-
+#include "fonts.h"
+#include "rulesets.h"
+#include "menu.h"
 
 #define     MINIMUM_CONBUFSIZE     (1 << 15)
 #define     DEFAULT_CONBUFSIZE     (1 << 16)
@@ -77,6 +78,7 @@ cvar_t      con_timestamps          = {"con_timestamps", "0"};
 cvar_t      con_shift               = {"con_shift", "-10"};
 cvar_t      cl_textEncoding         = {"cl_textencoding", "0"};
 cvar_t      con_mm2_only            = {"con_mm2_only", "0"};
+cvar_t      con_proportional        = {"con_proportional", "0"};
 
 #ifdef _WIN32
 cvar_t      con_toggle_deadkey      = {"con_deadkey", "1"};
@@ -132,6 +134,14 @@ void Date_f (void)
     con_ormask = 0;
 }
 
+static qbool Con_MenuWillDrawConsole(void)
+{
+	if (m_state == m_none || key_dest != key_menu || m_state == m_proxy) {
+		return false;
+	}
+	return (SCR_NEED_CONSOLE_BACKGROUND);
+}
+
 void MakeStringRed(char *s)
 {
     while (*s)
@@ -424,14 +434,14 @@ void Con_Init (void) {
 	int i, conbufsize;
 
 	//Tei: moved there, because on windows can't capture the output for debug purposes
-	if (!qconsole_log && COM_CheckParm("-condebug")) {
+	if (!qconsole_log && COM_CheckParm(cmdline_param_console_debug)) {
 		char tmp_path[MAX_OSPATH] = {0};
 
 		snprintf(&tmp_path[0], sizeof(tmp_path), "%s/qw/qconsole.log", com_basedir);
 		qconsole_log = fopen(tmp_path, "a");
 	}
 
-	if ((i = COM_CheckParm("-conbufsize")) && i + 1 < COM_Argc()) {
+	if ((i = COM_CheckParm(cmdline_param_console_buffersize)) && i + 1 < COM_Argc()) {
 		conbufsize = Q_atoi(COM_Argv(i + 1)) << 10;
 		conbufsize = bound (MINIMUM_CONBUFSIZE , conbufsize, MAXIMUM_CONBUFSIZE);
 	} else {
@@ -466,6 +476,7 @@ void Con_Init (void) {
 	Cvar_Register (&con_timestamps); 
 	Cvar_Register (&con_shift); 
 	Cvar_Register (&con_mm2_only);
+	Cvar_Register (&con_proportional);
 
 	Cvar_ResetCurrentGroup();
 
@@ -479,15 +490,6 @@ void Con_Init (void) {
 	Cvar_Register (&con_sound_mm2_volume); 
 	Cvar_Register (&con_sound_spec_volume); 
 	Cvar_Register (&con_sound_other_volume); 
-	// if you don't like renaming things in this way, let's have some talk with tea - johnnycz
-	Cmd_AddLegacyCommand("con_sound_mm1_file", "s_mm1_file");
-	Cmd_AddLegacyCommand("con_sound_mm2_file", "s_mm2_file");
-	Cmd_AddLegacyCommand("con_sound_spec_file", "s_spec_file");
-	Cmd_AddLegacyCommand("con_sound_other_file", "s_otherchat_file");
-	Cmd_AddLegacyCommand("con_sound_mm1_volume", "s_mm1_volume");
-	Cmd_AddLegacyCommand("con_sound_mm2_volume", "s_mm2_volume");
-	Cmd_AddLegacyCommand("con_sound_spec_volume", "s_spec_volume");
-	Cmd_AddLegacyCommand("con_sound_other_volume", "s_otherchat_volume");
 
     Cvar_ResetCurrentGroup();
 
@@ -556,27 +558,29 @@ void Con_SafePrintf (char *fmt, ...)
 }
 
 //Handles cursor positioning, line wrapping, etc
-void Con_PrintW (wchar *txt) {
+void Con_PrintW(wchar *txt)
+{
 	int y, c, l, d, mask, color = COLOR_WHITE, r, g, b, idx;
 	wchar *s;
 	static int cr;
 
 	if (!(Print_flags[Print_current] & PR_LOG_SKIP)) {
-		if (qconsole_log) {
-			char *tempbuf = wcs2str_malloc(txt);
+		if (Ruleset_CanLogConsole() && qconsole_log) {
+			char *tempbuf = Q_wcs2str_malloc(txt);
 			fprintf(qconsole_log, "%s", tempbuf);
 			fflush(qconsole_log);
 			Q_free(tempbuf);
 		}
 		if (Log_IsLogging()) {
-			if (log_readable.value) {
-				char *s, *tempbuf = wcs2str_malloc(txt);
+			if (log_readable.integer) {
+				char *s, *tempbuf = Q_wcs2str_malloc(txt);
 				for (s = tempbuf; *s; s++)
-					*s = readableChars[(unsigned char) *s];
+					*s = readableChars[(unsigned char)*s];
 				Log_Write(tempbuf);
 				Q_free(tempbuf);
-			} else {
-				Log_Write(wcs2str(txt));	
+			}
+			else {
+				Log_Write(wcs2str(txt));
 			}
 		}
 	}
@@ -587,10 +591,11 @@ void Con_PrintW (wchar *txt) {
 	if (!con_initialized || con_suppress)
 		goto zomfg;
 
-	if (txt[0] == 1 || txt[0] == 2)	{
+	if (txt[0] == 1 || txt[0] == 2) {
 		mask = 128;		// go to colored text
 		txt++;
-	} else {
+	}
+	else {
 		mask = 0;
 	}
 
@@ -606,7 +611,8 @@ void Con_PrintW (wchar *txt) {
 					txt += 5;
 					continue; // we got color, get now normal char
 				}
-            } else if (txt[1] == 'r') {
+			}
+			else if (txt[1] == 'r') {
 				color = COLOR_WHITE;
 				txt += 2;
 				continue; // we got color, get now normal char
@@ -625,16 +631,17 @@ void Con_PrintW (wchar *txt) {
 						s += 5;
 						continue; // we got color, get now normal char
 					}
-                } else if (s[1] == 'r') {
+				}
+				else if (s[1] == 'r') {
 					s += 2;
 					continue; // we got color, get now normal char
 				}
 			}
-		
+
 			d = (s[0] & ~128);
-			if (   ( con_wordwrap.value && (!d || d == 0x09 || d == 0x0D || d == 0x0A || d == 0x20))
+			if ((con_wordwrap.value && (!d || d == 0x09 || d == 0x0D || d == 0x0A || d == 0x20))
 				|| (!con_wordwrap.value && d <= 32) // 32 is a space as well as 0x20
-			   )
+				)
 				break;
 
 			l++; // increase word length
@@ -653,7 +660,7 @@ void Con_PrintW (wchar *txt) {
 		}
 
 		if (!con.x)
-			Con_Linefeed ();
+			Con_Linefeed();
 
 		switch (c) {
 			case '\n':
@@ -666,11 +673,12 @@ void Con_PrintW (wchar *txt) {
 				break;
 
 			default:	// display character and advance
-				if (con.x >= con_linewidth)
-					Con_Linefeed ();
+				if (con.x >= con_linewidth) {
+					Con_Linefeed();
+				}
 				y = con.current % con_totallines;
 				idx = y * con_linewidth + con.x;
-				con.text[idx] = c | (c <= 0x7F ? mask | con_ormask : 0);	// only apply mask if in 'standard' charset
+				con.text[idx] = c | (c > 0 && c <= 0x7F ? (mask | con_ormask) : 0);	// only apply mask if in 'standard' charset
 				memset(&con.clr[idx], 0, sizeof(clrinfo_t)); // zeroing whole struct
 				con.clr[idx].c = color;
 				con.clr[idx].i = idx; // no, that not stupid :P
@@ -679,7 +687,9 @@ void Con_PrintW (wchar *txt) {
 		}
 	}
 zomfg:
-	Print_flags[Print_current] = 0;
+	if (!(Print_flags[Print_current] & PR_NORESET)) {
+		Print_flags[Print_current] = 0;
+	}
 }
 
 /*
@@ -694,8 +704,9 @@ static void Con_DrawInput(void) {
 	wchar	*text;
 	wchar	temp[MAXCMDLINE + 1];       //+ 1 for cursor if stlen(key_lines[edit_line]) == 255
 
-	if (key_dest != key_console && cls.state == ca_active)
+	if (key_dest != key_console && cls.state == ca_active) {
 		return;
+	}
 
 	qwcslcpy (temp, key_lines[edit_line], MAXCMDLINE);
 	len = qwcslen(temp);
@@ -703,20 +714,23 @@ static void Con_DrawInput(void) {
 	text = temp;
 
 	// fill out remainder with spaces
-	for (i = 0; i < MAXCMDLINE - len; i++)
+	for (i = 0; i < MAXCMDLINE - len; i++) {
 		(text + len)[i] = ' ';
+	}
 	
 	text[MAXCMDLINE] = 0;
 
 	// add the cursor frame
-	if ( (int)(curtime*con_cursorspeed) & 1 )
+	if ((int)(curtime*con_cursorspeed) & 1) {
 		text[key_linepos] = con_redchars ? (11 + 128) : 11;
+	}
 
 	//	prestep if horizontally scrolling
-	if (key_linepos >= con_linewidth)
+	if (key_linepos >= con_linewidth) {
 		text += 1 + key_linepos - con_linewidth;
+	}
 
-	Draw_StringW (8, con_vislines-22 + bound(0, con_shift.value, 8), text);
+	Draw_ConsoleString(8, con_vislines - 22 + bound(0, con_shift.value, 8), text, NULL, 0, 0, 1, con_proportional.integer);
 }
 
 // Returns first line to start printing from in order to fill up notify area
@@ -739,274 +753,159 @@ static int Con_FirstNotifyLine (int notification_lines, float notification_timel
 	return first_line;
 }
 
-//Draws the last few lines of output transparently over the game top
-void Con_DrawNotify (void) {
-	int x, v, skip = 0, i, idx;
-	wchar *text, *s;
-	wchar buf[1024];
-	clrinfo_t clr[sizeof(buf)];
-	float time;
-	float timeout = bound (0, con_notifytime.value, MAX_NOTIFICATION_TIME);
-	int first_line = Con_FirstNotifyLine(_con_notifylines.integer, timeout);
-
-	if (!con_notify.value)
-		return;
-
-	v = 0;
-	if (_con_notifylines.integer) {
-		for (i = first_line; i <= con.current; i++) {
-			if (i < 0)
-				continue;
-			time = con_times[i % NUM_CON_TIMES];
-			if (time == 0)
-				continue;
-			time = cls.realtime - time;
-			if (time > timeout)
-				continue;
-			idx = (i % con_totallines)*con_linewidth;
-			text = con.text + idx;
-
-			clearnotify = 0;
-			scr_copytop = 1;
-
-			// copy current line to buffer
-			for (x = 0; x < con_linewidth; x++) {
-				buf[x] = text[x];
-				clr[x] = con.clr[idx + x]; // copy whole color struct
-				clr[x].i = x; // set proper index
-			}
-			buf[x] = '\0';
-			Draw_ColoredString3W(8, v + bound(0, con_shift.value, 8), buf, clr, con_linewidth, 0);
-			v += 8;
-		}
-	}
+static qbool Con_NotifyMessageLine(float posX, float posY, float width, float height, float scale, qbool proportional)
+{
+	int x;
 
 	if (key_dest == key_message) {
 		wchar temp[MAXCMDLINE + 1];
+		wchar* s;
+		qbool display_cursor;
 
 		clearnotify = 0;
 		scr_copytop = 1;
 
 		if (chat_team == chat_mm2) {
-			Draw_String (8, v + bound(0, con_shift.value, 8), "say_team:");
-			skip = 11;
+			posX += Draw_SString(posX, posY + bound(0, con_shift.value, 8), "say_team:", scale, proportional);
 		}
 		else if (chat_team == chat_mm1) {
-			Draw_String (8, v + bound(0, con_shift.value, 8), "say:");
-			skip = 5;
+			posX += Draw_SString(posX, posY + bound(0, con_shift.value, 8), "say:", scale, proportional);
 		}
 #ifdef WITH_IRC
 		else if (chat_team == chat_irc) {
 			char dest[256];
-			
+
 			strlcpy(dest, IRC_GetCurrentChan(), sizeof(dest));
 			strlcat(dest, ":", sizeof(dest));
-			skip = strlen(dest) + 1; // is this correct? not sure
 
-			Draw_String (8, v + bound(0, con_shift.value, 8), dest);
+			indent += Draw_SString(posX, v + bound(0, con_shift.value, 8), dest, scale, proportional);
 		}
 #endif
 		else if (chat_team == chat_qtvtogame) {
-			Draw_String (8, v + bound(0, con_shift.value, 8), "say_game:");
-			skip = 11;
+			posX += Draw_SString(posX, posY + bound(0, con_shift.value, 8), "say_game:", scale, proportional);
 		}
 
 		// FIXME: clean this up
-		qwcslcpy (temp, chat_buffer, sizeof(temp) / sizeof(wchar));
+		qwcslcpy(temp, chat_buffer, sizeof(temp) / sizeof(wchar));
 		s = temp;
 
 		// add the cursor frame
-		if ((int) (curtime * con_cursorspeed) & 1) {
-			if (chat_linepos == (int) qwcslen(s))
-				s[chat_linepos+1] = '\0';
-			s[chat_linepos] = 11;
+		display_cursor = (int)(curtime * con_cursorspeed) & 1;
+		if (display_cursor && chat_linepos == (int)qwcslen(s)) {
+			s[chat_linepos + 1] = '\0';
 		}
 
 		// prestep if horizontally scrolling
-		if (chat_linepos + skip >= (vid.width >> 3))
-			s += 1 + chat_linepos + skip - (vid.width >> 3);
+		if (!proportional) {
+			if (chat_linepos * 8 * scale + posX >= width) {
+				s += (int)((1 + chat_linepos)) + (int)((posX - width) / (8 * scale));
+			}
+		}
+		else {
+			// measure full string, cut off characters until string fits
+			float length = Draw_StringLengthW(s, chat_linepos, scale, proportional);
+
+			while (*s && length >= 0 && length >= width - posX) {
+				length -= FontCharacterWidthWide(*s, scale, proportional);
+				++s;
+			}
+		}
 
 		x = 0;
-		while (s[x] && x+skip < (vid.width>>3)) {
-			Draw_CharacterW ( (x+skip)<<3, v + bound(0, con_shift.value, 8), s[x]);
+		while (s[x] && posX < width) {
+			float char_width = Draw_CharacterWSP(posX, posY + bound(0, con_shift.value, 8), s[x], scale, proportional);
+
+			if (display_cursor && x == chat_linepos) {
+				Draw_SCharacterP(posX, posY + bound(0, con_shift.value, 8), 11, scale, proportional);
+				display_cursor = false;
+			}
+			posX += char_width;
 			x++;
 		}
-		v += 8;
+		if (display_cursor) {
+			Draw_SCharacterP(posX, posY + bound(0, con_shift.value, 8), 11, scale, proportional);
+		}
+		posY += 8 * scale;
+		return true;
 	}
 
-	if (v > con_notifylines)
-		con_notifylines = v + bound(0, con_shift.value, 8);
+	return false;
+}
+
+//Draws the last few lines of output transparently over the game top
+void Con_DrawNotify(void)
+{
+	if (con_notify.integer) {
+		SCR_DrawNotify(8, bound(0, con_shift.value, 8), 1.0f, con_notifytime.value, _con_notifylines.integer, con_linewidth, con_proportional.integer);
+	}
 }
 
 // Draws the last few lines of output as a custom HUD element.
-void SCR_DrawNotify(int posX, int posY, float scale, int notifyTime, int notifyLines, int notifyCols)
+void SCR_DrawNotify(int posX, int posY, float scale, int notifyTime, int notifyLines, int notifyCols, qbool proportional)
 {
-	int v, skip, i, j, k, idx, draw, offset;
-	wchar *text, *s;
-	wchar buf[1024];
-	clrinfo_t clr[sizeof(buf)];
+	int v, i, idx;
 	float time;
 	float timeout = bound (0, notifyTime, MAX_NOTIFICATION_TIME);
-	int first_line = Con_FirstNotifyLine(notifyLines, timeout);
+	float indent = 0;
 
-	if (notifyCols > (con_linewidth))
-		notifyCols = con_linewidth;
-
-	if (notifyCols < 10)
-		notifyCols = 10;
+	notifyCols = bound(10, notifyCols, con_linewidth);
 
 	v = 0;
 	if (notifyLines) {
+		int first_line = Con_FirstNotifyLine(notifyLines, timeout);
+
 		for (i = first_line; i <= con.current; i++) {
-			if (i < 0)
+			if (i < 0) {
 				continue;
-
+			}
 			time = con_times[i % NUM_CON_TIMES];
-			if (time == 0)
+			if (time == 0) {
 				continue;
-
+			}
 			time = cls.realtime - time;
-			if (time > timeout)
+			if (time > timeout) {
 				continue;
-
-			idx = (i % con_totallines)*con_linewidth;
-			text = con.text + idx;
+			}
+			idx = (i % con_totallines) * con_linewidth;
 
 			clearnotify = 0;
 			scr_copytop = 1;
 
-			// Copy current line to buffer
-			offset = 0;
-			draw = 0;
-			for (j = 0; j < con_linewidth; j++)
-			{
-				// each new line of a notify hud element
-				if ((j % notifyCols) == 0 && j != 0)
-				{
-					for (k = 0; k < (j - offset); ++k)
-					{
-						if (buf[k] != ' ')
-						{
-							draw = 1;
-							break;
-						}
-					}
+			int printed = 0;
+			int last = idx + con_linewidth - 1;
+			while (last > idx && (con.text + last)[0] == 32) {
+				--last;
+			}
 
-					buf[j - offset] = '\0';
-					offset = j;
+			while (last > idx) {
+				printed = Draw_ConsoleString(posX, v + posY, con.text + idx, con.clr + idx, min(notifyCols, last - idx + 1), 0, scale, proportional);
+				idx += printed;
+				v += (8 * scale);
+				if (printed == 0 || con.text[idx] == 0 || con.text[idx] == 0x80) {
+					break;
 				}
-				else if (j == (con_linewidth - 1)) // Ending of the string.
-				{
-					for (k = 0; k < (j - offset); ++k)
-					{
-						if (buf[k] != ' ')
-						{
-							draw = 1;
-							break;
-						}
-					}
-
-					buf[j - offset] = '\0';
-				}
-
-				// Output.
-				if (draw)
-				{
-					if (!draw)
-						continue;
-
-					Draw_SColoredString(
-						posX,
-						v + posY,
-						buf,
-						clr,
-						notifyCols,
-						0,
-						scale
-						);
-
-					// move text down
-					v += (8 * scale);
-
-					if (v > (notifyLines * scale))
-						notifyLines = v;
-
-					draw = 0;
-				}
-
-				buf[j - offset] = text[j];
-				clr[j - offset] = con.clr[idx + j]; // copy whole color struct
-				clr[j - offset].i = j; // set proper index
 			}
 		}
 	}
 
-	if (key_dest == key_message)
-	{
-		wchar temp[MAXCMDLINE + 1];
+	if (Con_NotifyMessageLine(posX, v + posY, notifyCols * 8 * scale, v * 8 * scale, scale, proportional)) {
+		v += 8 * scale;
+	}
 
-		clearnotify = 0;
-		scr_copytop = 1;
-
-		if (chat_team)
-		{
-			Draw_SString (posX, v + posY, "say_team: ", scale);
-			skip = 10;
-		}
-		else
-		{
-			Draw_SString (posX, v + posY, "say: ", scale);
-			skip = 5;
-		}
-
-		// FIXME: clean this up
-		qwcslcpy (temp, chat_buffer, sizeof(temp) / sizeof(wchar));
-		s = temp;
-
-		// Add the cursor frame.
-		if ((int) (curtime * con_cursorspeed) & 1)
-		{
-			if (chat_linepos == (int) qwcslen(s))
-				s[chat_linepos+1] = '\0';
-			s[chat_linepos] = 11;
-		}
-
-
-		if (chat_linepos + skip >= notifyCols)
-			s += 1 + chat_linepos + skip - notifyCols;
-
-		j = 0;
-
-		while (s[j] && (j + skip < notifyCols))
-		{
-			Draw_SCharacter (
-				posX + (j + skip) * 8 * scale,
-				v + posY,
-				s[j],
-				scale
-				);
-
-			j++;
-		}
-
-		v += (8 * scale);
-		if (v > con_notifylines)
-			con_notifylines = v + bound(0, con_shift.value, 8);
+	if (v > con_notifylines) {
+		con_notifylines = v + bound(0, con_shift.value, 8);
 	}
 }
 
 //Draws the console with the solid background
 void Con_DrawConsole (int lines) {
 	int i, j, x, y, n=0, rows, row, idx;
-	wchar *wtext;
 	char *text, dlbar[1024];
-	wchar buf[1024];
-	clrinfo_t clr[sizeof(buf)];
 
-	if (lines <= 0)
+	if (lines <= 0 || Con_MenuWillDrawConsole())
 		return;
 
-// draw the background
+	// draw the background
 	Draw_ConsoleBackground (lines);
 
 	// draw the text
@@ -1037,17 +936,9 @@ void Con_DrawConsole (int lines) {
 			break;		// past scrollback wrap point
 
 		idx = (row % con_totallines)*con_linewidth;
-		wtext = con.text + idx;
 
 		// copy current line to buffer
-		for(x = 0; x < con_linewidth; x++) {
-			buf[x] = wtext[x];
-			clr[x] = con.clr[idx + x]; // copy whole color struct
-			clr[x].i = x; // set proper index
-		}
-		buf[x] = '\0';
-
-		Draw_ColoredString3W( 1 << 3, y + bound(0, con_shift.value, 8), buf, clr, con_linewidth, 0);
+		Draw_ConsoleString( 1 << 3, y + bound(0, con_shift.value, 8), con.text + idx, con.clr + idx, con_linewidth, 0, 1, con_proportional.integer);
 	}
 
 	// draw the download bar
