@@ -26,26 +26,27 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define VAR_HASHPOOL_SIZE 32
 #else
 #include "common.h"
-#ifdef WITH_TCL
-#include "embed_tcl.h"
-#endif
 #include "teamplay.h"
 #include "rulesets.h"
 #include "keys.h"
 #include "utils.h"
 #include "parser.h"
+#include "r_renderer.h"
 
-extern void CL_UserinfoChanged (char *key, char *value);
-extern void SV_ServerinfoChanged (char *key, char *value);
-extern void Help_DescribeCvar (cvar_t *v);
+static void Cvar_ApplyLatchedUpdate(cvar_t* var);
+
+void CL_UserinfoChanged(char *key, char *value);
+void SV_ServerinfoChanged(char *key, char *value);
+void Help_DescribeCvar(cvar_t *v);
+void VID_ReloadCvarChanged(cvar_t* var);
 
 extern cvar_t r_fullbrightSkins;
 extern cvar_t cl_fakeshaft;
 extern cvar_t allow_scripts;
 
-cvar_t	cvar_viewdefault = {"cvar_viewdefault", "1"};
-cvar_t	cvar_viewhelp    = {"cvar_viewhelp",    "1"};
-cvar_t  cvar_viewlatched = {"cvar_viewlatched", "1"};
+static cvar_t cvar_viewdefault = {"cvar_viewdefault", "1"};
+static cvar_t cvar_viewhelp    = {"cvar_viewhelp",    "1"};
+static cvar_t cvar_viewlatched = {"cvar_viewlatched", "1"};
 
 #define VAR_HASHPOOL_SIZE 1024
 
@@ -57,15 +58,17 @@ cvar_t *cvar_vars;
 static char	*cvar_null_string = "";
 
 // Use this to walk through all vars
-cvar_t *Cvar_Next (cvar_t *var)
+cvar_t* Cvar_Next(cvar_t *var)
 {
-	if (var)
+	if (var) {
 		return var->next;
-	else
+	}
+	else {
 		return cvar_vars;
+	}
 }
 
-cvar_t *Cvar_Find (const char *var_name)
+cvar_t *Cvar_Find(const char *var_name)
 {
 	cvar_t *var;
 	int key = Com_HashKey (var_name) % VAR_HASHPOOL_SIZE;
@@ -83,8 +86,9 @@ float Cvar_Value (const char *var_name)
 {
 	cvar_t *var = Cvar_Find (var_name);
 
-	if (!var)
+	if (!var) {
 		return 0;
+	}
 	return Q_atof (var->string);
 }
 
@@ -99,6 +103,16 @@ char *Cvar_String (const char *var_name)
 
 void SV_SendServerInfoChange(char *key, char *value);
 
+char* Cvar_ServerInfoValue(char* key, char* value)
+{
+	// force serverinfo "0" vars to be "".
+	// meag: deathmatch is a special case as clients default 'not in serverinfo' to non-coop
+	if (!strcmp(value, "0") && strcmp(key, "deathmatch")) {
+		return "";
+	}
+	return value;
+}
+
 void Cvar_SetEx(cvar_t *var, char *value, qbool ignore_callback)
 {
 	static qbool changing = false;
@@ -114,8 +128,8 @@ void Cvar_SetEx(cvar_t *var, char *value, qbool ignore_callback)
 		return;
 
 	// force serverinfo "0" vars to be "".
-	if ((var->flags & CVAR_SERVERINFO) && !strcmp(value, "0") && strcmp(var->name, "deathmatch")) {
-		value = "";
+	if (var->flags & CVAR_SERVERINFO) {
+		value = Cvar_ServerInfoValue(var->name, value);
 	}
 
 #ifndef SERVERONLY
@@ -188,7 +202,7 @@ void Cvar_SetEx(cvar_t *var, char *value, qbool ignore_callback)
 			}
 			Com_Printf ("%s needs %s to take effect.\n", var->name, restartcmd);
 		}
-		var->latchedString = Q_strdup(value);
+		var->latchedString = Q_strdup_named(value, var->name);
 		var->modified = true; // set to true even car->string is not changed yet, that how q3 does
 		return;
 	}
@@ -198,6 +212,7 @@ void Cvar_SetEx(cvar_t *var, char *value, qbool ignore_callback)
 		qbool cancel = false;
 
 		changing = true;
+		var->flags &= ~(CVAR_AUTOSETRECENT);
 		var->OnChange(var, value, &cancel);
 		changing = false;
 
@@ -210,7 +225,7 @@ void Cvar_SetEx(cvar_t *var, char *value, qbool ignore_callback)
 	same_value = value && var->string && (value == var->string || !strcmp (value, var->string));
 #endif
 	// dup string first (before free) since 'value' and 'var->string' can point at the same memory area.
-	new_val = Q_strdup(value);
+	new_val = Q_strdup_named(value, var->name);
 	// free the old value string
 	Q_free(var->string);
 	// assign new value
@@ -218,10 +233,16 @@ void Cvar_SetEx(cvar_t *var, char *value, qbool ignore_callback)
 	var->value = Q_atof (var->string);
 #ifndef SERVERONLY
 	var->integer = Q_atoi (var->string);
-	StringToRGB_W(var->string, var->color);
-	if (!same_value) {
+	if (var->flags & CVAR_TRACKERCOLOR) {
+		TrackerStringToRGB_W(var->string, var->color);
+	}
+	else {
+		StringToRGB_W(var->string, var->color);
+	}
+	if (!same_value && !(var->flags & CVAR_AUTOSETRECENT)) {
 		Cvar_AutoReset (var);
 	}
+	var->flags &= ~(CVAR_AUTOSETRECENT);
 	var->modified = true;
 #endif
 
@@ -235,6 +256,12 @@ void Cvar_SetEx(cvar_t *var, char *value, qbool ignore_callback)
 	if (var->flags & CVAR_USERINFO) {
 		CL_UserinfoChanged(var->name, var->string);
 	}
+	if (var->flags & CVAR_RECOMPILE_PROGS) {
+		renderer.CvarForceRecompile(var);
+	}
+	if ((var->flags & CVAR_RELOAD_GFX) && host_everything_loaded && !same_value) {
+		VID_ReloadCvarChanged(var);
+	}
 #endif
 }
 
@@ -275,36 +302,41 @@ void Cvar_SetByName (const char *var_name, char *value)
 	Cvar_Set (var, value);
 }
 
-void Cvar_SetValue (cvar_t *var, const float value)
+static void Cvar_ValueToString(char* val, int size, float value)
 {
-	char val[128];
 	int	i;
 
-	snprintf (val, sizeof(val), "%f", value);
-
+	snprintf(val, size - 1, "%f", value);
 	for (i = strlen(val) - 1; i > 0 && val[i] == '0'; i--) {
 		val[i] = 0;
 	}
 
 	if (val[i] == '.') {
-		val[i] = 0;
+		val[i] = '\0';
 	}
-
-	Cvar_Set (var, val);
 }
 
-void Cvar_SetValueByName (const char *var_name, const float value)
+void Cvar_SetValue (cvar_t *var, const float value)
+{
+	char val[128];
+
+	Cvar_ValueToString(val, sizeof(val), value);
+
+	Cvar_Set(var, val);
+}
+
+void Cvar_SetValueByName(const char *var_name, const float value)
 {
 	char val[32];
 
-	snprintf (val, sizeof (val), "%.8g", value);
-	Cvar_SetByName (var_name, val);
+	snprintf(val, sizeof(val), "%.8g", value);
+	Cvar_SetByName(var_name, val);
 }
 
-void Cvar_Register (cvar_t *var)
+void Cvar_Register(cvar_t *var)
 {
 	int key;
-	cvar_t *old = Cvar_Find (var->name);
+	cvar_t *old = Cvar_Find(var->name);
 
 	// All variables must be named :)
 	if (var->name[0] == '\0')
@@ -315,18 +347,18 @@ void Cvar_Register (cvar_t *var)
 
 	// first check to see if it has already been defined
 	if (old) {
-		Con_Printf ("Can't register variable %s, already defined\n", var->name);
+		Con_Printf("Can't register variable %s, already defined\n", var->name);
 		return;
 	}
 
 	// check for overlap with a command
-	if (Cmd_Exists (var->name)) {
-		Con_Printf ("Cvar_Register: %s is a command\n", var->name);
+	if (Cmd_Exists(var->name)) {
+		Con_Printf("Cvar_Register: %s is a command\n", var->name);
 		return;
 	}
 
 	// link the variable in
-	key = Com_HashKey (var->name);
+	key = Com_HashKey(var->name);
 	var->hash_next = cvar_hash[key];
 	cvar_hash[key] = var;
 	var->next = cvar_vars;
@@ -335,76 +367,79 @@ void Cvar_Register (cvar_t *var)
 	// set it through the function to be consistent
 	value = var->string;
 	var->string = Q_strdup("");
-	Cvar_SetROM (var, value);
+	Cvar_SetROM(var, value);
 #else
 	// we already register cvar, warn about it
-	if (old && !(old->flags & CVAR_USER_CREATED))
-	{
-		// allow re-register lacthed cvar
-		if (old->flags & CVAR_LATCH)
-		{
+	if (old && !(old->flags & CVAR_USER_CREATED)) {
+		// allow re-register latched cvar
+		if (old->flags & CVAR_LATCH) {
 			// if we have a latched string, take that value now
-			if ( old->latchedString )
-			{
-				// I did't want bother with all this CVAR_ROM and OnChange handler, just set value
-				Q_free(old->string);
-				old->string  = old->latchedString;
-				old->latchedString = NULL;
-				old->value   = Q_atof (old->string);
-				old->integer = Q_atoi (old->string);
-				StringToRGB_W(old->string, old->color);
-				old->modified = true;
-			}
+			Cvar_ApplyLatchedUpdate(old);
 			Cvar_AutoReset(old);
 
 			return;
 		}
 
 		// warn if CVAR_SILENT is not set
-		if (!(old->flags & CVAR_SILENT))
-			Com_Printf ("Can't register variable %s, already defined\n", var->name);
+		if (!(old->flags & CVAR_SILENT)) {
+			Com_Printf("Can't register variable %s, already defined\n", var->name);
+		}
 
 		return;
 	}
 
-	if (old && old == var)
+	if (old && old == var) {
 		Sys_Error("Cvar_Register: something wrong with %s", var->name);
+	}
 
-	if (var->defaultvalue)
-		Sys_Error("Cvar_Register: defaultvalue alredy set for %s", var->name);
+	if (var->defaultvalue) {
+		Sys_Error("Cvar_Register: defaultvalue already set for %s", var->name);
+	}
 
-	var->defaultvalue = Q_strdup(var->string);
-	if (old)
-	{
+	var->defaultvalue = Q_strdup_named(var->string, var->name);
+	if (old) {
 		char string[512];
+		qbool queued_replace;
 
-		var->flags |= old->flags & ~(CVAR_USER_CREATED|CVAR_TEMP);
-		strlcpy (string, (var->flags & CVAR_ROM) ? var->string : old->string, sizeof(string));
-		Cvar_Delete (old->name);
-		var->string = Q_strdup(string);
+		// Trigger change later if this is a proper cvar replacing a user one
+		queued_replace = (old->flags & CVAR_USER_CREATED) && (var->flags & CVAR_QUEUED_TRIGGER) && !(var->flags & CVAR_USER_CREATED);
+		if (queued_replace) {
+			// Created by executing config, now we have the 'proper' cvar - store old value to trigger later
+			var->latchedString = Q_strdup_named(old->string, var->name);
+		}
+
+		var->flags |= old->flags & ~(CVAR_USER_CREATED | CVAR_TEMP);
+		strlcpy(string, (var->flags & CVAR_ROM) || queued_replace ? var->string : old->string, sizeof(string));
+		Cvar_Delete(old->name);
+		var->string = Q_strdup_named(string, var->name);
 	}
-	else
-	{
+	else {
 		// allocate the string on zone because future sets will Q_free it
-		var->string = Q_strdup(var->string);
+		var->string = Q_strdup_named(var->string, var->name);
 	}
 
-	var->value = Q_atof (var->string);
-	var->integer = Q_atoi (var->string);
+	var->value = Q_atof(var->string);
+	if (((var->flags & CVAR_RULESET_MIN) && var->value < var->minrulesetvalue) ||
+	    ((var->flags & CVAR_RULESET_MAX) && var->value > var->maxrulesetvalue)) {
+		char val[128];
+
+		var->value = bound(var->minrulesetvalue, var->value, var->maxrulesetvalue);
+		Cvar_ValueToString(val, sizeof(val), var->value);
+
+		Q_free(var->string);
+		var->string = Q_strdup_named(val, var->name);
+	}
+	var->integer = Q_atoi(var->string);
 	StringToRGB_W(var->string, var->color);
 	var->modified = true;
 
 	// link the variable in
-	key = Com_HashKey (var->name) % VAR_HASHPOOL_SIZE;
+	key = Com_HashKey(var->name) % VAR_HASHPOOL_SIZE;
 	var->hash_next = cvar_hash[key];
 	cvar_hash[key] = var;
 	var->next = cvar_vars;
 	cvar_vars = var;
 
-#ifdef WITH_TCL
-	TCL_RegisterVariable (var);
-#endif
-
 	Cvar_AddCvarToGroup(var);
 
 #ifndef CLIENTONLY
@@ -419,6 +454,20 @@ void Cvar_Register (cvar_t *var)
 #endif // !SERVERONLY
 }
 
+static void Cvar_ApplyLatchedUpdate(cvar_t* var)
+{
+	if (var->latchedString) {
+		// I did't want bother with all this CVAR_ROM and OnChange handler, just set value
+		Q_free(var->string);
+		var->string = var->latchedString;
+		var->latchedString = NULL;
+		var->value = Q_atof(var->string);
+		var->integer = Q_atoi(var->string);
+		StringToRGB_W(var->string, var->color);
+		var->modified = true;
+	}
+}
+
 qbool Cvar_Command (void)
 {
 	cvar_t *v;
@@ -498,7 +547,7 @@ static void Cvar_Toggle_f_base (qbool use_regex)
 		name = Cmd_Argv(i);
 
 		if (use_regex && IsRegexp(name)) {
-			if (!ReSearchInit(name)) {
+			if (!ReSearchInitEx(name, false)) {
 				continue;
 			}
 
@@ -563,7 +612,7 @@ void Cvar_CvarList(qbool use_regex)
 	pattern = (Cmd_Argc() > 1) ? Cmd_Argv(1) : NULL;
 
 	if (((c = Cmd_Argc()) > 1) && use_regex) {
-		if (!ReSearchInit(Cmd_Argv(1))) {
+		if (!ReSearchInitEx(Cmd_Argv(1), false)) {
 			return;
 		}
 	}
@@ -612,6 +661,34 @@ void Cvar_CvarList_re_f (void)
 	Cvar_CvarList (true);
 }
 
+static void Cvar_CvarEdit_f(void)
+{
+	cvar_t* cvar;
+	char *s, final_string[MAXCMDLINE - 1];
+	int c;
+
+	c = Cmd_Argc();
+	if (c == 1) {
+		Com_Printf("%s <name> : modify a cvar\n", Cmd_Argv(0));
+		Com_Printf("cvarlist : list all cvars\n");
+		return;
+	}
+
+	cvar = Cvar_Find(Cmd_Argv(1));
+	if (cvar == NULL) {
+		Com_Printf("\"%s\" not a valid cvar\n", Cmd_Argv(1));
+		return;
+	}
+	else {
+		s = Q_strdup(cvar->string);
+	}
+
+	snprintf(final_string, sizeof(final_string), "/%s \"%s\"", Cmd_Argv(1), s);
+	Key_ClearTyping();
+	memcpy(key_lines[edit_line] + 1, str2wcs(final_string), strlen(final_string) * sizeof(wchar));
+	Q_free(s);
+}
+
 cvar_t *Cvar_Create(const char *name, const char *string, int cvarflags)
 {
 	cvar_t *v;
@@ -634,22 +711,19 @@ cvar_t *Cvar_Create(const char *name, const char *string, int cvarflags)
 	v->hash_next = cvar_hash[key];
 	cvar_hash[key] = v;
 
-	v->name = Q_strdup(name);
-	v->string = Q_strdup(string);
+	v->name = Q_strdup_named(name, name);
+	v->string = Q_strdup_named(string, name);
 	v->value = Q_atof(v->string);
 	v->flags = cvarflags;
 #ifndef SERVERONLY
 	v->flags |= CVAR_USER_CREATED;
-	v->defaultvalue = Q_strdup(string);
+	v->defaultvalue = Q_strdup_named(string, name);
 	if (cbuf_current == &cbuf_server) {
 		v->flags |= CVAR_MOD_CREATED;
 	}
 	v->integer = Q_atoi(v->string);
 	StringToRGB_W(v->string, v->color);
 	v->modified = true;
-#ifdef WITH_TCL
-	TCL_RegisterVariable(v);
-#endif
 #endif
 
 	return v;
@@ -690,9 +764,6 @@ qbool Cvar_Delete(const char *name)
 				cvar_vars = var->next;
 			}
 #ifndef SERVERONLY
-#ifdef WITH_TCL
-			TCL_UnregisterVariable(name);
-#endif
 			Q_free(var->defaultvalue);
 #endif
 			// free
@@ -796,7 +867,7 @@ void Cvar_Reset(qbool use_regex)
 		name = Cmd_Argv(i);
 
 		if (use_regex && (re_search = IsRegexp(name))) {
-			if (!ReSearchInit(name)) {
+			if (!ReSearchInitEx(name, false)) {
 				continue;
 			}
 		}
@@ -832,19 +903,19 @@ void Cvar_Reset_re_f (void)
 	Cvar_Reset(true);
 }
 
-void Cvar_SetDefault(cvar_t *var, float value)
+void Cvar_SetDefaultAndValue(cvar_t *var, float default_value, float actual_value)
 {
 	char val[128];
 	int i;
 
-	snprintf (val, sizeof(val), "%f", value);
+	snprintf (val, sizeof(val), "%f", default_value);
 	for (i = strlen(val) - 1; i > 0 && val[i] == '0'; i--)
 		val[i] = 0;
 	if (val[i] == '.')
 		val[i] = 0;
 	Q_free(var->defaultvalue);
-	var->defaultvalue = Q_strdup(val);
-	Cvar_Set(var, val);
+	var->defaultvalue = Q_strdup_named(val, var->name);
+	Cvar_SetValue(var, actual_value);
 }
 
 char *Cvar_CompleteVariable (char *partial)
@@ -912,7 +983,8 @@ void Cvar_AutoSet(cvar_t *var, char *value)
 
 	Q_free(var->autoString);
 
-	var->autoString = strdup(value);
+	var->autoString = Q_strdup(value);
+	var->flags |= CVAR_AUTOSETRECENT;
 }
 
 void Cvar_AutoSetInt(cvar_t *var, int value)
@@ -927,7 +999,8 @@ void Cvar_AutoSetInt(cvar_t *var, int value)
 
 	snprintf(&val[0], sizeof(val), "%d", value);
 
-	var->autoString = strdup(val);
+	var->autoString = Q_strdup_named(val, var->name);
+	var->flags |= CVAR_AUTOSETRECENT;
 }
 
 void Cvar_AutoReset(cvar_t *var)
@@ -940,36 +1013,38 @@ void Cvar_AutoReset(cvar_t *var)
 }
 
 // silently set value for latched cvar
-void Cvar_LatchedSet (cvar_t *var, char *value)
+void Cvar_LatchedSet(cvar_t *var, char *value)
 {
 	// yea, do not mess things
-	if ( !(var->flags & CVAR_LATCH) )
+	if (!(var->flags & CVAR_LATCH)) {
 		Sys_Error("Cvar_LatchedSet: non latched var %s", var->name);
+	}
 
 	var->flags |= CVAR_SILENT; // shut up warnings, at least some...
 	Cvar_Set(var, value);
 	// remove this flag, btw if variable have this flag before set this flag will be removed anyway..
 	var->flags &= ~CVAR_SILENT;
 	// actually set value
-	Cvar_Register( var );
+	Cvar_Register(var);
 }
 
 // silently set value for latched cvar
-void Cvar_LatchedSetValue (cvar_t *var, float value)
+void Cvar_LatchedSetValue(cvar_t *var, float value)
 {
 	// yea, do not mess things
-	if ( !(var->flags & CVAR_LATCH) )
+	if (!(var->flags & CVAR_LATCH)) {
 		Sys_Error("Cvar_LatchedSet: non latched var %s", var->name);
+	}
 
 	var->flags |= CVAR_SILENT; // shut up warnings, at least some...
 	Cvar_SetValue(var, value);
 	// remove this flag, btw if variable have this flag before set this flag will be removed anyway..
 	var->flags &= ~CVAR_SILENT;
 	// actually set value
-	Cvar_Register( var );
+	Cvar_Register(var);
 }
 
-void Cvar_SetFlags (cvar_t *var, int flags)
+void Cvar_SetFlags(cvar_t *var, int flags)
 {
 	var->flags = flags;
 }
@@ -1069,35 +1144,37 @@ void Cvar_Set_tp_f(void)
 	}
 }
 
-void Cvar_Set_ex_f (void)
+static void Cvar_Set_ex_f(void)
 {
-	cvar_t	*var;
-	char	*var_name;
-	char	*st = NULL;
+	cvar_t* var;
+	char* var_name;
+	char* st = NULL;
 	char	text_exp[1024];
+	qbool   parse_funchars = !strcasecmp(Cmd_Argv(0), "set_ex");
 
 	if (Cmd_Argc() != 3) {
-		Com_Printf ("usage: set_ex <cvar> <value>\n");
+		Com_Printf("usage: %s <cvar> <value>\n", Cmd_Argv(0));
 		return;
 	}
 
-	var_name = Cmd_Argv (1);
-	var = Cvar_Find (var_name);
+	var_name = Cmd_Argv(1);
+	var = Cvar_Find(var_name);
 
-
-	if ( !var ) {
+	if (!var) {
 		if (Cmd_Exists(var_name)) {
-			Com_Printf ("\"%s\" is a command\n", var_name);
+			Com_Printf("\"%s\" is a command\n", var_name);
 			return;
 		}
 		var = Cvar_Create(var_name, "", 0);
 	}
 
-	Cmd_ExpandString( Cmd_Argv(2), text_exp);
-	st = TP_ParseMacroString( text_exp );
-	st = TP_ParseFunChars(st, false);
+	Cmd_ExpandString(Cmd_Argv(2), text_exp);
+	st = TP_ParseMacroString(text_exp);
+	if (parse_funchars) {
+		st = TP_ParseFunChars(st, false);
+	}
 
-	Cvar_Set (var, st );
+	Cvar_Set(var, st);
 }
 
 void Cvar_Set_Alias_Str_f (void)
@@ -1191,7 +1268,6 @@ void Cvar_UnSet (qbool use_regex)
 	int		i;
 	qbool	re_search = false;
 
-
 	if (Cmd_Argc() < 2) {
 		Com_Printf ("unset <cvar> [<cvar2>..]: erase user-created variable\n");
 		return;
@@ -1200,9 +1276,11 @@ void Cvar_UnSet (qbool use_regex)
 	for (i=1; i<Cmd_Argc(); i++) {
 		name = Cmd_Argv(i);
 
-		if (use_regex && (re_search = IsRegexp(name)))
-			if(!ReSearchInit(name))
+		if (use_regex && (re_search = IsRegexp(name))) {
+			if (!ReSearchInitEx(name, false)) {
 				continue;
+			}
+		}
 
 		if (use_regex && re_search) {
 			for (var = cvar_vars ; var ; var = next) {
@@ -1441,7 +1519,7 @@ void Cvar_Set_Eval_f(void)
 				break;
 			case ET_STR:
 				Cvar_Set(var, result.s_val);
-				free(result.s_val);
+				Q_free(result.s_val);
 				break;
 			}
 		}
@@ -1476,32 +1554,131 @@ void Cvar_CleanUpTempVars (void)
 }
 #endif
 
-void Cvar_Init (void)
+void Cvar_Init(void)
 {
-	Cmd_AddCommand ("cvarlist", Cvar_CvarList_f);
-	Cmd_AddCommand ("cvarlist_re", Cvar_CvarList_re_f);
-	Cmd_AddCommand ("toggle", Cvar_Toggle_f);
-	Cmd_AddCommand ("set", Cvar_Set_f);
-	Cmd_AddCommand ("inc", Cvar_Inc_f);
+	Cmd_AddCommand("cvarlist", Cvar_CvarList_f);
+	Cmd_AddCommand("cvaredit", Cvar_CvarEdit_f);
+	Cmd_AddCommand("cvarlist_re", Cvar_CvarList_re_f);
+	Cmd_AddCommand("toggle", Cvar_Toggle_f);
+	Cmd_AddCommand("set", Cvar_Set_f);
+	Cmd_AddCommand("inc", Cvar_Inc_f);
 
 #ifndef SERVERONLY
-	Cmd_AddCommand ("set_tp", Cvar_Set_tp_f);
-	Cmd_AddCommand ("set_ex", Cvar_Set_ex_f);
-	Cmd_AddCommand ("set_alias_str", Cvar_Set_Alias_Str_f);
-	Cmd_AddCommand ("set_bind_str", Cvar_Set_Bind_Str_f);
-	Cmd_AddCommand ("unset", Cvar_UnSet_f);
-	Cmd_AddCommand ("unset_re", Cvar_UnSet_re_f);
-	Cmd_AddCommand ("toggle_re", Cvar_Toggle_re_f);
-	Cmd_AddCommand ("cvar_reset", Cvar_Reset_f);
-	Cmd_AddCommand ("cvar_reset_re", Cvar_Reset_re_f);
-	Cmd_AddCommand ("set_calc", Cvar_Set_Calc_f);
-	Cmd_AddCommand ("set_eval", Cvar_Set_Eval_f);
+	Cmd_AddCommand("set_tp", Cvar_Set_tp_f);
+	Cmd_AddCommand("set_ex", Cvar_Set_ex_f);
+	Cmd_AddCommand("set_ex2", Cvar_Set_ex_f);
+	Cmd_AddCommand("set_alias_str", Cvar_Set_Alias_Str_f);
+	Cmd_AddCommand("set_bind_str", Cvar_Set_Bind_Str_f);
+	Cmd_AddCommand("unset", Cvar_UnSet_f);
+	Cmd_AddCommand("unset_re", Cvar_UnSet_re_f);
+	Cmd_AddCommand("toggle_re", Cvar_Toggle_re_f);
+	Cmd_AddCommand("cvar_reset", Cvar_Reset_f);
+	Cmd_AddCommand("cvar_reset_re", Cvar_Reset_re_f);
+	Cmd_AddCommand("set_calc", Cvar_Set_Calc_f);
+	Cmd_AddCommand("set_eval", Cvar_Set_Eval_f);
 
 	Cvar_SetCurrentGroup(CVAR_GROUP_CONSOLE);
-	Cvar_Register (&cvar_viewdefault);
-	Cvar_Register (&cvar_viewhelp);
-	Cvar_Register (&cvar_viewlatched);
+	Cvar_Register(&cvar_viewdefault);
+	Cvar_Register(&cvar_viewhelp);
+	Cvar_Register(&cvar_viewlatched);
 
 	Cvar_ResetCurrentGroup();
 #endif
 }
+
+void Cvar_Shutdown(void)
+{
+	cvar_t *cvar;
+	cvar_t *next;
+	cvar_group_t* group;
+	cvar_group_t* next_group;
+
+	for (cvar = cvar_vars; cvar; cvar = next) {
+		next = cvar->next;
+
+#ifndef SERVERONLY
+		Q_free(cvar->latchedString);
+		Q_free(cvar->autoString);
+		Q_free(cvar->defaultvalue);
+#endif
+		Q_free(cvar->string);
+
+#ifndef SERVERONLY
+		if (cvar->flags & CVAR_USER_CREATED) {
+			Q_free(cvar->name);
+			Q_free(cvar);
+		}
+#endif
+	}
+
+#ifndef SERVERONLY
+	for (group = cvar_groups; group; group = next_group) {
+		next_group = group->next;
+		Q_free(group);
+	}
+#endif
+}
+
+void Cvar_ExecuteQueuedChanges(void)
+{
+	cvar_t* cvar;
+	qbool vid_restart = false;
+	qbool vid_reload = false;
+	qbool sound_restart = false;
+
+	for (cvar = cvar_vars; cvar; cvar = cvar->next) {
+		if ((cvar->flags & CVAR_QUEUED_TRIGGER) && cvar->latchedString) {
+			if (cvar->OnChange) {
+				qbool cancel = false;
+
+				cvar->OnChange(cvar, cvar->latchedString, &cancel);
+				if (!cancel) {
+					Cvar_ApplyLatchedUpdate(cvar);
+				}
+			}
+
+			Q_free(cvar->latchedString);
+		}
+
+		vid_restart |= (cvar->flags & CVAR_LATCH_GFX) && cvar->latchedString;
+		vid_reload |= (cvar->flags & CVAR_RELOAD_GFX && cvar->modified);
+		sound_restart |= (cvar->flags & CVAR_LATCH_SOUND) && cvar->latchedString;
+	}
+
+	if (vid_restart) {
+		Cbuf_AddTextEx(&cbuf_main, "\nvid_restart\n");
+	}
+	else {
+		if (sound_restart) {
+			Cbuf_AddTextEx(&cbuf_main, "\ns_restart\n");
+		}
+		if (vid_reload) {
+			Cbuf_AddTextEx(&cbuf_main, "\nvid_reload\n");
+		}
+	}
+
+	Cvar_ClearAllModifiedFlags(CVAR_RELOAD_GFX);
+}
+
+void Cvar_ClearAllModifiedFlags(int flags)
+{
+	cvar_t* cvar;
+
+	for (cvar = cvar_vars; cvar; cvar = cvar->next) {
+		if (cvar->flags & flags) {
+			cvar->modified = false;
+		}
+	}
+}
+
+qbool Cvar_AnyModified(int flags)
+{
+	cvar_t* cvar;
+
+	for (cvar = cvar_vars; cvar; cvar = cvar->next) {
+		if ((cvar->flags & flags) && cvar->modified) {
+			return true;
+		}
+	}
+	return false;
+}
