@@ -37,11 +37,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "quakedef.h"
 #include "EX_browser.h"
 #include "fs.h"
-#ifdef WITH_TCL
-#include "embed_tcl.h"
-#endif
 #include "gl_model.h"
-#include "gl_local.h"
 #include "rulesets.h"
 #include "teamplay.h"
 #include "pmove.h"
@@ -50,6 +46,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "keys.h"
 #include "config_manager.h"
 #include "EX_qtvlist.h"
+#include "r_renderer.h"
+#include "central.h"
 
 double		curtime;
 
@@ -76,6 +74,12 @@ int					SYSINFO_MHz = 0;
 char				*SYSINFO_processor_description = NULL;
 char				*SYSINFO_3D_description        = NULL;
 
+static void SYSINFO_Shutdown(void)
+{
+	Q_free(SYSINFO_processor_description);
+	Q_free(SYSINFO_3D_description);
+}
+
 #ifdef _WIN32
 typedef BOOL (WINAPI *PGMSE)(LPMEMORYSTATUSEX);
 
@@ -85,7 +89,7 @@ void SYSINFO_Init(void)
 	HKEY  hKey;
 	PGMSE pGMSE;
 
-	extern const char *gl_renderer;
+	const char *gl_renderer = renderer.DescriptiveString();
 
 	// Get memory size.
 	if ((pGMSE = (PGMSE)GetProcAddress(GetModuleHandle(TEXT("kernel32.dll")), "GlobalMemoryStatusEx")) != NULL) {
@@ -140,7 +144,7 @@ void SYSINFO_Init(void)
 
 	if (gl_renderer && gl_renderer[0]) {
 		if (SYSINFO_3D_description != NULL) {
-			free(SYSINFO_3D_description);
+			Q_free(SYSINFO_3D_description);
 		}
 		SYSINFO_3D_description = Q_strdup(gl_renderer);
 	}
@@ -175,7 +179,7 @@ void SYSINFO_Init(void)
 	char *match;
 	FILE *f;
 
-	extern const char *gl_renderer;
+	const char *gl_renderer = renderer.DescriptiveString();
 
 	// MEM
 	f = fopen("/proc/meminfo", "r");
@@ -197,7 +201,9 @@ void SYSINFO_Init(void)
 		buffer[fread (buffer, 1, sizeof(buffer) - 1, f)] = '\0';
 		fclose (f);
 		match = strstr (buffer, "cpu MHz");
-		sscanf (match, "cpu MHz : %i", &SYSINFO_MHz);
+		if (match) {
+			sscanf (match, "cpu MHz : %i", &SYSINFO_MHz);
+		}
 	} else {
 		Com_Printf ("could not open /proc/cpuinfo!\n");
 	}
@@ -259,14 +265,14 @@ void SYSINFO_Init(void)
 	char cpu_brand_string[100] = {0};
 	size_t cpu_brand_string_len = sizeof(cpu_brand_string) - 1; /* Don't trust Apple, make sure its NULL terminated */
 
-	extern const char *gl_renderer;
+	const char *gl_renderer = renderer.DescriptiveString();
 
 	if (sysctl(mib, 2, &memsize_value, &length, NULL, 0) != -1) {
 		SYSINFO_memory = memsize_value;
 	}
 
 	if (sysctlbyname("machdep.cpu.brand_string", &cpu_brand_string, &cpu_brand_string_len, NULL, 0) != -1) {
-		SYSINFO_processor_description = cpu_brand_string;
+		SYSINFO_processor_description = Q_strdup(cpu_brand_string);
 	}
 
 	mib[0] = CTL_HW;
@@ -279,7 +285,7 @@ void SYSINFO_Init(void)
 
 	if (gl_renderer && gl_renderer[0]) {
 		if (SYSINFO_3D_description != NULL) {
-			free(SYSINFO_3D_description);
+			Q_free(SYSINFO_3D_description);
 		}
 		SYSINFO_3D_description = Q_strdup(gl_renderer);
 	}
@@ -303,12 +309,12 @@ void SYSINFO_Init(void)
 {
 	char cpu_model[256];
 
-	int mib[2];
-	unsigned long long val;
+	int mib[2], val;
+	unsigned long val_ul;
 	size_t len;
 	unsigned long long old_tsc, tsc_freq;
 	struct timeval tp, old_tp;
-	extern const char *gl_renderer;
+	const char *gl_renderer = renderer.DescriptiveString();
 
 	mib[0] = CTL_HW;
 	mib[1] =
@@ -320,9 +326,9 @@ void SYSINFO_Init(void)
 // VVD: We can use HW_REALMEM (hw.realmem) for RELENG_5/6/7 for getting exact result,
 // but RELENG_4 have only HW_PHYSMEM (hw.physmem).
 	len = sizeof(val);
-	sysctl(mib, sizeof(mib) / sizeof(mib[0]), &val, &len, NULL, 0);
+	sysctl(mib, sizeof(mib) / sizeof(mib[0]), &val_ul, &len, NULL, 0);
 
-	SYSINFO_memory = val;
+	SYSINFO_memory = val_ul;
 
 	mib[0] = CTL_HW;
 	mib[1] = HW_MODEL;
@@ -333,11 +339,19 @@ void SYSINFO_Init(void)
 	SYSINFO_processor_description = cpu_model;
 
 	gettimeofday(&old_tp, NULL);
+#ifdef __powerpc__
+	__asm__ __volatile__("mfspr %%r3, 268": "=r" (old_tsc));
+#else
 	old_tsc = rdtsc();
+#endif
 	do {
 		gettimeofday(&tp, NULL);
 	} while ((tp.tv_sec - old_tp.tv_sec) * 1000000. + tp.tv_usec - old_tp.tv_usec < 1000000.);
+#ifdef __powerpc__
+	__asm__ __volatile__("mfspr %%r3, 268": "=r" (tsc_freq));
+#else
 	tsc_freq = rdtsc();
+#endif
 	SYSINFO_MHz = (int)((tsc_freq - old_tsc) /
 						(tp.tv_sec - old_tp.tv_sec + (tp.tv_usec - old_tp.tv_usec) / 1000000.) /
 						1000000. + .5);
@@ -346,12 +360,12 @@ void SYSINFO_Init(void)
 
 	if (gl_renderer  &&  gl_renderer[0]) {
 		if (SYSINFO_3D_description != NULL) {
-			free(SYSINFO_3D_description);
+			Q_free(SYSINFO_3D_description);
 		}
 		SYSINFO_3D_description = Q_strdup(gl_renderer);
 	}
 
-	snprintf(f_system_string, sizeof(f_system_string), "%lluMB", (int)(SYSINFO_memory / 1024LLU / 1024LLU));
+	snprintf(f_system_string, sizeof(f_system_string), "%dMB", (int)(SYSINFO_memory / 1024. / 1024. + .5));
 
 	if (SYSINFO_processor_description) {
 		strlcat(f_system_string, ", ", sizeof(f_system_string));
@@ -382,9 +396,10 @@ void Host_EndGame (void)
 #ifndef CLIENTONLY
 	SV_Shutdown ("Server was killed");
 #endif
-	CL_Disconnect ();
+	CL_Disconnect();
 	// clear disconnect messages from loopback
-	NET_ClearLoopback ();
+	CL_ClearQueuedPackets();
+	NET_ClearLoopback();
 }
 
 //This shuts down both the client and server
@@ -424,13 +439,13 @@ void Host_InitMemory (int memsize)
 {
 	int t;
 
-	if (COM_CheckParm ("-minmemory"))
+	if (COM_CheckParm (cmdline_param_host_memory_minimum))
 		memsize = MINIMUM_MEMORY;
 
-	if ((t = COM_CheckParm ("-heapsize")) != 0 && t + 1 < COM_Argc())
+	if ((t = COM_CheckParm (cmdline_param_host_memory_kb)) != 0 && t + 1 < COM_Argc())
 		memsize = Q_atoi (COM_Argv(t + 1)) * 1024;
 
-	if ((t = COM_CheckParm ("-mem")) != 0 && t + 1 < COM_Argc())
+	if ((t = COM_CheckParm (cmdline_param_host_memory_mb)) != 0 && t + 1 < COM_Argc())
 		memsize = Q_atoi (COM_Argv(t + 1)) * 1024 * 1024;
 
 	if (memsize < MINIMUM_MEMORY)
@@ -445,9 +460,6 @@ void Host_InitMemory (int memsize)
 //Can only be called when changing levels!
 void Host_ClearMemory (void)
 {
-	// FIXME, move to CL_ClearState
-	D_FlushCaches ();
-
 	// FIXME, move to CL_ClearState
 	Mod_ClearAll ();
 
@@ -465,6 +477,8 @@ void Host_Frame (double time)
 	curtime += time;
 
 	CL_Frame (time);	// will also call SV_Frame
+
+	Central_ProcessResponses();
 }
 
 char *Host_PrintBars(char *s, int len)
@@ -498,22 +512,20 @@ char *Host_PrintBars(char *s, int len)
 
 static void Commands_For_Configs_Init (void)
 {
-extern void SV_Floodprot_f (void);
-extern void SV_Floodprotmsg_f (void);
-extern void TP_MsgTrigger_f (void);
-extern void TP_MsgFilter_f (void);
-extern void TP_Took_f (void);
-extern void TP_Pickup_f (void);
-extern void TP_Point_f (void);
-extern void MT_AddMapGroups (void);
-extern void MT_MapGroup_f (void);
-extern void MT_AddSkyGroups (void);
-extern void MT_SkyGroup_f (void);
-extern void CL_Fog_f (void);
-extern void SB_SourceUnmarkAll(void);
-extern void SB_SourceMark(void);
-extern void LoadConfig_f(void);
-
+	extern void SV_Floodprot_f (void);
+	extern void SV_Floodprotmsg_f (void);
+	extern void TP_MsgTrigger_f (void);
+	extern void TP_MsgFilter_f (void);
+	extern void TP_Took_f (void);
+	extern void TP_Pickup_f (void);
+	extern void TP_Point_f (void);
+	extern void MT_AddMapGroups (void);
+	extern void MT_MapGroup_f (void);
+	extern void MT_AddSkyGroups (void);
+	extern void MT_SkyGroup_f (void);
+	extern void SB_SourceUnmarkAll(void);
+	extern void SB_SourceMark(void);
+	extern void LoadConfig_f(void);
 
 	//disconnect: fix it if i forgot something
 #ifndef CLIENTONLY
@@ -531,13 +543,57 @@ extern void LoadConfig_f(void);
 
 	MT_AddSkyGroups ();
 	Cmd_AddCommand ("skygroup", MT_SkyGroup_f);
-	Cmd_AddCommand ("fog", CL_Fog_f);
 	Cmd_AddCommand ("allskins", Skin_AllSkins_f);
 
 	Cmd_AddCommand ("sb_sourceunmarkall", SB_SourceUnmarkAll);
 	Cmd_AddCommand ("sb_sourcemark", SB_SourceMark);
 }
 
+// meag: try to move all renamed cvars here so they exist prior to config load, and
+//       eventually old configs will set the new values
+// fixme: would be better to have these all in the format of v_gamma, or (dream) all built-in cvars as enumerated type
+static void Host_RegisterLegacyCvars(void)
+{
+	// Sound
+	Cmd_AddLegacyCommand("nosound", "s_nosound");
+	Cmd_AddLegacyCommand("precache", "s_precache");
+	Cmd_AddLegacyCommand("loadas8bit", "s_loadas8bit");
+	Cmd_AddLegacyCommand("ambient_level", "s_ambientlevel");
+	Cmd_AddLegacyCommand("ambient_fade", "s_ambientfade");
+	Cmd_AddLegacyCommand("snd_show", "s_show");
+	Cmd_AddLegacyCommand("cl_chatsound", "s_chat_custom");
+
+	// Weaponstats
+	Cmd_AddLegacyCommand("scr_weaponstats_order", "hud_weaponstats_format");
+	Cmd_AddLegacyCommand("scr_weaponstats_frame_color", "hud_weaponstats_frame_color");
+	Cmd_AddLegacyCommand("scr_weaponstats_scale", "hud_weaponstats_scale");
+	Cmd_AddLegacyCommand("scr_weaponstats_y", "hud_weaponstats_y");
+	Cmd_AddLegacyCommand("scr_weaponstats_x", "hud_weaponstats_x");
+
+	// Drawing (now r_smoothtext/images/crosshair)
+	Cmd_AddLegacyCommand("gl_smoothfont", "r_smoothtext");
+
+	// Typo
+	Cmd_AddLegacyCommand("gl_lighting_colour", "gl_lighting_color");
+
+	Cmd_AddLegacyCommand("cl_truelightning", "cl_fakeshaft");
+	Cmd_AddLegacyCommand("demotimescale", "cl_demospeed");
+
+	Cmd_AddLegacyCommand("gamma", v_gamma.name);
+	Cmd_AddLegacyCommand("contrast", v_contrast.name);
+	Cmd_AddLegacyCommand("gl_gammacorrection", "vid_gammacorrection");
+
+	// if you don't like renaming things in this way, let's have some talk with tea - johnnycz
+	Cmd_AddLegacyCommand("con_sound_mm1_file", "s_mm1_file");
+	Cmd_AddLegacyCommand("con_sound_mm2_file", "s_mm2_file");
+	Cmd_AddLegacyCommand("con_sound_spec_file", "s_spec_file");
+	Cmd_AddLegacyCommand("con_sound_other_file", "s_otherchat_file");
+	Cmd_AddLegacyCommand("con_sound_mm1_volume", "s_mm1_volume");
+	Cmd_AddLegacyCommand("con_sound_mm2_volume", "s_mm2_volume");
+	Cmd_AddLegacyCommand("con_sound_spec_volume", "s_spec_volume");
+	Cmd_AddLegacyCommand("con_sound_other_volume", "s_otherchat_volume");
+}
+
 void Startup_Place(void)
 {
     extern cvar_t cl_onload;
@@ -545,7 +601,18 @@ void Startup_Place(void)
 
 	if (cl_startupdemo.string[0]) {
 		Cbuf_AddText(va("playdemo %s\n", cl_startupdemo.string));
-		key_dest = key_startupdemo;
+		if (!strcmp(cl_onload.string, "menu")) {
+			key_dest = key_startupdemo_menu;
+		}
+		else if (!strcmp(cl_onload.string, "browser")) {
+			key_dest = key_startupdemo_browser;
+		}
+		else if (!strcmp(cl_onload.string, "console")) {
+			key_dest = key_startupdemo_console;
+		}
+		else {
+			key_dest = key_startupdemo_game;
+		}
 	}
 	else if (!strcmp (cl_onload.string, "menu")) {
 		Cbuf_AddText ("togglemenu\n");
@@ -582,28 +649,31 @@ void Host_Init (int argc, char **argv, int default_memsize)
 
 	Host_InitMemory (default_memsize);
 
-#ifdef WITH_TCL
-	// interpreter should be initialized
-	// before any cvar definitions
-	TCL_InterpInit ();
-#endif
 	Cbuf_Init ();
 	Cmd_Init ();
 	Cvar_Init ();
 	COM_Init ();
 	Key_Init ();
 
-	Cache_Init_Commands ();
-
 	FS_InitFilesystem ();
 	NET_Init ();
 
+#ifdef WITH_RENDERING_TRACE
+	// Start immediately: (have to wait until filesystem is started up so we know where to save files)
+	if (COM_CheckParm(cmdline_param_client_video_r_trace)) {
+		Dev_VidFrameStart();
+	}
+#endif
+
 	Commands_For_Configs_Init ();
+	Host_RegisterLegacyCvars();
 	Browser_Init2();
 	ConfigManager_Init();
 	ResetBinds();
+	Cfg_ExecuteDefaultConfig();
+	Cbuf_Execute();
 
-	i = COM_CheckParm("+cfg_load");
+	i = COM_FindParm("+cfg_load");
 
 	if (i && (i + 1 < COM_Argc())) {
 		cfg_name = COM_Argv(i + 1);
@@ -632,24 +702,20 @@ void Host_Init (int argc, char **argv, int default_memsize)
 	SV_Init ();
 #endif
 	CL_Init ();
+	Central_Init();
 
 	Cvar_CleanUpTempVars ();
 
 	SYSINFO_Init();
 
-#ifdef WITH_TCL
-	if (!TCL_InterpLoaded())
-		Com_Printf_State (PRINT_FAIL, "Could not load "TCL_LIB_NAME", embedded Tcl disabled\n");
-#endif
-
 	Hunk_AllocName (0, "-HOST_HUNKLEVEL-");
 	host_hunklevel = Hunk_LowMark ();
 
 	host_initialized = true;
 
-	// walk through all vars and forse OnChange event if cvar was modified,
-	// also apply that to variables which mirrored in userinfo because of cl_parsefunchars was't applyed as this moment,
-	// same for serverinfo and may be this fix something also.
+	// walk through all vars and force OnChange event if cvar was modified,
+	// also apply that to variables which mirrored in userinfo because of cl_parsefunchars wasn't applied at this moment,
+	// same for serverinfo and maybe this fix something also.
 	for ( v = NULL; (v = Cvar_Next ( v )); ) {
 		char val[2048];
 
@@ -668,7 +734,7 @@ void Host_Init (int argc, char **argv, int default_memsize)
 	Com_Printf_State (PRINT_INFO, "Exe: "__DATE__" "__TIME__"\n");
 	Com_Printf_State (PRINT_INFO, "Hunk allocation: %4.1f MB\n", (float) host_memsize / (1024 * 1024));
 	Com_Printf("\n");
-	Com_Printf("http://ezquake.github.io/\n");
+	Com_Printf(EZ_VERSION_WEBSITE "\n");
 	Com_Printf("\n");
 //	Com_Printf(Host_PrintBars("ezQuake\x9c" "SourceForge\x9c" "net", 38));
 	Com_Printf("ezQuake %s\n", VersionStringColour());
@@ -683,7 +749,7 @@ void Host_Init (int argc, char **argv, int default_memsize)
 	}
 
 	Cmd_StuffCmds_f ();		// process command line arguments
-	Cbuf_AddText ("cl_warncmd 1\n");
+	Cbuf_AddText("cl_warncmd 1\n");
 
 	Sys_CheckQWProtocolHandler();
 
@@ -692,17 +758,23 @@ void Host_Init (int argc, char **argv, int default_memsize)
 	{
 		char cmd[1024] = {0};
 
-		if (COM_CheckArgsForPlayableFiles(cmd, sizeof(cmd)))
-		{
+		if (COM_CheckArgsForPlayableFiles(cmd, sizeof(cmd))) {
 			Cbuf_AddText(cmd);
 		}
-		else
-		{
+		else {
 			Startup_Place();
 		}
 	}
 
+	// Trigger changes config has made to defaults
+	Cbuf_Flush(&cbuf_main);
+	Cvar_ExecuteQueuedChanges();
+	Cbuf_Execute();
+
 	host_everything_loaded = true;
+#ifdef DEBUG_MEMORY_ALLOCATIONS
+	Sys_Printf("\nevent,init\n");
+#endif
 }
 
 //FIXME: this is a callback from Sys_Quit and Sys_Error.  It would be better
@@ -726,17 +798,23 @@ void Host_Shutdown (void)
 	SV_Shutdown ("Server quit\n");
 #endif
 
+	Central_Shutdown();
 	CL_Shutdown ();
 	NET_Shutdown ();
 	Con_Shutdown();
-#ifdef WITH_TCL
-	TCL_Shutdown ();
-#endif
 	qtvlist_deinit();
+	Cvar_Shutdown();
+	FS_Shutdown();
+	SYSINFO_Shutdown();
+	Q_free(com_args_original);
 }
 
 void Host_Quit (void)
 {
+#ifdef DEBUG_MEMORY_ALLOCATIONS
+	Sys_Printf("\nevent,quit\n");
+#endif
+
 	// execute user's trigger
 	TP_ExecTrigger ("f_exit");
 	Cbuf_Execute();
@@ -746,5 +824,8 @@ void Host_Quit (void)
 
 	// turn off
 	Host_Shutdown ();
+
+	Q_free(host_membase);
+
 	Sys_Quit ();
 }
