@@ -31,8 +31,14 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #endif
 
 #ifdef WITH_JPEG
-#include "jpeglib.h"
-#include "jerror.h"
+#if defined(_MSC_VER)
+#pragma warning(disable: 4005)
+#endif
+#include <jpeglib.h>
+#include <jerror.h>
+#if defined(_MSC_VER)
+#pragma warning(default: 4005)
+#endif
 /*#ifdef _WIN32
 #pragma comment(lib, "libs/libjpeg.lib")
 #endif*/
@@ -523,19 +529,19 @@ static int apng_compression;
 static void PNG_IO_user_read_data(png_structp png_ptr, png_bytep data, png_size_t length) {
 	vfsfile_t *v = (vfsfile_t *) png_get_io_ptr(png_ptr);
 	vfserrno_t err;
-	VFS_READ(v, data, length, &err);
+	VFS_READ(v, data, (int)length, &err);
 }
 
 static void PNG_IO_user_write_data(png_structp png_ptr, png_bytep data, png_size_t length) {
 	vfsfile_t *v = (vfsfile_t *) png_get_io_ptr(png_ptr);
 
-	VFS_WRITE(v, data, length);
+	VFS_WRITE(v, data, (int)length);
 }
 
 #ifdef WITH_APNG
 static byte* apng_data = NULL;
-static unsigned int apng_data_limit = 0;
-static unsigned int apng_data_length = 0;
+static size_t apng_data_limit = 0;
+static size_t apng_data_length = 0;
 
 static void PNG_IO_user_write_data_apng_discard(png_structp png_ptr, png_bytep data, png_size_t length)
 {
@@ -598,6 +604,37 @@ static qbool PNG_HasHeader (vfsfile_t *fin)
 	return true;
 }
 
+static void Image_PngErrorHandler(png_structp png_ptr, png_const_charp error_msg)
+{
+	const char* filename = (const char*)png_get_error_ptr(png_ptr);
+
+	if (filename == NULL || !filename[0]) {
+		filename = "(unknown path)";
+	}
+	if (error_msg == NULL || !error_msg[0]) {
+		error_msg = "unknown error";
+	}
+
+	Sys_Error("Invalid PNG detected: %s (%s)\n", filename, error_msg);
+}
+
+static void Image_PngWarningHandler(png_structp png_ptr, png_const_charp error_msg)
+{
+	const char* filename = (const char*)png_get_error_ptr(png_ptr);
+
+	if (filename == NULL || !filename[0]) {
+		filename = "(unknown path)";
+	}
+	if (error_msg == NULL || !error_msg[0]) {
+		error_msg = "unknown error";
+	}
+	if (strstr(error_msg, "known incorrect sRGB profile")) {
+		return; // only matters if we would subsequently save the .png
+	}
+
+	Con_Printf("&cdd0libpng&r: %s (%s)\n", filename, error_msg);
+}
+
 png_data *Image_LoadPNG_All (vfsfile_t *fin, const char *filename, int matchwidth, int matchheight, int loadflag, int *real_width, int *real_height)
 {
 	byte **rowpointers = NULL;
@@ -609,7 +646,7 @@ png_data *Image_LoadPNG_All (vfsfile_t *fin, const char *filename, int matchwidt
 	png_data *png_return_val = NULL;	// Return struct containing data + text chunks.
 	int n_textcount = 0;				// Number of text chunks.
 	int y, width, height, bitdepth, colortype, interlace, compression, filter, bytesperpixel;
-	unsigned long rowbytes;
+	png_size_t rowbytes;
 
 	// Check if we were given a non-null file pointer
 	// if so then use it, otherwise try to open the specified filename.
@@ -626,7 +663,7 @@ png_data *Image_LoadPNG_All (vfsfile_t *fin, const char *filename, int matchwidt
 	}
 
 	// Try creating a PNG structure for reading the file.
-	if (!(png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL))) 
+	if (!(png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, (void*)filename, Image_PngErrorHandler, Image_PngWarningHandler))) 
 	{
 		VFS_CLOSE(fin);
 		fin = NULL;
@@ -642,7 +679,7 @@ png_data *Image_LoadPNG_All (vfsfile_t *fin, const char *filename, int matchwidt
 		return NULL;
 	}
 
-	// Set the return adress that PNGLib should return to if
+	// Set the return address that PNGLib should return to if
 	// an error occurs during reading.
 #if 0
 	if (setjmp(png_ptr->jmpbuf)) 
@@ -709,7 +746,7 @@ png_data *Image_LoadPNG_All (vfsfile_t *fin, const char *filename, int matchwidt
 
 		if (colortype == PNG_COLOR_TYPE_GRAY && bitdepth < 8) 
 		{
-#ifdef __Q_PNG14__
+#if PNG_LIBPNG_VER >= 10209
 			png_set_expand_gray_1_2_4_to_8(png_ptr);
 #else
 			png_set_gray_1_2_4_to_8(png_ptr);
@@ -774,7 +811,7 @@ png_data *Image_LoadPNG_All (vfsfile_t *fin, const char *filename, int matchwidt
 		}
 
 		// Allocate a byte array for the actual image data.
-		data = (byte *) Q_malloc(height * rowbytes );
+		data = (byte *) Q_malloc_named(height * rowbytes, filename);
 
 		// Even though we just allocated the memory for all the image data
 		// PNG lib wants this in the form of rowpointers.
@@ -787,6 +824,8 @@ png_data *Image_LoadPNG_All (vfsfile_t *fin, const char *filename, int matchwidt
 		// Read the actual image data.
 		png_read_image(png_ptr, rowpointers);
 		png_read_end(png_ptr, pnginfo);
+
+		Q_free(rowpointers);
 	}
 	
 	//
@@ -826,7 +865,6 @@ png_data *Image_LoadPNG_All (vfsfile_t *fin, const char *filename, int matchwidt
 
 	// Clean up.
 	png_destroy_read_struct (&png_ptr, &pnginfo, NULL);
-	Q_free (rowpointers);
 	VFS_CLOSE(fin);
 	fin = NULL;
 
@@ -855,9 +893,10 @@ png_textp Image_LoadPNG_Comments (char *filename, int *text_count)
 	if (pdata)
 	{		
 		// Return text chunks + count.
-		(*text_count) = pdata->text_count;
+		(*text_count) = (int)pdata->text_count;
 		textchunks = pdata->textchunks;
 
+		Q_free(pdata->data);
 		Q_free(pdata);
 	}
 
@@ -882,10 +921,10 @@ byte *Image_LoadPNG (vfsfile_t *fin, const char *filename, int matchwidth, int m
 	return data;
 }
 
-int Image_WritePNG (char *filename, int compression, byte *pixels, int width, int height) 
+int Image_WritePNG(char *filename, int compression, byte *pixels, size_t width, size_t height)
 {
 	char name[MAX_PATH];
-	int i, bpp = 3, pngformat, width_sign;
+	int i, bpp = 3, pngformat;
 	vfsfile_t *fp;
 
 	png_structp png_ptr;
@@ -893,9 +932,6 @@ int Image_WritePNG (char *filename, int compression, byte *pixels, int width, in
 	png_byte **rowpointers;
 	snprintf (name, sizeof(name), "%s", filename);
 
-	width_sign = (width < 0) ? -1 : 1;
-	width = abs(width);
-
 	if (!(fp = FS_OpenVFS(name, "wb", FS_NONE_OS))) {
 		FS_CreatePath (name);
 		if (!(fp = FS_OpenVFS(name, "wb", FS_NONE_OS)))
@@ -925,14 +961,15 @@ int Image_WritePNG (char *filename, int compression, byte *pixels, int width, in
 	png_set_compression_level(png_ptr, bound(Z_NO_COMPRESSION, compression, Z_BEST_COMPRESSION));
 
 	pngformat = (bpp == 4) ? PNG_COLOR_TYPE_RGBA : PNG_COLOR_TYPE_RGB;
-	png_set_IHDR(png_ptr, info_ptr, width, height, 8, pngformat,
+	png_set_IHDR(png_ptr, info_ptr, (png_uint_32)width, (png_uint_32)height, 8, pngformat,
 		PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT);
 
 	png_write_info(png_ptr, info_ptr);
 
 	rowpointers = (png_byte **) Q_malloc (height * sizeof(*rowpointers));
-	for (i = 0; i < height; i++)
-		rowpointers[i] = pixels + i * width_sign * width * bpp;
+	for (i = 0; i < height; i++) {
+		rowpointers[i] = pixels + (height - i - 1) * width * bpp;
+	}
 	png_write_image(png_ptr, rowpointers);
 	png_write_end(png_ptr, info_ptr);
 	Q_free(rowpointers);
@@ -990,13 +1027,10 @@ qbool Image_OpenAPNG(char* filename, int compression, int width, int height, int
 	return true;
 }
 
-qbool Image_WriteAPNGFrame(byte* pixels, int width, int height, int fps)
+qbool Image_WriteAPNGFrame(byte* pixels, size_t width, size_t height, int fps)
 {
 	png_byte **rowpointers = (png_byte **)Q_malloc(height * sizeof(*rowpointers));
-	int i, width_sign, bpp = 3;
-
-	width_sign = (width < 0) ? -1 : 1;
-	width = abs(width);
+	int i, bpp = 3;
 
 	// Write fcTL chunk
 	{
@@ -1014,8 +1048,8 @@ qbool Image_WriteAPNGFrame(byte* pixels, int width, int height, int fps)
 		byte header[4] = { 'f', 'c', 'T', 'L' };
 
 		fcTL_chunk.sequence_number = htonl(apng_framenumber);
-		fcTL_chunk.width = htonl(width);
-		fcTL_chunk.height = htonl(height);
+		fcTL_chunk.width = htonl((u_long)width);
+		fcTL_chunk.height = htonl((u_long)height);
 		fcTL_chunk.x_offset = htonl(0);
 		fcTL_chunk.y_offset = htonl(0);
 		fcTL_chunk.delay_num = htons(1);
@@ -1029,7 +1063,7 @@ qbool Image_WriteAPNGFrame(byte* pixels, int width, int height, int fps)
 	}
 
 	for (i = 0; i < height; i++) {
-		rowpointers[i] = pixels + i * width_sign * width * bpp;
+		rowpointers[i] = pixels + (height - i - 1) * width * bpp;
 	}
 	if (apng_framenumber >= 2) {
 		// Create a pretend 'new' .png so the IDAT is correct
@@ -1043,7 +1077,7 @@ qbool Image_WriteAPNGFrame(byte* pixels, int width, int height, int fps)
 
 			png_set_write_fn(fake_apng_ptr, NULL, PNG_IO_user_write_data_apng_discard, PNG_IO_user_flush_data_apng_discard);
 			png_set_compression_level(fake_apng_ptr, bound(Z_NO_COMPRESSION, apng_compression, Z_BEST_COMPRESSION));
-			png_set_IHDR(fake_apng_ptr, fake_apng_info_ptr, width, height, 8, PNG_COLOR_TYPE_RGB, PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT);
+			png_set_IHDR(fake_apng_ptr, fake_apng_info_ptr, (png_uint_32)width, (png_uint_32)height, 8, PNG_COLOR_TYPE_RGB, PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT);
 			png_write_info(fake_apng_ptr, fake_apng_info_ptr);
 
 			png_write_flush(fake_apng_ptr);
@@ -1087,13 +1121,17 @@ qbool Image_CloseAPNG(void)
 
 	return true;
 }
-#else // WITH_APNG
+#endif // WITH_APNG
+#endif // WITH_PNG
+
+#ifndef WITH_APNG
+// This might be because IO-state isn't supported, or we're building without PNG support
 qbool Image_OpenAPNG(char* filename, int compression, int width, int height, int frames)
 {
 	return false;
 }
 
-qbool Image_WriteAPNGFrame(byte* pixels, int width, int height, int fps)
+qbool Image_WriteAPNGFrame(byte* pixels, size_t width, size_t height, int fps)
 {
 	return false;
 }
@@ -1104,8 +1142,6 @@ qbool Image_CloseAPNG(void)
 }
 #endif
 
-#endif // WITH_PNG
-
 /************************************ TGA ************************************/
 
 // Definitions for image types
@@ -1320,7 +1356,7 @@ byte *Image_LoadTGA(vfsfile_t *fin, const char *filename, int matchwidth, int ma
 	return data;
 }
 
-int Image_WriteTGA (char *filename, byte *pixels, int width, int height) 
+int Image_WriteTGA (char *filename, byte *pixels, size_t width, size_t height)
 {
 	char name[MAX_PATH];
 	byte buffer[18] = { 0 };
@@ -1328,9 +1364,9 @@ int Image_WriteTGA (char *filename, byte *pixels, int width, int height)
 
 	buffer[2] = 2;          // uncompressed type
 	buffer[12] = width & 255;
-	buffer[13] = width >> 8;
+	buffer[13] = (width >> 8) & 0xFF;
 	buffer[14] = height & 255;
-	buffer[15] = height >> 8;
+	buffer[15] = (height >> 8) & 0xFF;
 	buffer[16] = 24;
 
 	snprintf (name, sizeof(name), "%s", filename);
@@ -1342,7 +1378,7 @@ int Image_WriteTGA (char *filename, byte *pixels, int width, int height)
 	}
 
 	VFS_WRITE(outfile, buffer, sizeof(buffer));
-	VFS_WRITE(outfile, pixels, width * height * 3);
+	VFS_WRITE(outfile, pixels, (int)(width * height * 3));
 	VFS_CLOSE(outfile);
 	return true;
 }
@@ -1350,7 +1386,6 @@ int Image_WriteTGA (char *filename, byte *pixels, int width, int height)
 /*********************************** JPEG ************************************/
 
 #ifdef WITH_JPEG
-
 #define jpeg_create_compress(cinfo) \
     jpeg_CreateCompress((cinfo), JPEG_LIB_VERSION, (size_t) sizeof(struct jpeg_compress_struct))
 
@@ -1383,11 +1418,11 @@ static boolean JPEG_IO_empty_output_buffer (j_compress_ptr cinfo)
 	if (VFS_WRITE(dest->outfile, dest->buffer, JPEG_OUTPUT_BUF_SIZE) != JPEG_OUTPUT_BUF_SIZE)
 	{
 		jpeg_in_error = true;
-		return false;
+		return (boolean)false;
 	}
 	dest->pub.next_output_byte = dest->buffer;
 	dest->pub.free_in_buffer = JPEG_OUTPUT_BUF_SIZE;
-	return true;
+	return (boolean)true;
 }
 
 static void JPEG_IO_term_destination (j_compress_ptr cinfo)
@@ -1396,7 +1431,7 @@ static void JPEG_IO_term_destination (j_compress_ptr cinfo)
 	size_t datacount = JPEG_OUTPUT_BUF_SIZE - dest->pub.free_in_buffer;
 
 	if (datacount > 0) {
-		if (((size_t) VFS_WRITE(dest->outfile, dest->buffer, datacount)) != datacount)
+		if (((size_t) VFS_WRITE(dest->outfile, dest->buffer, (int)datacount)) != datacount)
 		{
 			jpeg_in_error = true;
 			return;
@@ -1463,8 +1498,8 @@ int Image_WriteJPEG(char *filename, int quality, byte *pixels, int width, int he
 	cinfo.input_components = 3;
 	cinfo.in_color_space = JCS_RGB;
 	jpeg_set_defaults(&cinfo);
-	jpeg_set_quality (&cinfo, bound(0, quality, 100), true);
-	jpeg_start_compress(&cinfo, true);
+	jpeg_set_quality (&cinfo, bound(0, quality, 100), (boolean)true);
+	jpeg_start_compress(&cinfo, (boolean)true);
 
 	while (cinfo.next_scanline < height) {
 	    *row_pointer = &pixels[(int)cinfo.next_scanline * width * 3];
@@ -1741,7 +1776,6 @@ badjpeg:
 	Q_free(infile);
 	return mem;
 }
-
 #endif // WITH_JPEG
 
 /************************************ PCX ************************************/
@@ -1976,13 +2010,13 @@ void Image_Init(void)
 {
 	Cvar_SetCurrentGroup(CVAR_GROUP_SCREENSHOTS);
 
-#ifdef WITH_PNG
+	#ifdef WITH_PNG
 	Cvar_Register (&image_png_compression_level);
-#endif // WITH_PNG
+	#endif // WITH_PNG
 
-#ifdef WITH_JPEG
+	#ifdef WITH_JPEG
 	Cvar_Register (&image_jpeg_quality_level);
-#endif // WITH_JPEG
+	#endif // WITH_JPEG
 
 	Cvar_ResetCurrentGroup();
 }
