@@ -32,7 +32,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #endif
 
 #include "gl_model.h"
-#include "gl_local.h"
 #include "tr_types.h"
 
 #include "hud.h"
@@ -544,31 +543,43 @@ void CompleteCommandNew (void)
 
 			if (c)
 			{
-				if (con_completion_format.integer)
-					Com_Printf ("&c%sCommands (%d):&r\n", con_completion_color_title.string, c);
-				else
-					Com_Printf ("\x02" "Commands (%d):\n", c);
+				legacycmd_t* lcmd;
 
-				for (s_c = cmd_functions, s_count = 0; s_c && s_count < MAX_SORTED_CMDS; s_c = s_c->next, s_count++)
-					sorted_cmds[s_count] = s_c;
+				if (con_completion_format.integer) {
+					Com_Printf("&c%sCommands (%d):&r\n", con_completion_color_title.string, c);
+				}
+				else {
+					Com_Printf("\x02" "Commands (%d):\n", c);
+				}
 
-				qsort(sorted_cmds, s_count, sizeof (cmd_function_t *), Cmd_CommandCompare);
 				escaped = escape_regex(s);
 				snprintf(completebuff, sizeof(completebuff), "^((?i)%s)", escaped);
 				Q_free(escaped);
 
-				for (i = 0; i < s_count; i++) {
-					if (Utils_RegExpMatch(completebuff, sorted_cmds[i]->name))
-					{
-						PaddedPrint (sorted_cmds[i]->name);
-						FindCommonSubString (sorted_cmds[i]->name);
-						jogi_avail_complete[count].name = sorted_cmds[i]->name;
-						jogi_avail_complete[count].type = "command";
-						count++;
-						count_cmd++;
+				for (s_c = cmd_functions, s_count = 0; s_c && s_count < MAX_SORTED_CMDS; s_c = s_c->next) {
+					if (Utils_RegExpMatch(completebuff, s_c->name)) {
+						sorted_cmds[s_count++] = s_c;
 					}
 				}
 
+				// Legacy commands
+				for (lcmd = legacycmds; lcmd; lcmd = lcmd->next) {
+					if (Utils_RegExpMatch(completebuff, lcmd->dummy_cmd.name)) {
+						sorted_cmds[s_count++] = &lcmd->dummy_cmd;
+					}
+				}
+
+				qsort(sorted_cmds, s_count, sizeof (cmd_function_t *), Cmd_CommandCompare);
+
+				for (i = 0; i < s_count; i++) {
+					PaddedPrint (sorted_cmds[i]->name);
+					FindCommonSubString (sorted_cmds[i]->name);
+					jogi_avail_complete[count].name = sorted_cmds[i]->name;
+					jogi_avail_complete[count].type = "command";
+					count++;
+					count_cmd++;
+				}
+
 				if (con.x)
 					Com_Printf ("\n");
 			}
@@ -580,26 +591,27 @@ void CompleteCommandNew (void)
 				else
 					Com_Printf ("\x02" "Variables (%d):\n", v);
 
-				for (s_v = cvar_vars, s_count = 0; s_v && s_count < MAX_SORTED_CVARS; s_v = s_v->next, s_count++)
-					sorted_cvars[s_count] = s_v;
-
-				qsort(sorted_cvars, s_count, sizeof (cvar_t *), Cvar_CvarCompare);
 				escaped = escape_regex(s);
 				snprintf(completebuff, sizeof(completebuff), "^((?i)%s)", escaped);
 				Q_free(escaped);
 
-				for (i = 0; i < s_count; i++) {
-					if (Utils_RegExpMatch(completebuff, sorted_cvars[i]->name))
-					{
-						PaddedPrintValue (sorted_cvars[i]->name, sorted_cvars[i]->string, sorted_cvars[i]->defaultvalue);
-						FindCommonSubString (sorted_cvars[i]->name);
-						jogi_avail_complete[count].name = sorted_cvars[i]->name;
-						jogi_avail_complete[count].type = "variable";
-						count++;
-						count_cvar++;
+				for (s_v = cvar_vars, s_count = 0; s_v && s_count < MAX_SORTED_CVARS; s_v = s_v->next) {
+					if (Utils_RegExpMatch(completebuff, s_v->name)) {
+						sorted_cvars[s_count++] = s_v;
 					}
 				}
 
+				qsort(sorted_cvars, s_count, sizeof (cvar_t *), Cvar_CvarCompare);
+
+				for (i = 0; i < s_count; i++) {
+					PaddedPrintValue (sorted_cvars[i]->name, sorted_cvars[i]->string, sorted_cvars[i]->defaultvalue);
+					FindCommonSubString (sorted_cvars[i]->name);
+					jogi_avail_complete[count].name = sorted_cvars[i]->name;
+					jogi_avail_complete[count].type = "variable";
+					count++;
+					count_cvar++;
+				}
+
 				if (con.x)
 					Com_Printf ("\n");
 			}
@@ -611,28 +623,30 @@ void CompleteCommandNew (void)
 				else
 					Com_Printf ("\x02" "Aliases (%d):\n", a);
 
-				for (s_a = cmd_alias, s_count = 0; s_a && s_count < MAX_SORTED_ALIASES; s_a = s_a->next, s_count++)
-					sorted_aliases[s_count] = s_a;
-
-				qsort(sorted_aliases, s_count, sizeof (cmd_alias_t *), Cmd_AliasCompare);				
 				escaped = escape_regex(s);
-				snprintf(completebuff, sizeof(completebuff), "^((?i)%s)", escaped);				
+				snprintf(completebuff, sizeof(completebuff), "^((?i)%s)", escaped);
 				Q_free(escaped);
 
-				for (i = 0; i < s_count; i++) {
-					if (Utils_RegExpMatch(completebuff, sorted_aliases[i]->name))
-					{
-						PaddedPrintValue(sorted_aliases[i]->name, sorted_aliases[i]->value, sorted_aliases[i]->name);
-						FindCommonSubString(sorted_aliases[i]->name);
-						jogi_avail_complete[count].name = sorted_aliases[i]->name;
-						jogi_avail_complete[count].type = "alias";
-						count++;
-						count_alias++;
+				for (s_a = cmd_alias, s_count = 0; s_a && s_count < MAX_SORTED_ALIASES; s_a = s_a->next) {
+					if (Utils_RegExpMatch(completebuff, s_a->name)) {
+						sorted_aliases[s_count++] = s_a;
 					}
 				}
 
-				if (con.x)
-					Com_Printf ("\n");
+				qsort(sorted_aliases, s_count, sizeof (cmd_alias_t *), Cmd_AliasCompare);				
+
+				for (i = 0; i < s_count; i++) {
+					PaddedPrintValue(sorted_aliases[i]->name, sorted_aliases[i]->value, sorted_aliases[i]->name);
+					FindCommonSubString(sorted_aliases[i]->name);
+					jogi_avail_complete[count].name = sorted_aliases[i]->name;
+					jogi_avail_complete[count].type = "alias";
+					count++;
+					count_alias++;
+				}
+
+				if (con.x) {
+					Com_Printf("\n");
+				}
 			}
 		}
 
@@ -1982,7 +1996,7 @@ void Mouse_MoveEvent(void)
 	Mouse_EventDispatch();  // so just dispatch the message with new state
 }
 
-static qbool Key_IsConsoleToggle (int key)
+qbool Key_IsConsoleToggle (int key)
 {
 	return (key == '`' || key == '~');
 }
@@ -1996,11 +2010,12 @@ static qbool Key_ConsoleKey(int key)
 	qbool con_key = (con_tilde_mode.integer && con_toggle && (con_tilde_mode.integer == 1 || !CONSOLE_LINE_EMPTY())) ? true : consolekeys[key];
 	qbool hud_key = (con_tilde_mode.integer && con_toggle) ? true : hudeditorkeys[key];
 	qbool demo_controls_key = (con_tilde_mode.integer && con_toggle) ? true : democontrolskey[key];
+	qbool con_keypad_key = cl_keypad.integer && key >= KP_HOME && key <= KP_ENTER;
 
 	if (key_dest == key_menu && Menu_ExecuteKey(key))
 		return false;
 
-	if ((key_dest == key_console || key_dest == key_message) && !con_key)
+	if ((key_dest == key_console || key_dest == key_message) && !con_key && !con_keypad_key)
 		return false;
 
 	if (key_dest == key_game && (cls.state == ca_active || !con_key))
@@ -2066,6 +2081,11 @@ void Key_EventEx (int key, wchar unichar, qbool down)
 			return;
 		}
 
+		if (KeyDestStartupDemo(key_dest)) {
+			Cbuf_AddText("togglemenu\n");
+			return;
+		}
+
 		switch (key_dest) 
 		{
 			case key_message:
@@ -2078,10 +2098,12 @@ void Key_EventEx (int key, wchar unichar, qbool down)
 				M_ToggleMenu_f ();
 				break;
 			case key_console:
-				if (!SCR_NEED_CONSOLE_BACKGROUND)
-					Con_ToggleConsole_f ();
-				else
-					M_ToggleMenu_f ();
+				if (!SCR_NEED_CONSOLE_BACKGROUND) {
+					Con_ToggleConsole_f();
+				}
+				else {
+					M_ToggleMenu_f();
+				}
 				break;
 			case key_hudeditor:
 				HUD_Editor_Key(key, unichar, down);
@@ -2089,9 +2111,7 @@ void Key_EventEx (int key, wchar unichar, qbool down)
 			case key_demo_controls:
 				DemoControls_KeyEvent(key, unichar, down);
 				break;
-			case key_startupdemo:
-				Cbuf_AddText("disconnect;togglemenu\n");
-				break;
+
 			default:
 				assert(!"Bad key_dest");
 		}
@@ -2233,8 +2253,15 @@ void Key_EventEx (int key, wchar unichar, qbool down)
 			DemoControls_KeyEvent(key, unichar, down);
 			break;
 
-		case key_startupdemo:
-			Cbuf_AddText("disconnect;togglemenu\n");
+		case key_startupdemo_browser:
+			Cbuf_AddText("menu_slist\n");
+			break;
+		case key_startupdemo_console:
+		case key_startupdemo_game:
+			key_dest = key_console;
+			break;
+		case key_startupdemo_menu:
+			Cbuf_AddText("togglemenu\n");
 			break;
 
 		default:
@@ -2376,3 +2403,11 @@ char* escape_regex(char* string)
 	return out;
 }
 
+void Key_Shutdown(void)
+{
+	int i;
+
+	for (i = 0; i < sizeof(keybindings) / sizeof(keybindings[0]); ++i) {
+		Q_free(keybindings[i]);
+	}
+}
