@@ -23,7 +23,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #endif
 #include "quakedef.h"
 #include "gl_model.h"
-#include "gl_local.h"
 #ifndef CLIENTONLY
 #include "server.h"
 #endif
@@ -41,7 +40,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "qsound.h"
 #include "keys.h"
 #include "common_draw.h"
-
+#include "r_matrix.h"
 
 qbool vid_windowedmouse = true;
 void (*vid_menudrawfn)(void);
@@ -91,13 +90,10 @@ void M_Main_Key (int key);
 
 void M_Menu_Help_f (void);
 
-#define QUAKE_ID_PLAQUE_PATH	"gfx/qplaque.lmp"
-
 m_state_t m_state;
 
 qbool    m_entersound;          // play after drawing a frame, so caching
                                 // won't disrupt the sound
-qbool    m_recursiveDraw;
 int            m_topmenu;       // set if a submenu was entered via a
                                 // menu_* command
 #define    SLIDER_RANGE    10
@@ -118,6 +114,8 @@ int        menuheight = 240;
 
 cvar_t     scr_centerMenu = {"scr_centerMenu","1"};
 cvar_t     menu_ingame = {"menu_ingame", "1"};
+cvar_t     menu_botmatch_gamedir = { "menu_botmatch_gamedir", "fbca" };
+cvar_t     menu_botmatch_mod_old = { "menu_botmatch_mod_old", "1" };
 int        m_yofs = 0;
 
 void M_DrawCharacter (int cx, int line, int num) {
@@ -129,10 +127,12 @@ void M_Print_GetPoint(int cx, int cy, int *rx, int *ry, const char *str, qbool r
 	cy += m_yofs;
 	*rx = cx;
 	*ry = cy;
-	if (red)
-		Draw_Alt_String (cx, cy, str);
-	else
+	if (red) {
+		Draw_Alt_String(cx, cy, str, 1, false);
+	}
+	else {
 		Draw_String(cx, cy, str);
+	}
 }
 
 void M_Print (int cx, int cy, char *str) {
@@ -217,9 +217,7 @@ void M_Unscale_Menu(void)
 	{
 		menuwidth = vid.width;
 		menuheight = vid.height;
-		glMatrixMode(GL_PROJECTION);
-		glLoadIdentity ();
-		glOrtho  (0, menuwidth, menuheight, 0, -99999, 99999);
+		R_OrthographicProjection(0, menuwidth, menuheight, 0, -99999, 99999);
 	}
 }
 
@@ -426,10 +424,10 @@ void M_Main_Draw (void) {
 	mpic_t *p;
 	int itemheight;
 
-	M_DrawTransPic (16, BIGMENU_TOP, Draw_CachePic (QUAKE_ID_PLAQUE_PATH) );
+	M_DrawTransPic (16, BIGMENU_TOP, Draw_CachePic (CACHEPIC_QPLAQUE) );
 
 	// the Main Manu heading
-	p = Draw_CachePic ("gfx/ttl_main.lmp");
+	p = Draw_CachePic (CACHEPIC_TTL_MAIN);
 	M_DrawPic ( (320-p->width)/2, 4, p);
 
 	// Main Menu items
@@ -443,7 +441,7 @@ void M_Main_Draw (void) {
 	}
 	else {
 		newmainmenu = false;
-		p = Draw_CachePic ("gfx/mainmenu.lmp");
+		p = Draw_CachePic (CACHEPIC_MAINMENU);
 		m_main_window.w = p->width;
 		m_main_window.h = p->height;
 		M_DrawTransPic_GetPoint (72, 32, &m_main_window.x, &m_main_window.y, p);
@@ -456,7 +454,8 @@ void M_Main_Draw (void) {
 	}	
 
 	M_DrawTransPic (54, BIGMENU_TOP + m_main_cursor * itemheight,
-		Draw_CachePic(va("gfx/menudot%i.lmp", f+1)) );
+		Draw_CachePic(CACHEPIC_MENUDOT1 + f)
+	);
 }
 
 static void M_Main_Enter(const unsigned int entry)
@@ -728,8 +727,8 @@ void M_SinglePlayer_Draw (void) {
 		return;
 	}
 
-	M_DrawTransPic (16, BIGMENU_TOP, Draw_CachePic (QUAKE_ID_PLAQUE_PATH) );
-	p = Draw_CachePic ("gfx/ttl_sgl.lmp");
+	M_DrawTransPic (16, BIGMENU_TOP, Draw_CachePic (CACHEPIC_QPLAQUE) );
+	p = Draw_CachePic (CACHEPIC_TTL_SGL);
 	M_DrawPic ( (320-p->width)/2, 4, p);
 
 	if (Draw_BigFontAvailable()) {
@@ -743,7 +742,7 @@ void M_SinglePlayer_Draw (void) {
 	}
 	else {
 		m_singleplayer_big = false;
-		p = Draw_CachePic ("gfx/sp_menu.lmp");
+		p = Draw_CachePic (CACHEPIC_SP_MENU);
 		m_singleplayer_window.w = p->width;
 		m_singleplayer_window.h = p->height;
 		M_DrawTransPic_GetPoint(72, 32, &m_singleplayer_window.x, &m_singleplayer_window.y, p);
@@ -751,8 +750,8 @@ void M_SinglePlayer_Draw (void) {
 	}
 
 	M_DrawTransPic (54, BIGMENU_TOP + m_singleplayer_cursor * itemheight,
-		Draw_CachePic( va("gfx/menudot%i.lmp", f+1 ) ) );
-
+		Draw_CachePic(CACHEPIC_MENUDOT1 + f)
+	);
 }
 
 #ifndef WITH_NQPROGS
@@ -767,28 +766,27 @@ static void CheckSPGame (void) {
 }
 #endif	// !WITH_NQPROGS
 
-static void StartNewGame (void) {
+static void StartNewGame(void)
+{
 	extern cvar_t sv_progtype;
 
 	key_dest = key_game;
 	m_state = m_none;
-	Cvar_Set (&maxclients, "1");
-	Cvar_Set (&teamplay, "0");
-	Cvar_Set (&deathmatch, "0");
-	Cvar_Set (&coop, "0");
+	Cvar_Set(&maxclients, "1");
+	Cvar_Set(&teamplay, "0");
+	Cvar_Set(&deathmatch, "0");
+	Cvar_Set(&coop, "0");
 
-	Cvar_Set (&sv_progsname, "spprogs"); // force progsname
+	Cvar_Set(&sv_progsname, "spprogs"); // force progsname
 #ifdef USE_PR2
-	Cvar_Set (&sv_progtype, "0"); // force .dat
+	Cvar_Set(&sv_progtype, "0"); // force .dat
 #endif
 
-	if (com_serveractive)
-		Cbuf_AddText ("disconnect\n");
+	if (com_serveractive) {
+		Cbuf_AddText("disconnect\n");
+	}
 
-	progs = (dprograms_t *) FS_LoadHunkFile ("spprogs.dat", NULL);
-	//if (progs && !file_from_gamedir)
-	//	Cbuf_AddText ("gamedir qw\n");
-	Cbuf_AddText ("map start\n");
+	Cbuf_AddText("map start\n");
 }
 
 void M_SinglePlayer_Key (int key) {
@@ -905,10 +903,10 @@ void M_Menu_SinglePlayer_f (void) {
 void M_SinglePlayer_Draw (void) {
 	mpic_t *p;
 
-	M_DrawTransPic (16, 4, Draw_CachePic (QUAKE_ID_PLAQUE_PATH) );
-	p = Draw_CachePic ("gfx/ttl_sgl.lmp");
+	M_DrawTransPic (16, 4, Draw_CachePic (CACHEPIC_QPLAQUE) );
+	p = Draw_CachePic (CACHEPIC_TTL_SGL);
 	M_DrawPic ( (320-p->width)/2, 4, p);
-	//    M_DrawTransPic (72, 32, Draw_CachePic ("gfx/sp_menu.lmp") );
+	//    M_DrawTransPic (72, 32, Draw_CachePic (CACHEPIC_SP_MENU) );
 
 	M_DrawTextBox (60, 10*8, 23, 4);
 	M_PrintWhite (88, 12*8, "This client is for");
@@ -941,31 +939,43 @@ void M_SinglePlayer_Key (int key) {
 
 #define    MAX_SAVEGAMES        12
 
+const char* save_filenames[] = {
+	"s0.sav", "s1.sav", "s2.sav", "s3.sav", "s4.sav", "s5.sav", "s6.sav", "s7.sav", "s8.sav", "s9.sav", "s10.sav", "s11.sav"
+};
+#ifdef C_ASSERT
+C_ASSERT(sizeof(save_filenames) / sizeof(save_filenames[0]) == MAX_SAVEGAMES);
+#endif
+
 int        load_cursor;        // 0 < load_cursor < MAX_SAVEGAMES
 char    m_filenames[MAX_SAVEGAMES][SAVEGAME_COMMENT_LENGTH + 1];
 int        loadable[MAX_SAVEGAMES];
 menu_window_t load_window, save_window;
 
-void M_ScanSaves (char *sp_gamedir) {
+void M_ScanSaves(char* sp_gamedir)
+{
 	int i, j;
 	char name[MAX_OSPATH];
-	vfsfile_t *f;
+	vfsfile_t* f;
 
 	for (i = 0; i < MAX_SAVEGAMES; i++) {
-		strlcpy (m_filenames[i], "--- UNUSED SLOT ---", SAVEGAME_COMMENT_LENGTH + 1);
 		loadable[i] = false;
+		strlcpy(m_filenames[i], "--- UNUSED SLOT ---", SAVEGAME_COMMENT_LENGTH + 1);
 
-		snprintf (name, sizeof(name), "save/s%i.sav", i);
-		if (!(f = FS_OpenVFS(name, "rb", FS_GAME_OS)))
+		FS_SaveGameDirectory(name, sizeof(name));
+		strlcat(name, save_filenames[i], sizeof(name));
+		if (!(f = FS_OpenVFS(name, "rb", FS_NONE_OS))) {
 			continue;
+		}
 		VFS_GETS(f, name, sizeof(name));
 		VFS_GETS(f, name, sizeof(name));
-		strlcpy (m_filenames[i], name, sizeof(m_filenames[i]));
+		strlcpy(m_filenames[i], name, sizeof(m_filenames[i]));
 
 		// change _ back to space
-		for (j = 0; j < SAVEGAME_COMMENT_LENGTH; j++)
-			if (m_filenames[i][j] == '_')
+		for (j = 0; j < SAVEGAME_COMMENT_LENGTH; j++) {
+			if (m_filenames[i][j] == '_') {
 				m_filenames[i][j] = ' ';
+			}
+		}
 		loadable[i] = true;
 		VFS_CLOSE(f);
 	}
@@ -997,7 +1007,7 @@ void M_Load_Draw (void) {
 	mpic_t *p;
 	int lx = 0, ly = 0;	// lower bounds of the window
 
-	p = Draw_CachePic ("gfx/p_load.lmp");
+	p = Draw_CachePic (CACHEPIC_P_LOAD);
 	M_DrawPic ( (320 - p->width) >> 1, 4, p);
 
 	for (i = 0; i < MAX_SAVEGAMES; i++)
@@ -1020,7 +1030,7 @@ void M_Save_Draw (void) {
 	mpic_t *p;
 	int lx = 0, ly = 0;	// lower bounds of the window
 
-	p = Draw_CachePic ("gfx/p_save.lmp");
+	p = Draw_CachePic (CACHEPIC_P_SAVE);
 	M_DrawPic ( (320 - p->width) >> 1, 4, p);
 
 	for (i = 0; i < MAX_SAVEGAMES; i++)
@@ -1062,9 +1072,7 @@ void M_Load_Key (int key) {
 			key_dest = key_game;
 
 			// issue the load command
-			if (FS_LoadHunkFile ("spprogs.dat", NULL) && !file_from_gamedir)
-				; //Cbuf_AddText("disconnect; gamedir qw\n");
-			Cbuf_AddText (va ("load s%i\n", load_cursor) );
+			Cbuf_AddText(va("load s%i\n", load_cursor));
 			return;
 
 		case K_UPARROW:
@@ -1164,8 +1172,8 @@ void M_MultiPlayerSub_Draw (void) {
 	mpic_t    *p;
 	int lx, ly;
 
-	M_DrawTransPic (16, 4, Draw_CachePic ("gfx/qplaque.lmp") );
-	p = Draw_CachePic ("gfx/p_multi.lmp");
+	M_DrawTransPic (16, 4, Draw_CachePic (CACHEPIC_QPLAQUE) );
+	p = Draw_CachePic (CACHEPIC_P_MULTI);
 	M_DrawPic ( (320-p->width)/2, 4, p);
 	M_Print_GetPoint (80, 40, &m_multiplayer_window.x, &m_multiplayer_window.y, "Join Game", m_multiplayer_cursor == 0);
 	m_multiplayer_window.h = 8;
@@ -1267,12 +1275,14 @@ void M_Init (void) {
 	extern cvar_t menu_marked_fade;
 
 	Cvar_SetCurrentGroup(CVAR_GROUP_MENU);
-	Cvar_Register (&scr_centerMenu);
-	Cvar_Register (&menu_ingame);
-	Cvar_Register (&scr_scaleMenu);
-	Cvar_Register (&menu_marked_fade);
+	Cvar_Register(&scr_centerMenu);
+	Cvar_Register(&menu_ingame);
+	Cvar_Register(&scr_scaleMenu);
+	Cvar_Register(&menu_marked_fade);
+	Cvar_Register(&menu_botmatch_gamedir);
+	Cvar_Register(&menu_botmatch_mod_old);
 
-	Cvar_Register (&menu_marked_bgcolor);
+	Cvar_Register(&menu_marked_bgcolor);
 	Browser_Init();
 	Cvar_ResetCurrentGroup();
 	Menu_Help_Init();	// help_files module
@@ -1299,62 +1309,72 @@ void M_Init (void) {
 	Cmd_AddCommand ("menu_quit", M_Menu_Quit_f);
 }
 
-void M_Draw (void) {
-	if (m_state == m_none || key_dest != key_menu || m_state == m_proxy)
+void M_Shutdown(void)
+{
+	Menu_Help_Shutdown();
+	Menu_Demo_Shutdown();
+	Menu_Options_Shutdown();
+	Menu_Ingame_Shutdown();
+	Menu_MultiPlayer_Shutdown();
+}
+
+void M_Draw(void)
+{
+	if (m_state == m_none || key_dest != key_menu || m_state == m_proxy) {
 		return;
-
-	if (!m_recursiveDraw) {
-		scr_copyeverything = 1;
-
-		if (SCR_NEED_CONSOLE_BACKGROUND) {
-			Draw_ConsoleBackground (scr_con_current);
-		} else {
-			// if you don't like fade in ingame menu, uncomment this
-			// if (m_state != m_ingame && m_state != m_democtrl)
-			Draw_FadeScreen (scr_menualpha.value);
-		}
-
-		scr_fullupdate = 0;
-	} else {
-		m_recursiveDraw = false;
 	}
 
+	scr_copyeverything = 1;
+
+	if (SCR_NEED_CONSOLE_BACKGROUND) {
+		Draw_ConsoleBackground(scr_con_current);
+	}
+	else {
+		// if you don't like fade in ingame menu, uncomment this
+		// if (m_state != m_ingame && m_state != m_democtrl)
+		Draw_FadeScreen(scr_menualpha.value);
+	}
+
+	scr_fullupdate = 0;
+
 	if (scr_scaleMenu.value) {
 		if (vid.aspect > 1.0) {
-			menuheight = bound (240, (int)((vid.height / scr_scaleMenu.value) + 0.5f), 960);
-			menuwidth  = (int) ((menuheight * vid.aspect) + 0.5f);
-		} else {
-			menuwidth  = bound (320, (int)((vid.width / scr_scaleMenu.value) + 0.5f), 960);
-			menuheight = (int) ((menuwidth / vid.aspect) + 0.5f);
+			menuheight = bound(240, (int)((vid.height / scr_scaleMenu.value) + 0.5f), 960);
+			menuwidth = (int)((menuheight * vid.aspect) + 0.5f);
 		}
-		
-		glMatrixMode(GL_PROJECTION);
-		glLoadIdentity ();
-		glOrtho  (0, menuwidth, menuheight, 0, -99999, 99999);
-	} else {
+		else {
+			menuwidth = bound(320, (int)((vid.width / scr_scaleMenu.value) + 0.5f), 960);
+			menuheight = (int)((menuwidth / vid.aspect) + 0.5f);
+		}
+
+		R_OrthographicProjection(0, menuwidth, menuheight, 0, -99999, 99999);
+	}
+	else {
 		menuwidth = vid.width;
 		menuheight = vid.height;
 	}
 
-	if (scr_centerMenu.value)
+	if (scr_centerMenu.value) {
 		m_yofs = (menuheight - 200) / 2;
-	else
+	}
+	else {
 		m_yofs = 0;
+	}
 
 	switch (m_state) {
 		case m_none: break;
-		case m_main:			M_Main_Draw (); break;
-		case m_singleplayer:	M_SinglePlayer_Draw (); break;
+		case m_main:			M_Main_Draw(); break;
+		case m_singleplayer:	M_SinglePlayer_Draw(); break;
 #ifndef CLIENTONLY
-		case m_load:			M_Load_Draw (); break;
-		case m_save:			M_Save_Draw (); break;
+		case m_load:			M_Load_Draw(); break;
+		case m_save:			M_Save_Draw(); break;
 #else
-		// keeps gcc happy
+			// keeps gcc happy
 		case m_load:
 		case m_save:
 			break;
 #endif
-		case m_multiplayer:		Menu_MultiPlayer_Draw (); break;
+		case m_multiplayer:		Menu_MultiPlayer_Draw(); break;
 		case m_multiplayer_submenu: M_MultiPlayerSub_Draw(); break;
 		case m_options:			M_Options_Draw(); break;
 		case m_proxy:			Menu_Proxy_Draw(); break;
@@ -1365,13 +1385,11 @@ void M_Draw (void) {
 	}
 
 	if (scr_scaleMenu.value) {
-		glMatrixMode (GL_PROJECTION);
-		glLoadIdentity ();
-		glOrtho  (0, vid.width, vid.height, 0, -99999, 99999);
+		R_OrthographicProjection(0, vid.width, vid.height, 0, -99999, 99999);
 	}
 
 	if (m_entersound) {
-		S_LocalSound ("misc/menu2.wav");
+		S_LocalSound("misc/menu2.wav");
 		m_entersound = false;
 	}
 }
