@@ -39,7 +39,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "vx_stuff.h"
 #include "vx_tracker.h"
 #include "gl_model.h"
-#include "gl_local.h"
 #include "tr_types.h"
 #include "teamplay.h"
 #include "EX_FileList.h"
@@ -52,6 +51,10 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "hud.h"
 #include "hud_common.h"
 
+extern cvar_t r_farclip, gl_max_size, gl_miptexLevel;
+extern cvar_t r_bloom;
+extern cvar_t gl_flashblend, r_dynamic, gl_lightmode, gl_modulate;
+
 typedef enum {
 	OPTPG_MISC,
 	OPTPG_PLAYER,
@@ -169,6 +172,7 @@ const char* autorecord_enum[] = { "off", "don't save", "auto save" };
 const char* hud_enum[] = { "classic", "new", "combined" };
 const char* ignorespec_enum[] = { "off", "on (as player)", "on (always)" };
 const char* gender_enum[] = { "Male", "m", "Female", "f", "Neutral", "n", "Not specified", "" };
+const char* lighting_enum[] = { "Off", "0", "On (CPU)", "1", "On (GPU - GLSL only)", "2" };
 
 const char* scr_sshot_format_enum[] = {
 	"JPG", "jpg", "PNG", "png", "TGA", "tga" };
@@ -188,7 +192,7 @@ extern cvar_t mvd_autotrack, mvd_moreinfo, mvd_status, cl_weaponpreselect, cl_we
 extern cvar_t demo_format, sys_highpriority, cl_window_caption, vid_flashonactivity;
 void Sys_RegisterQWURLProtocol_f(void);
 #endif
-extern cvar_t scr_autoid, crosshairalpha, gl_smoothfont, amf_hidenails, amf_hiderockets, gl_anisotropy, gl_lumaTextures, gl_textureless, gl_colorlights, scr_conalpha, scr_conback, gl_clear, gl_powerupshells, gl_powerupshells_size,
+extern cvar_t scr_autoid, crosshairalpha, amf_hidenails, amf_hiderockets, gl_anisotropy, gl_lumatextures, gl_textureless, gl_colorlights, scr_conalpha, scr_conback, gl_clear, gl_powerupshells,
 	scr_teaminfo
 ;
 
@@ -357,13 +361,13 @@ const char* rocketmodel_enum[] =
 { "rocket", "grenade" };
 
 const char* rockettrail_enum[] =
-{ "off", "normal", "grenade", "alt normal", "slight blood", "big blood", "tracer 1", "tracer 2", "plasma", "lavaball", "fuel rod", "plasma 2", "alt normal 2", "motion trail" };
+{ "off", "normal", "grenade", "alt normal", "slight blood", "big blood", "tracer 1", "tracer 2", "plasma", "lavaball", "fuel rod", "plasma 2", "alt normal 2" };
 
 const char* powerupglow_enum[] =
 { "off", "on", "own off" };
 
 const char* grenadetrail_enum[] =
-{ "off", "normal", "grenade", "alt normal", "slight blood", "big blood", "tracer 1", "tracer 2", "plasma", "lavaball", "fuel rod", "plasma 2", "alt normal 2", "motion trail" };
+{ "off", "normal", "grenade", "alt normal", "slight blood", "big blood", "tracer 1", "tracer 2", "plasma", "lavaball", "fuel rod", "plasma 2", "alt normal 2" };
 
 extern cvar_t cl_maxfps;
 const char* FpslimitRead(void) {
@@ -849,7 +853,7 @@ setting settplayer_arr[] = {
 	ADDSET_COLOR	("Shirt Color", topcolor),
 	ADDSET_COLOR	("Pants Color", bottomcolor),
 	ADDSET_ADVANCED_SECTION(),
-	ADDSET_BOOL		("Fullbright Skins", r_fullbrightSkins),
+	ADDSET_NUMBER	("Fullbright Skins", r_fullbrightSkins, 0, 1, 0.05),
 	ADDSET_ENUM    	("Ruleset", ruleset, ruleset_enum),
 	ADDSET_BASIC_SECTION(),
 	
@@ -913,7 +917,7 @@ setting settfps_arr[] = {
 
 	ADDSET_ADVANCED_SECTION(),	
 	ADDSET_SEPARATOR("Textures"),
-	ADDSET_BOOL		("Luma", gl_lumaTextures),
+	ADDSET_BOOL		("Luma", gl_lumatextures),
 	ADDSET_ENUM 	("Detail", gl_max_size, gl_max_size_enum),
 	ADDSET_NUMBER	("Miptex", gl_miptexLevel, 0, 3, 1),
 	ADDSET_BOOL		("No Textures", gl_textureless),
@@ -922,7 +926,6 @@ setting settfps_arr[] = {
 	ADDSET_SEPARATOR("Player & Weapon Model"),
 	ADDSET_ADVANCED_SECTION(),
 	ADDSET_BOOL		("Powerup Luma", gl_powerupshells),
-	ADDSET_NUMBER	("Powerup Luma Size", gl_powerupshells_size, 0, 10, 1),
 	ADDSET_NUMBER	("Weapon Opacity", cl_drawgun, 0, 1, 0.05),
 	ADDSET_BASIC_SECTION(),
 	ADDSET_NUMBER	("Weapon Size", r_viewmodelsize, 0.1, 1, 0.05),
@@ -954,7 +957,7 @@ setting settfps_arr[] = {
 	ADDSET_BASIC_SECTION(),
 	ADDSET_NAMED	("Rocket Trail", r_rockettrail, rockettrail_enum),
 	ADDSET_ADVANCED_SECTION(),
-	ADDSET_BOOL		("Rocket Light", r_rocketlight),
+	ADDSET_NUMBER	("Rocket Light", r_rocketlight, 0, 1, 0.05),
 	ADDSET_NAMED	("Grenade Trail", r_grenadetrail, grenadetrail_enum),
 	ADDSET_BASIC_SECTION(),
 	ADDSET_NUMBER	("Fakeshaft", cl_fakeshaft, 0, 1, 0.05),
@@ -964,7 +967,9 @@ setting settfps_arr[] = {
 	ADDSET_BASIC_SECTION(),
 
 	ADDSET_SEPARATOR("Lighting"),
+#ifdef RENDERER_OPTION_CLASSIC_OPENGL
 	ADDSET_BOOL		("GL Bloom", r_bloom),
+#endif
 	ADDSET_NAMED	("Powerup Glow", r_powerupglow, powerupglow_enum),
 	ADDSET_NUMBER	("Damage Flash", v_damagecshift, 0, 1, 0.1),
 	ADDSET_ADVANCED_SECTION(),
@@ -972,7 +977,7 @@ setting settfps_arr[] = {
 	ADDSET_BASIC_SECTION(),
 	ADDSET_BOOL		("Colored Lights", gl_colorlights),
 	ADDSET_BOOL		("Fast Lights", gl_flashblend),
-	ADDSET_BOOL		("Dynamic Lights", r_dynamic),
+	ADDSET_ENUM     ("Dynamic Lights", r_dynamic, lighting_enum),
 	ADDSET_ADVANCED_SECTION(),
 	ADDSET_BOOL		("Darken Map", gl_lightmode),
 	ADDSET_BOOL		("Particle Shaft", amf_lightning),
@@ -1112,6 +1117,8 @@ setting settbinds_arr[] = {
 	ADDSET_BIND("Move Backward", "+back"),
 	ADDSET_BIND("Strafe Left", "+moveleft"),
 	ADDSET_BIND("Strafe Right", "+moveright"),
+	ADDSET_BIND("Turn Left", "+left"),
+	ADDSET_BIND("Turn Right", "+right"),
 	ADDSET_ADVANCED_SECTION(),
 	ADDSET_BIND("Swim Up", "+moveup"),
 	ADDSET_BIND("Swim Down", "+movedown"),
@@ -1219,9 +1226,10 @@ setting settsystem_arr[] = {
 
 	//Video
 	ADDSET_SEPARATOR("Video"),
-	ADDSET_NUMBER	("Gamma", v_gamma, 0.1, 2.0, 0.1),
+	ADDSET_NUMBER	("Gamma", v_gamma, 0.3, 3.0, 0.1),
 	ADDSET_NUMBER	("Contrast", v_contrast, 1, 5, 0.1),
 	ADDSET_ADVANCED_SECTION(),
+	ADDSET_NUMBER	("Lightmap Intensity", gl_modulate, 0.5, 3.0, 0.1),
 	ADDSET_BOOL		("Clear Video Buffer", gl_clear),
 	ADDSET_NUMBER	("Anisotropy Filter", gl_anisotropy, 0, 16, 1),
 	ADDSET_ENUM		("Quality Mode", gl_texturemode, gl_texturemode_enum),
@@ -1243,7 +1251,6 @@ setting settsystem_arr[] = {
 	ADDSET_SEPARATOR("Font"),
 	ADDSET_NUMBER("Width", r_conwidth, 320, 2048, 8),
 	ADDSET_NUMBER("Height", r_conheight, 240, 1538, 4),
-	ADDSET_BOOL("Font Smoothing", gl_smoothfont),
 	ADDSET_BASIC_SECTION(),
 
 	//Sound & Volume
@@ -1370,6 +1377,19 @@ void Menu_Options_Init(void) {
 	CTab_SetCurrentId(&options_tab, OPTPG_PLAYER);
 }
 
+void Menu_Options_Shutdown(void)
+{
+	FL_Shutdown(&configs_filelist);
+	Settings_Shutdown(&settmisc);
+	Settings_Shutdown(&settfps);
+	Settings_Shutdown(&settview);
+	Settings_Shutdown(&settplayer);
+	Settings_Shutdown(&settbinds);
+	Settings_Shutdown(&settsystem);
+	Settings_Shutdown(&settconfig);
+	Settings_MainShutdown();
+}
+
 qbool Menu_Options_IsBindingKey (void)
 {
 	// Options/Binds, and waiting for a keypress
