@@ -1,4 +1,4 @@
-docker run \
+docker run --rm \
 	-v $SOURCEDIR:/source \
 	-v $OUTPUTDIR:/nightly \
 	-e "TARGET_PLATFORM=LINUX" \
