@@ -1,4 +1,4 @@
-docker run \
+docker run --rm \
 	-v $SOURCEDIR:/source \
 	-v $OUTPUTDIR:/nightly \
 	localghost/ezquake_nightly
