@@ -34,7 +34,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 static void OnChange_movie_dir(cvar_t *var, char *string, qbool *cancel);
 static void WAVCaptureStop (void);
 static void WAVCaptureStart (void);
-int SCR_Screenshot(char * name, qbool movie_capture);
 void SCR_Movieshot (char *);	//joe: capturing to avi
 
 //joe: capturing audio
@@ -375,7 +374,7 @@ void Movie_FinishFrame(void)
 	}
 	#else
 	snprintf(fname, sizeof(fname), "%s/capture_%02d-%02d-%04d_%02d-%02d-%02d/shot-%06d.%s",
-		movie_dir.string, movie_start_date.tm_mday, movie_start_date.tm_mon, movie_start_date.tm_year,
+		movie_dir.string, movie_start_date.tm_mday, 1 + movie_start_date.tm_mon, 1900 + movie_start_date.tm_year,
 		movie_start_date.tm_hour, movie_start_date.tm_min, movie_start_date.tm_sec, movie_frame_count, image_ext);
 
 	con_suppress = true;
@@ -432,7 +431,7 @@ static void WAVCaptureStart (void)
 		movie_start_date.wHour, movie_start_date.wMinute, movie_start_date.wSecond);
 #else
 	snprintf (fname, sizeof (fname), "%s/capture_%02d-%02d-%04d_%02d-%02d-%02d/audio.wav",
-		movie_dir.string, movie_start_date.tm_mday, movie_start_date.tm_mon, movie_start_date.tm_year,
+		movie_dir.string, movie_start_date.tm_mday, 1 + movie_start_date.tm_mon, 1900 + movie_start_date.tm_year,
 		movie_start_date.tm_hour, movie_start_date.tm_min, movie_start_date.tm_sec);
 #endif
 	if (!(wav_output = FS_OpenVFS (fname, "wb", FS_NONE_OS))) {
@@ -573,8 +572,8 @@ static background_thread_t threads[MAX_SCREENSHOT_THREADS] = { { 0 } };
 static byte* tempBuffer = 0;
 static int background_threads = 0;
 static int next_thread = 0;
-static int movie_width = 0;
-static int movie_height = 0;
+static size_t movie_width = 0;
+static size_t movie_height = 0;
 
 static int Movie_BackgroundThread(void* thread_data)
 {
@@ -646,7 +645,7 @@ void Movie_BackgroundShutdown(void)
 
 }
 
-byte* Movie_TempBuffer(int width, int height)
+byte* Movie_TempBuffer(size_t width, size_t height)
 {
 	if (width != movie_width || height != movie_height) {
 		return NULL;
