@@ -20,7 +20,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include "quakedef.h"
 #include "gl_model.h"
-#include "gl_local.h"
 #include "rulesets.h"
 #include "input.h"
 
@@ -28,18 +27,31 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 static void Rulesets_OnChange_ruleset (cvar_t *var, char *value, qbool *cancel);
 
+typedef struct rulesetDef_s {
+	ruleset_t ruleset;
+	float maxfps;
+	qbool restrictTriggers;
+	qbool restrictPacket;
+	qbool restrictParticles;
+	qbool restrictSound;
+	qbool restrictLogging;
+	qbool restrictRollAngle;
+} rulesetDef_t;
+
 static rulesetDef_t rulesetDef = {
-	rs_default,
-	72.0,
-	false,
-	false,
-	false,
-	false
+	rs_default,    // ruleset
+	72.0,          // maxfps
+	false,         // restrict triggers
+	false,         // restrict /packet command
+	false,         // restrict particles
+	false,         // restrict sound
+	false,         // restrict logging
+	false          // restrict rollangle
 };
 
 cvar_t ruleset = {"ruleset", "default", 0, Rulesets_OnChange_ruleset};
 
-qbool RuleSets_DisallowExternalTexture(model_t *mod)
+qbool RuleSets_DisallowExternalTexture(struct model_s *mod)
 {
 	switch (mod->modhint) {
 		case MOD_EYES:
@@ -81,8 +93,11 @@ qbool RuleSets_DisallowModelOutline(struct model_s *mod)
 			return true;
 		case MOD_THUNDERBOLT:
 			return true;
+		case MOD_BACKPACK:
+			return !cls.demoplayback && (rulesetDef.ruleset == rs_qcon || rulesetDef.ruleset == rs_smackdown);
 		default:
-			return rulesetDef.ruleset == rs_qcon || rulesetDef.ruleset == rs_smackdown;
+			// return to just rs_qcon once backface outlining tested
+			return !cls.demoplayback && (rulesetDef.ruleset == rs_qcon || rulesetDef.ruleset == rs_smackdown);
 	}
 }
 
@@ -152,7 +167,7 @@ qbool Rulesets_RestrictSound(const char* name)
 
 qbool Rulesets_RestrictPacket(void)
 {
-	return !cl.spectator && !cls.demoplayback && !cl.standby && rulesetDef.restrictPacket;
+	return cls.state == ca_active && !cl.spectator && !cls.demoplayback && !cl.standby && rulesetDef.restrictPacket;
 }
 
 qbool Rulesets_RestrictParticles(void)
@@ -223,8 +238,9 @@ static void Rulesets_Smackdown(qbool enable)
 		rulesetDef.restrictTriggers = true;
 		rulesetDef.restrictPacket = true; // packet command could have been exploited for external timers
 		rulesetDef.restrictParticles = true;
-		rulesetDef.ruleset = rs_smackdown;
+		rulesetDef.restrictLogging = true;
 		rulesetDef.restrictRollAngle = true;
+		rulesetDef.ruleset = rs_smackdown;
 	} else {
 		for (i = 0; i < (sizeof(disabled_cvars) / sizeof(disabled_cvars[0])); i++)
 			Cvar_SetFlags(disabled_cvars[i].var, Cvar_GetFlags(disabled_cvars[i].var) & ~CVAR_ROM);
@@ -236,8 +252,9 @@ static void Rulesets_Smackdown(qbool enable)
 		rulesetDef.restrictTriggers = false;
 		rulesetDef.restrictPacket = false;
 		rulesetDef.restrictParticles = false;
-		rulesetDef.ruleset = rs_default;
+		rulesetDef.restrictLogging = false;
 		rulesetDef.restrictRollAngle = false;
+		rulesetDef.ruleset = rs_default;
 	}
 }
 
@@ -276,8 +293,9 @@ static void Rulesets_Qcon(qbool enable)
 		rulesetDef.restrictPacket = true; // packet command could have been exploited for external timers
 		rulesetDef.restrictParticles = true;
 		rulesetDef.restrictSound = true;
-		rulesetDef.ruleset = rs_qcon;
+		rulesetDef.restrictLogging = true;
 		rulesetDef.restrictRollAngle = true;
+		rulesetDef.ruleset = rs_qcon;
 	} else {
 		for (i = 0; i < (sizeof(disabled_cvars) / sizeof(disabled_cvars[0])); i++)
 			Cvar_SetFlags(disabled_cvars[i].var, Cvar_GetFlags(disabled_cvars[i].var) & ~CVAR_ROM);
@@ -290,8 +308,9 @@ static void Rulesets_Qcon(qbool enable)
 		rulesetDef.restrictPacket = false;
 		rulesetDef.restrictParticles = false;
 		rulesetDef.restrictSound = false;
-		rulesetDef.ruleset = rs_default;
+		rulesetDef.restrictLogging = false;
 		rulesetDef.restrictRollAngle = false;
+		rulesetDef.ruleset = rs_default;
 	}
 }
 static void Rulesets_Thunderdome(qbool enable)
@@ -326,8 +345,9 @@ static void Rulesets_Thunderdome(qbool enable)
 		rulesetDef.restrictTriggers = true;
 		rulesetDef.restrictPacket = true; // packet command could have been exploited for external timers
 		rulesetDef.restrictParticles = false;
-		rulesetDef.ruleset = rs_thunderdome;
+		rulesetDef.restrictLogging = true;
 		rulesetDef.restrictRollAngle = true;
+		rulesetDef.ruleset = rs_thunderdome;
 	} else {
 		for (i = 0; i < (sizeof(disabled_cvars) / sizeof(disabled_cvars[0])); i++)
 			Cvar_SetFlags(disabled_cvars[i].var, Cvar_GetFlags(disabled_cvars[i].var) & ~CVAR_ROM);
@@ -339,8 +359,9 @@ static void Rulesets_Thunderdome(qbool enable)
 		rulesetDef.restrictTriggers = false;
 		rulesetDef.restrictPacket = false;
 		rulesetDef.restrictParticles = false;
-		rulesetDef.ruleset = rs_default;
+		rulesetDef.restrictLogging = false;
 		rulesetDef.restrictRollAngle = false;
+		rulesetDef.ruleset = rs_default;
 	}
 }
 static void Rulesets_MTFL(qbool enable)
@@ -353,6 +374,7 @@ static void Rulesets_MTFL(qbool enable)
 	extern cvar_t cl_c2spps, r_fullbrightSkins;
 	extern cvar_t amf_detpacklights;
 	extern cvar_t gl_picmip, gl_max_size, r_drawflat;
+	extern cvar_t vid_hwgammacontrol;
 	extern cvar_t gl_textureless;
 
 	int i = 0;
@@ -362,6 +384,7 @@ static void Rulesets_MTFL(qbool enable)
 		{&amf_detpacklights, "0"},
 		{&gl_textureless, "0"},
 		{&r_fullbrightSkins, "0"},
+		{&vid_hwgammacontrol, "2"},
 		{&cl_c2spps, "0"},
 	};
 
@@ -392,6 +415,7 @@ static void Rulesets_MTFL(qbool enable)
 
 		rulesetDef.restrictRollAngle = false;
 		rulesetDef.ruleset = rs_mtfl;
+		v_gamma.modified = true;
 	} else {
 		for (i = 0; i < (sizeof(disabled_cvars) / sizeof(disabled_cvars[0])); i++)
 			Cvar_SetFlags(disabled_cvars[i].var, Cvar_GetFlags(disabled_cvars[i].var) & ~CVAR_ROM);
@@ -402,8 +426,9 @@ static void Rulesets_MTFL(qbool enable)
 		for (i = 0; i < (sizeof(limited_min_cvars) / sizeof(limited_min_cvars[0])); i++)
 			Cvar_SetFlags(limited_min_cvars[i].var, Cvar_GetFlags(limited_min_cvars[i].var) & ~CVAR_RULESET_MIN);
 
-		rulesetDef.restrictRollAngle = false;
 		rulesetDef.ruleset = rs_default;
+		rulesetDef.restrictRollAngle = false;
+		v_gamma.modified = true;
 	}
 }
 
@@ -419,7 +444,7 @@ void Rulesets_Init(void)
 	Cvar_SetCurrentGroup(CVAR_GROUP_USERINFO);
 	Cvar_Register(&ruleset);
 
-	if ((temp = COM_CheckParm("-ruleset")) && temp + 1 < COM_Argc()) {
+	if ((temp = COM_CheckParm(cmdline_param_client_ruleset)) && temp + 1 < COM_Argc()) {
 		if (!strcasecmp(COM_Argv(temp + 1), "smackdown")) {
 			Cvar_Set(&ruleset, "smackdown");
 			return;
@@ -518,15 +543,27 @@ void Rulesets_OnChange_cl_delay_packet(cvar_t *var, char *value, qbool *cancel)
 		return;
 	}
 
+	if (var == &cl_delay_packet_target && (ival < 0 || ival > CL_MAX_PACKET_DELAY_TARGET)) {
+		Com_Printf("%s must be between 0 and %d\n", var->name, CL_MAX_PACKET_DELAY_TARGET);
+		*cancel = true;
+		return;
+	}
+
 	if (cls.state == ca_active) {
 		if ((cl.standby) || (cl.teamfortress)) {
 			char announce[128];
+			int delay_target_ms = (var == &cl_delay_packet_target ? ival : cl_delay_packet_target.integer);
+			int delay_deviation = (var == &cl_delay_packet_dev ? ival : cl_delay_packet_dev.integer);
+			int delay_constant = (var == &cl_delay_packet ? ival : cl_delay_packet.integer);
 
-			if (var == &cl_delay_packet) {
-				snprintf(announce, sizeof(announce), "say delay packet: %d ms (%d dev)\n", ival, cl_delay_packet_dev.integer);
+			if (delay_target_ms) {
+				snprintf(announce, sizeof(announce), "say delay packet: target ping %d ms (%dms dev)\n", delay_target_ms, delay_deviation);
+			}
+			else if (delay_constant) {
+				snprintf(announce, sizeof(announce), "say delay packet: adding %d ms (%dms dev)\n", delay_constant, delay_deviation);
 			}
 			else {
-				snprintf(announce, sizeof(announce), "say delay packet: %d ms (%d dev)\n", cl_delay_packet.integer, ival);
+				snprintf(announce, sizeof(announce), "say delay packet: off\n");
 			}
 
 			// allow in standby or teamfortress. For teamfortress, more often than not
@@ -695,6 +732,67 @@ qbool Ruleset_AllowPolygonOffset(entity_t* ent)
 	}
 }
 
+// Not technically ruleset-based but limits functionaly for similar reasons...
+qbool Ruleset_IsLumaAllowed(struct model_s *mod)
+{
+	switch (mod->modhint)
+	{
+	case MOD_EYES:
+	case MOD_BACKPACK:
+	case MOD_PLAYER:
+
+	case MOD_SENTRYGUN: // tf
+	case MOD_DETPACK:   // tf
+
+		return false; // no luma for such models
+
+	default:
+
+		return true; // luma allowed
+	}
+}
+
+qbool Rulesets_ToggleWhenFlashed(void)
+{
+	return rulesetDef.ruleset == rs_mtfl;
+}
+
+qbool Rulesets_FullbrightModel(struct model_s* model)
+{
+	extern cvar_t gl_fb_models;
+	qbool protected_model = (model->modhint == MOD_EYES || model->modhint == MOD_BACKPACK) && rulesetDef.ruleset != rs_default;
+	qbool fb_requested = gl_fb_models.integer == 1 && model->modhint != MOD_GIB && model->modhint != MOD_VMODEL && !Ruleset_IsLocalSinglePlayerGame();
+
+	return !protected_model && fb_requested;
+}
+
+const char* Ruleset_BlockPlayerCountMacros(void)
+{
+	if (rulesetDef.ruleset == rs_mtfl) {
+		return BANNED_BY_MTFL;
+	}
+	return NULL;
+}
+
+qbool Ruleset_AllowPowerupShell(model_t* model)
+{
+	// always allow powerupshells for specs or demos.
+	// do not allow powerupshells for eyes in other cases
+	extern cvar_t gl_powerupshells;
+
+	return (bound(0, gl_powerupshells.value, 1) && ((cls.demoplayback || cl.spectator) || model->modhint != MOD_EYES));
+}
+
+qbool Ruleset_CanLogConsole(void)
+{
+	return cls.demoplayback || cls.state != ca_active || cl.standby || cl.countdown || !rulesetDef.restrictLogging;
+}
+
+qbool Ruleset_AllowNoHardwareGamma(void)
+{
+	return rulesetDef.ruleset != rs_mtfl;
+}
+
 float Ruleset_RollAngle(void)
 {
 	extern cvar_t cl_rollangle;
@@ -705,3 +803,11 @@ float Ruleset_RollAngle(void)
 
 	return bound(0.0f, cl_rollangle.value, 5.0f);
 }
+
+#ifndef CLIENTONLY
+extern cvar_t     maxclients;
+qbool Ruleset_IsLocalSinglePlayerGame(void)
+{
+	return com_serveractive && cls.state == ca_active && !cl.deathmatch && maxclients.integer == 1;
+}
+#endif
