@@ -91,11 +91,11 @@ cvar_t s_ambientlevel = {"s_ambientlevel", "0.3"};
 cvar_t s_ambientfade = {"s_ambientfade", "100"};
 cvar_t s_show = {"s_show", "0"};
 cvar_t s_swapstereo = {"s_swapstereo", "0"};
-cvar_t s_linearresample = {"s_linearresample", "0", CVAR_LATCH};
+cvar_t s_linearresample = {"s_linearresample", "0", CVAR_LATCH_SOUND };
 cvar_t s_linearresample_stream = {"s_linearresample_stream", "0"};
 cvar_t s_khz = {"s_khz", "11", CVAR_NONE, OnChange_s_khz}; // If > 11, default sounds are noticeably different.
 cvar_t s_desiredsamples = {"s_desiredsamples", "0", CVAR_AUTO, OnChange_s_desiredsamples };
-cvar_t s_audiodevice = {"s_audiodevice", "0", CVAR_LATCH};
+cvar_t s_audiodevice = {"s_audiodevice", "0", CVAR_LATCH_SOUND };
 cvar_t s_silent_racing = { "s_silent_racing", "0" };
 
 SDL_mutex *smutex;
@@ -198,7 +198,7 @@ static void S_SDL_Shutdown(void)
 	}
 
 	if (shw != NULL) {
-		free(shw);
+		Q_free(shw);
 		shw = NULL;
 	}
 }
@@ -443,12 +443,6 @@ static void S_Register_RegularCvarsAndCommands(void)
 	Cvar_ResetCurrentGroup();
 
 	// compatibility with old configs
-	Cmd_AddLegacyCommand("nosound", "s_nosound");
-	Cmd_AddLegacyCommand("precache", "s_precache");
-	Cmd_AddLegacyCommand("loadas8bit", "s_loadas8bit");
-	Cmd_AddLegacyCommand("ambient_level", "s_ambientlevel");
-	Cmd_AddLegacyCommand("ambient_fade", "s_ambientfade");
-	Cmd_AddLegacyCommand("snd_show", "s_show");
 	Cmd_AddLegacyCommand("snd_restart", "s_restart"); // and snd_restart a legacy command
 
 	Cmd_AddCommand("s_restart", S_Restart_f); // dimman: made s_restart the actual command
@@ -485,7 +479,7 @@ void S_Init (void)
 		Com_Printf_State (PRINT_INFO, "[sound] Sound is already initialized!\n");
 		return;
 	}
-	if (COM_CheckParm("-nosound")) {
+	if (COM_CheckParm(cmdline_param_client_nosound)) {
 		Cmd_AddLegacyCommand ("play", ""); // just suppress warnings
 		return;
 	}	
