@@ -20,6 +20,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 // sv_phys.c
 
+#ifndef CLIENTONLY
 #include "qwsvdef.h"
 
 /*
@@ -62,6 +63,7 @@ cvar_t	pm_slidefix		= { "pm_slidefix", "", CVAR_SERVERINFO};
 void OnChange_pm_airstep (cvar_t *var, char *value, qbool *cancel);
 cvar_t	pm_airstep		= { "pm_airstep", "", CVAR_SERVERINFO, OnChange_pm_airstep};
 cvar_t	pm_pground		= { "pm_pground", "", CVAR_SERVERINFO|CVAR_ROM};
+cvar_t  pm_rampjump     = { "pm_rampjump", "", CVAR_SERVERINFO };
 
 double	sv_frametime;
 
@@ -1129,3 +1131,5 @@ void SV_SetMoveVars(void)
 	movevars.waterfriction      = sv_waterfriction.value;
 	movevars.entgravity         = 1.0;
 }
+
+#endif // !CLIENTONLY
