@@ -31,6 +31,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "keys.h"
 #include "server.h"
 #include "pcre.h"
+#include <shlobj.h>
 
 // "Starting with the Release 302 drivers, application developers can direct the
 // Optimus driver at runtime to use the High Performance Graphics to render any
@@ -60,7 +61,7 @@ __attribute__((dllexport)) DWORD AmdPowerXpressRequestHighPerformance = 0x000000
 #endif
 
 #define MINIMUM_WIN_MEMORY	0x0c00000
-#define MAXIMUM_WIN_MEMORY	0x8000000
+#define MAXIMUM_WIN_MEMORY	0xfffffff
 
 #define PAUSE_SLEEP		50				// sleep time on pause or minimization
 #define NOT_FOCUS_SLEEP	20				// sleep time when not focus
@@ -118,7 +119,9 @@ static qbool RegisterKeyHook (void)
 
 void OnChange_sys_disableWinKeys(cvar_t *var, char *string, qbool *cancel) 
 {
-	if (Q_atof(string)) 
+	extern cvar_t r_fullscreen;
+
+	if (Q_atof(string) == 1 || (Q_atof(string) == 2 && r_fullscreen.value))
 	{
 		if (!WinKeyHook_isActive) 
 		{
@@ -168,7 +171,7 @@ LRESULT CALLBACK LLWinKeyHook(int Code, WPARAM wParam, LPARAM lParam)
 	return CallNextHookEx(NULL, Code, wParam, lParam);
 }
 
-void Sys_ActiveAppChanged (void)
+void Sys_ActiveAppChanged(void)
 {
 	static qbool appWasActive = true;
 	static qbool hookWasActive = false;
@@ -178,12 +181,12 @@ void Sys_ActiveAppChanged (void)
 
 	appWasActive = ActiveApp;
 	if (ActiveApp && hookWasActive) {
-		RegisterKeyHook ();
+		RegisterKeyHook();
 	}
 	else if (!ActiveApp) {
 		hookWasActive = WinKeyHook_isActive;
 
-		ReleaseKeyHook ();
+		ReleaseKeyHook();
 	}
 }
 
@@ -348,20 +351,21 @@ dir_t Sys_listdir (const char *path, const char *ext, int sort_type)
 
 	dir.files = list;
 	all = !strncmp(ext, ".*", 3);
-	if (!all)
-		if (!(preg = pcre_compile(ext, PCRE_CASELESS, &errbuf, &r, NULL)))
-		{
+	if (!all) {
+		if (!(preg = pcre_compile(ext, PCRE_CASELESS, &errbuf, &r, NULL))) {
 			Con_Printf("Sys_listdir: pcre_compile(%s) error: %s at offset %d\n",
-			           ext, errbuf, r);
-			Q_free(preg);
+				ext, errbuf, r);
+			pcre_free(preg);
 			return dir;
 		}
+	}
 
 	snprintf(pathname, sizeof(pathname), "%s/*.*", path);
 	if ((h = FindFirstFile (pathname , &fd)) == INVALID_HANDLE_VALUE)
 	{
-		if (!all)
-			Q_free(preg);
+		if (!all) {
+			pcre_free(preg);
+		}
 		return dir;
 	}
 
@@ -379,20 +383,20 @@ dir_t Sys_listdir (const char *path, const char *ext, int sort_type)
 			default:
 				Con_Printf("Sys_listdir: pcre_exec(%s, %s) error code: %d\n",
 				           ext, fd.cFileName, r);
-				if (!all)
-					Q_free(preg);
+				if (!all) {
+					pcre_free(preg);
+				}
 				return dir;
 			}
 		}
 
-		if (fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) //bliP: list dir
-		{
+		if (fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
+			//bliP: list dir
 			dir.numdirs++;
 			list[dir.numfiles].isdir = true;
 			list[dir.numfiles].size = list[dir.numfiles].time = 0;
 		}
-		else
-		{
+		else {
 			list[dir.numfiles].isdir = false;
 			snprintf(pathname, sizeof(pathname), "%s/%s", path, fd.cFileName);
 			list[dir.numfiles].time = 0; //Sys_FileTime(pathname);
@@ -400,15 +404,15 @@ dir_t Sys_listdir (const char *path, const char *ext, int sort_type)
 		}
 		strlcpy (list[dir.numfiles].name, fd.cFileName, sizeof(list[0].name));
 
-		if (++dir.numfiles == MAX_DIRFILES - 1)
+		if (++dir.numfiles == MAX_DIRFILES - 1) {
 			break;
-
-	}
-	while (FindNextFile(h, &fd));
+		}
+	} while (FindNextFile(h, &fd));
 
 	FindClose (h);
-	if (!all)
-		Q_free(preg);
+	if (!all) {
+		pcre_free(preg);
+	}
 
 	switch (sort_type)
 	{
@@ -428,15 +432,6 @@ dir_t Sys_listdir (const char *path, const char *ext, int sort_type)
 // SYSTEM IO
 // ===============================================================================
 
-void Sys_MakeCodeWriteable (unsigned long startaddr, unsigned long length) 
-{
-	DWORD  flOldProtect;
-
-	//@@@ copy on write or just read-write?
-	if (!VirtualProtect((LPVOID)startaddr, length, PAGE_EXECUTE_READWRITE, &flOldProtect))
-   		Sys_Error("Protection change failed");
-}
-
 /// turn back on screen saver and monitor power off
 static void Sys_RestoreScreenSaving(void)
 {
@@ -476,13 +471,12 @@ void Sys_Error (char *error, ...)
 	va_list argptr;
 	char text[1024];
 
+	va_start(argptr, error);
+	vsnprintf(text, sizeof(text), error, argptr);
+	va_end(argptr);
+
 	Host_Shutdown ();
 
-	va_start (argptr, error);
-
-	vsnprintf (text, sizeof(text), error, argptr);
-	va_end (argptr);
-
 	MessageBox(NULL, text, "Error", 0);
 
 	if (qwclsemaphore)
@@ -493,21 +487,56 @@ void Sys_Error (char *error, ...)
 	exit (1);
 }
 
+void Sys_Printf_Direct(const char* text)
+{
+#ifdef DEBUG_MEMORY_ALLOCATIONS
+	if (houtput == NULL) {
+		houtput = CreateFile("SysPrintf.log", GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
+	}
+
+	if (houtput != NULL) {
+		DWORD dummy;
+
+		WriteFile(houtput, text, strlen(text), &dummy, NULL);
+	}
+#endif
+}
+
 void Sys_Printf (char *fmt, ...) 
 {
-	va_list argptr;
-	char text[1024];
-	DWORD dummy;
-
-#ifdef NDEBUG
+#ifndef _DEBUG
 	return;
 #endif
 
-	va_start (argptr,fmt);
-	vsnprintf (text, sizeof(text), fmt, argptr);
-	va_end (argptr);
+#ifdef DEBUG_MEMORY_ALLOCATIONS
+	if (houtput == NULL) {
+		houtput = CreateFile("SysPrintf.log", GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
+	}
 
-	WriteFile (houtput, text, strlen(text), &dummy, NULL);
+	if (houtput != NULL) {
+		va_list argptr;
+		char text[1024];
+		DWORD dummy;
+		int n;
+
+		va_start(argptr, fmt);
+		n = vsnprintf(text, sizeof(text), fmt, argptr);
+		va_end(argptr);
+
+		if (n >= sizeof(text)) {
+			char* buffer = (char*)Q_malloc(n + 1);
+			va_start(argptr, fmt);
+			vsnprintf(buffer, n, fmt, argptr);
+			va_end(argptr);
+
+			WriteFile(houtput, (byte*)buffer, strlen(buffer), &dummy, NULL);
+			Q_free(buffer);
+		}
+		else {
+			WriteFile(houtput, text, strlen(text), &dummy, NULL);
+		}
+	}
+#endif
 }
 
 void Sys_Quit (void) 
@@ -523,6 +552,13 @@ void Sys_Quit (void)
 		UnhookWindowsHookEx(WinKeyHook);
 #endif
 
+	if (houtput) {
+		if (houtput != GetStdHandle(STD_OUTPUT_HANDLE)) {
+			CloseHandle(houtput);
+			houtput = NULL;
+		}
+	}
+
 	Sys_RestoreScreenSaving();
  
 	exit (0);
@@ -535,7 +571,7 @@ void Sys_InitDoubleTime (void)
 {
 	__int64 freq;
 
-	if (!COM_CheckParm("-nohwtimer") && QueryPerformanceFrequency((LARGE_INTEGER *)&freq) && freq > 0) 
+	if (!COM_CheckParm(cmdline_param_client_nohardwaretimers) && QueryPerformanceFrequency((LARGE_INTEGER *)&freq) && freq > 0)
 	{
 		// Hardware timer available
 		pfreq = (double)freq;
@@ -618,11 +654,15 @@ void WinCheckOSInfo(void)
 
 	vinfo.dwOSVersionInfoSize = sizeof(vinfo);
 
-	if (!GetVersionEx(&vinfo))
-		Sys_Error ("Couldn't get OS info");
+	if (!GetVersionEx(&vinfo)) {
+		Sys_Error("Couldn't get OS info");
+		return;
+	}
 
-	if (vinfo.dwPlatformId != VER_PLATFORM_WIN32_NT || vinfo.dwMajorVersion < 5 || (vinfo.dwMajorVersion == 5 && vinfo.dwMinorVersion < 1))
-		Sys_Error ("ezQuake requires at least Windows XP.");
+	if (vinfo.dwPlatformId != VER_PLATFORM_WIN32_NT || vinfo.dwMajorVersion < 5 || (vinfo.dwMajorVersion == 5 && vinfo.dwMinorVersion < 1)) {
+		Sys_Error("ezQuake requires at least Windows XP.");
+		return;
+	}
 
 	// Use raw resolutions, not scaled
 	{
@@ -645,7 +685,7 @@ void Sys_Init_ (void)
 	// Enabled for development purposes, but disabled for official builds.
 
 #ifdef _DEBUG
-	if (!COM_CheckParm("-allowmultiple"))
+	if (!COM_CheckParm(cmdline_param_client_allowmultipleclients))
 #endif
 	{
 		// Mutex will fail if semaphore already exists.
@@ -656,7 +696,7 @@ void Sys_Init_ (void)
 
 		if (!qwclsemaphore)
 		{
-			int qwurl_parm = COM_CheckParm("+qwurl");
+			int qwurl_parm = COM_FindParm("+qwurl");
 			char cmd[1024] = { 0 };
 
 			if (COM_CheckArgsForPlayableFiles(cmd, sizeof(cmd)))
@@ -829,7 +869,7 @@ void Sys_CheckQWProtocolHandler(void)
 
 	if (cl_verify_qwprotocol.integer >= 2) {
 		// Always register the qw:// protocol.
-		Cbuf_AddText("register_qwurl_protocol\n");
+		Cbuf_AddText("register_qwurl_protocol quiet\n");
 	} else if (cl_verify_qwprotocol.integer == 1 && !Sys_CheckIfQWProtocolHandler()) {
 		// Check if the running exe is the one associated with the qw:// protocol.
 		Com_PrintVerticalBar(INITIAL_CON_WIDTH);
@@ -859,6 +899,7 @@ void Sys_RegisterQWURLProtocol_f(void)
 	// admin this will fail. On Vista this requires UAC usage.
 	// Because of this, we always write specifically to "HKEY_CURRENT_USER\Software\Classes"
 	//
+	qbool quiet = Cmd_Argc() == 2 && !strcmp(Cmd_Argv(1), "quiet");
 
 	HKEY keyhandle;
 	char exe_path[MAX_PATH];
@@ -954,6 +995,10 @@ void Sys_RegisterQWURLProtocol_f(void)
 		}
 
 		RegCloseKey(keyhandle);
+
+		if (!quiet) {
+			Com_Printf_State(PRINT_WARNING, "qw:// protocol registered\n");
+		}
 	}
 }
 
@@ -1240,7 +1285,7 @@ int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLin
 	COM_InitArgv (argc, argv);
 
 	// Let me use -condebug C:\condebug.log before Quake FS init, so I get ALL messages before quake fully init
-	if ((i = COM_CheckParm("-condebug")) && i < COM_Argc() - 1) 
+	if ((i = COM_CheckParm(cmdline_param_console_debug)) && i < COM_Argc() - 1)
 	{
 		extern FILE *qconsole_log;
 		char *s = COM_Argv(i + 1);
@@ -1414,7 +1459,7 @@ void Sys_ReadIPC(void)
 unsigned int Sys_SendIPC(const char *buf)
 {
 	HANDLE hMailslot;
-	unsigned long num_bytes_written;
+	DWORD num_bytes_written;
 	qbool result = false;
 
 	// Connect to the server mailslot using CreateFile()
@@ -1597,3 +1642,33 @@ void Sys_CancelDeadKey (void)
 	input[1].ki.dwFlags = KEYEVENTF_KEYUP;
 	SendInput (2, input, sizeof (INPUT));
 }
+
+const char* Sys_FontsDirectory(void)
+{
+	static char path[MAX_OSPATH];
+
+	if (!SHGetSpecialFolderPath(NULL, path, CSIDL_FONTS, 0)) {
+		path[0] = 0;
+	}
+
+	return path;
+}
+
+const char* Sys_HomeDirectory(void)
+{
+	static char path[MAX_OSPATH];
+
+	// gets "C:\documents and settings\johnny\my documents" path
+	if (!SHGetSpecialFolderPath(0, path, CSIDL_PERSONAL, 0)) {
+		path[0] = 0;
+	}
+
+	// <Cokeman> yea, but it shouldn't be in My Documents
+	// <Cokeman> it should be in the application data dir
+	// c:\documents and settings\<user>\application data
+	//if (!SHGetSpecialFolderPath(0, path, CSIDL_APPDATA, 0)) {
+	//	path[0] = 0;
+	//}
+
+	return path;
+}
