@@ -882,7 +882,7 @@ static void InitInternalTriggers(void)
 	// lastip
 	AddInternalTrigger("([0-9]|[01]?\\d\\d|2[0-4]\\d|25[0-5])\\.([0-9]|[01]?\\d\\d|2[0-4]\\d|25[0-5])\\.([0-9]|[01]?\\d\\d|2[0-4]\\d|25[0-5])\\.([0-9]|[01]?\\d\\d|2[0-4]\\d|25[0-5])\\:(\\d{4,5})", 8, INTRIG_Lastip_port);
 	// lastip address, restricted to 64 bytes
-	AddInternalTrigger("\\b([A-Za-z0-9-.]{1,53}?\\.[A-Za-z]{2,4}\\:\\d{4,5})", 8, INTRIG_Lastip_port);
+	AddInternalTrigger("\\b([A-Za-z0-9-.]{1,53}?\\.[A-Za-z]{2,5}\\:\\d{4,5})", 8, INTRIG_Lastip_port);
 }
  
 void TP_InitTriggers (void)
@@ -907,3 +907,22 @@ void TP_InitTriggers (void)
 	Cvar_Register (&tp_forceTriggers);
 	Cvar_ResetCurrentGroup();
 }
+
+void TP_ShutdownTriggers(void)
+{
+	pcre_internal_trigger_t* trigger;
+	pcre_internal_trigger_t* next_trigger;
+
+	for (trigger = internal_triggers; trigger; trigger = next_trigger) {
+		next_trigger = trigger->next;
+		if (trigger->regexp) {
+			(pcre_free)(trigger->regexp);
+		}
+
+		if (trigger->regexp_extra) {
+			(pcre_free)(trigger->regexp_extra);
+		}
+
+		Q_free(trigger);
+	}
+}
