@@ -26,7 +26,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "utils.h"
 #include "qtv.h"
 #include "teamplay.h"
-
+#include "q_shared.h"
 
 int TP_CategorizeMessage (const char *s, int *offset);
 
@@ -139,13 +139,13 @@ int StringToRGB_W(char *s, byte *rgb)
 	int i;
 	char buf[20]; // "255 255 255 255" - the longest possible string
 	char *result;
+
 	rgb[0] = rgb[1] = rgb[2] = rgb[3] = 255;
 
 	strlcpy(buf, s, sizeof(buf));
 	result = strtok(buf, " ");
 
-	for (i = 0; i < 4 && result; i++, result = strtok(NULL, " "))
-	{
+	for (i = 0; i < 4 && result; i++, result = strtok(NULL, " ")) {
 		rgb[i] = (byte) Q_atoi(result);
 	}
 
@@ -162,6 +162,17 @@ int StringToRGB_W(char *s, byte *rgb)
 	return i;
 }
 
+void TrackerStringToRGB_W(const char *s, byte *rgb)
+{
+	rgb[0] = rgb[1] = rgb[2] = rgb[3] = 255;
+
+	if (s[0] >= '0' && s[0] <= '9' && s[1] >= '0' && s[1] <= '9' && s[2] >= '0' && s[2] <= '9') {
+		rgb[0] = ((s[0] - '0') / 9.0f) * 255;
+		rgb[1] = ((s[1] - '0') / 9.0f) * 255;
+		rgb[2] = ((s[2] - '0') / 9.0f) * 255;
+	}
+}
+
 byte* StringToRGB(char *s)
 {
 	static byte rgb[4];
@@ -181,8 +192,10 @@ int ParseFloats(char *s, float *f, int *f_size) {
 	int i, argc;
 	tokenizecontext_t ctx;
 
-	if (!s || !f || !f_size)
+	if (!s || !f || !f_size) {
 		Sys_Error("ParseFloats() wrong params");
+		return 0;
+	}
 
 	if (f_size[0] <= 0)
 		return (f_size[0] = 0); // array have no size, unusual but no crime
@@ -239,8 +252,10 @@ int strlen_color(const char *str)
 // skip ezquake color sequence
 void Util_SkipEZColors(char *dst, const char *src, size_t size)
 {
-	if ( !dst || !src )
+	if (!dst || !src) {
 		Sys_Error("Util_SkipColors: invalid input params");
+		return;
+	}
 
 	if ( !size )
 		return; // no space
@@ -347,6 +362,32 @@ int HexToInt(char c)
 		return -1;
 }
 
+/************************************** Game Mode Utils *********************************/
+
+char *get_ktx_mode (void)
+{
+	return Info_ValueForKey(cl.serverinfo, "mode");
+}
+
+qbool check_ktx_ca (void)	// playing clan arena
+{
+	char *ktxmode = get_ktx_mode();
+
+	return (strstr(ktxmode, "-ca") != NULL);
+}
+
+qbool check_ktx_wo (void)	// playing wipeout
+{
+	char *ktxmode = get_ktx_mode();
+
+	return (strstr(ktxmode, "-wo") != NULL);
+}
+
+qbool check_ktx_ca_wo (void) // playing clan arena or wipeout
+{
+	return (check_ktx_ca() || check_ktx_wo());
+}
+
 /************************************** File Utils **************************************/
 
 int Util_Extend_Filename(char *filename, char **ext) {
@@ -530,15 +571,39 @@ char *Player_StripNameColor(const char *name)
 	return stripped;
 }
 
-int Player_StringtoSlot(char *arg, qbool use_regular_expression)
+int Player_IdStringToSlot(const char* arg)
+{
+	int i;
+
+	// Check if the argument is a user id instead
+	// Make sure all chars in the given arg are digits in that case.
+	for (i = 0; arg[i]; i++)
+	{
+		if (!isdigit((byte)arg[i])) {
+			return PLAYER_NAME_NOMATCH;
+		}
+	}
+
+	// Get player ID.
+	return Player_IdtoSlot(Q_atoi(arg));
+}
+
+int Player_StringtoSlot(char *arg, qbool use_regular_expression, qbool prioritise_user_id)
 {
 	int i, slot, arg_length;
 
-	if (!arg[0])
-	{
+	if (!arg[0]) {
 		return PLAYER_NAME_NOMATCH;
 	}
 
+	if (prioritise_user_id) {
+		slot = Player_IdStringToSlot(arg);
+
+		if (slot >= 0) {
+			return slot;
+		}
+	}
+
 	// Match on partial names by only comparing the
 	// same amount of chars as in the given argument
 	// with all the player names. The first match will be picked.
@@ -592,19 +657,21 @@ int Player_StringtoSlot(char *arg, qbool use_regular_expression)
 		}
 	}
 
-	// Check if the argument is a user id instead
-	// Make sure all chars in the given arg are digits in that case.
-	for (i = 0; arg[i]; i++)
+	// Regexp match against stripped player name if previous attempts have failed
+	for (i = 0; i < MAX_CLIENTS; i++)
 	{
-		if (!isdigit(arg[i]))
+		char *stripped = Player_StripNameColor(cl.players[i].name);
+
+		if (cl.players[i].name[0] && Utils_RegExpMatch(arg, stripped))
 		{
-			return PLAYER_NAME_NOMATCH;
+			Q_free(stripped);
+			return i;
 		}
+
+		Q_free(stripped);
 	}
 
-	// Get player ID.
-	slot = Player_IdtoSlot(Q_atoi(arg));
-
+	slot = Player_IdStringToSlot(arg);
 	return (slot >= 0) ? slot : PLAYER_ID_NOMATCH;
 }
 
@@ -655,12 +722,12 @@ int Player_GetTrackId(int player_id)
 }
 
 
-int Player_GetSlot(char *arg) 
+int Player_GetSlot(char *arg, qbool prioritise_user_id)
 {
 	int response;
 
 	// Try getting the slot by name or id.
-	if ((response = Player_StringtoSlot(arg, false)) >= 0 )
+	if ((response = Player_StringtoSlot(arg, false, prioritise_user_id)) >= 0 )
 		//|| response == PLAYER_ID_NOMATCH)
 	{
 		return response;
@@ -792,6 +859,7 @@ void CopyToClipboard(const char *text)
 	}
 #else
 	strlcpy (clipboard, text, CLIPBOARDSIZE);
+	Sys_CopyToClipboard(text);
 #endif
 }
 
@@ -1203,33 +1271,25 @@ qbool Utils_RegExpMatch(char *regexp, char *matchstring)
 			NULL);				// use default character tables.
 
 	// Check for an error compiling the regexp.
-	if(error)
-	{
-		if(re)
-		{
-			Q_free(re);
+	if (error) {
+		if (re) {
+			pcre_free(re);
 		}
 
 		return false;
 	}
 
 	// Check if we have a match.
-	if(re && (match = pcre_exec(re, NULL, matchstring, strlen(matchstring), 0, 0, offsets, HUD_REGEXP_OFFSET_COUNT)) >= 0)
-	{
-		if(re)
-		{
-			free(re);
-		}
+	if (re && (match = pcre_exec(re, NULL, matchstring, strlen(matchstring), 0, 0, offsets, HUD_REGEXP_OFFSET_COUNT)) >= 0) {
+		pcre_free(re);
 
 		return true;
 	}
 
 	// Make sure we clean up.
-	if(re)
-	{
-		free(re);
+	if (re) {
+		pcre_free(re);
 	}
-
 	return false;
 }
 
@@ -1248,37 +1308,31 @@ qbool Utils_RegExpGetGroup(char *regexp, char *matchstring, const char **results
 			&erroffset,			// Error offset.
 			NULL);				// use default character tables.
 
-	if(error)
-	{
-		if(re)
-		{
-			Q_free(re);
+	if (error) {
+		if (re) {
+			pcre_free(re);
 		}
 
 		return false;
 	}
 
-	if(re && (match = pcre_exec(re, NULL, matchstring, strlen(matchstring), 0, 0, offsets, HUD_REGEXP_OFFSET_COUNT)) >= 0)
-	{
+	if (re && (match = pcre_exec(re, NULL, matchstring, strlen(matchstring), 0, 0, offsets, HUD_REGEXP_OFFSET_COUNT)) >= 0) {
 		int substring_length = 0;
 		substring_length = pcre_get_substring (matchstring, offsets, match, group, resultstring);
 
-		if (resultlength != NULL)
-		{
+		if (resultlength != NULL) {
 			(*resultlength) = substring_length;
 		}
 
-		if(re)
-		{
-			Q_free(re);
+		if (re) {
+			pcre_free(re);
 		}
 
 		return (substring_length != PCRE_ERROR_NOSUBSTRING && substring_length != PCRE_ERROR_NOMEMORY);
 	}
 
-	if(re)
-	{
-		Q_free(re);
+	if (re) {
+		pcre_free(re);
 	}
 
 	return false;
@@ -1293,22 +1347,29 @@ pcre_extra	*wildcard_re_extra[4];
 
 qbool IsRegexp(const char *str)
 {
-	if (*str == '+' || *str == '-') // +/- aliases; valid regexp can not start with +/-
+	if (*str == '+' || *str == '-') {
+		// +/- aliases; valid regexp can not start with +/-
 		return false;
+	}
 
 	return (strcspn(str, "\\\"()[]{}.*+?^$|")) != strlen(str) ? true : false;
 }
 
-qbool ReSearchInit (const char *wildcard)
+qbool ReSearchInit(const char* wildcard)
 {
-	const char	*error;
-	int		error_offset;
+	return ReSearchInitEx(wildcard, true);
+}
+
+qbool ReSearchInitEx(const char *wildcard, qbool case_sensitive)
+{
+	const char *error;
+	int error_offset;
 
 	if (wildcard_level == 4) {
 		Com_Printf("Error: Regexp commands nested too deep\n");
 		return false;
 	}
-	wildcard_re[wildcard_level] = pcre_compile(wildcard, 0, &error, &error_offset, NULL);
+	wildcard_re[wildcard_level] = pcre_compile(wildcard, (case_sensitive ? 0 : PCRE_CASELESS), &error, &error_offset, NULL);
 	if (error) {
 		Com_Printf ("Invalid regexp: %s\n", error);
 		return false;
@@ -1338,8 +1399,12 @@ qbool ReSearchMatch (const char *str)
 void ReSearchDone (void)
 {
 	wildcard_level--;
-	if (wildcard_re[wildcard_level]) (pcre_free)(wildcard_re[wildcard_level]);
-	if (wildcard_re_extra[wildcard_level]) (pcre_free)(wildcard_re_extra[wildcard_level]);
+	if (wildcard_re[wildcard_level]) {
+		(pcre_free)(wildcard_re[wildcard_level]);
+	}
+	if (wildcard_re_extra[wildcard_level]) {
+		(pcre_free)(wildcard_re_extra[wildcard_level]);
+	}
 }
 // <-- QW262
 
@@ -1459,6 +1524,10 @@ qbool Util_GetNextWordwrapString(const char *input, char *target, int start_inde
 	return retval;
 }
 
-
-
+void Utils_RegExpFreeSubstring(char* substring)
+{
+	if (substring) {
+		pcre_free_substring(substring);
+	}
+}
 
